@@ -29,22 +29,34 @@
 
 /*
   $Log: orbParameters.h,v $
-  Revision 1.1.2.9  2005/11/16 17:35:25  dgrisby
-  New connectionWatchPeriod and connectionWatchImmediate parameters.
+  Revision 1.1.4.9  2007/07/31 16:38:31  dgrisby
+  New resetTimeOutOnRetries parameter.
 
-  Revision 1.1.2.8  2005/11/14 10:57:18  dgrisby
-  Minor comment fix.
+  Revision 1.1.4.8  2007/02/26 15:16:31  dgrisby
+  New socketSendBuffer parameter, defaulting to 16384 on Windows.
+  Avoids a bug in Windows where select() on send waits for all sent data
+  to be acknowledged.
 
-  Revision 1.1.2.7  2005/08/23 11:45:06  dgrisby
-  New maxSocketSend and maxSocketRecv parameters.
+  Revision 1.1.4.7  2006/01/10 13:59:37  dgrisby
+  New clientConnectTimeOutPeriod configuration parameter.
 
-  Revision 1.1.2.6  2005/02/13 20:53:08  dgrisby
-  Change threadPoolWatchConnection parameter to be an integer rather
-  than a boolean. Value is the count of threads that can be handling a
-  connection when one decides to watch it.
+  Revision 1.1.4.6  2005/11/17 17:03:26  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.2.5  2004/03/02 15:31:22  dgrisby
-  Support for persistent server identifier.
+  Revision 1.1.4.5  2005/09/01 14:52:12  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.4.4  2005/03/02 12:39:23  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.4.3  2005/01/06 23:08:26  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.4.2  2003/11/06 11:56:55  dgrisby
+  Yet more valuetype. Plain valuetype and abstract valuetype are now working.
+
+  Revision 1.1.4.1  2003/03/23 21:03:43  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.4  2002/10/14 20:06:41  dgrisby
   Per objref / per thread timeouts.
@@ -125,6 +137,11 @@ _CORBA_MODULE_VAR _core_attr size_t maxSocketRecv;
 //
 //   Valid values = (n >= 8192)
 
+_CORBA_MODULE_VAR _core_attr int socketSendBuffer;
+//   Sets the socket send buffer size. -1 means leave the system
+//   default unchanged.
+//
+//   Valid values = (n >= -1)
 
 _CORBA_MODULE_VAR _core_attr omniCodeSet::NCS_C* nativeCharCodeSet;
 //  set the native code set for char and string
@@ -151,7 +168,6 @@ _CORBA_MODULE_VAR _core_attr CORBA::Boolean      lcdMode;
 //  Valid values = 0 or 1
 //
 
-
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean supportCurrent;
 //  If the value of this variable is TRUE, per-thread information is
 //  made available through the Current interfaces, e.g.
@@ -159,6 +175,14 @@ _CORBA_MODULE_VAR _core_attr CORBA::Boolean supportCurrent;
 //  can set the value to 0, resulting in a small performance
 //  improvement.
 
+_CORBA_MODULE_VAR _core_attr CORBA::Boolean copyValuesInLocalCalls;
+//  Valuetypes passed as parameters in local calls should be copied.
+//  Since values can be shared by separate parameters, the whole
+//  argument list must be copied in one go. We do this by marshalling
+//  via a temporary memory buffer. If this parameter is set false, the
+//  copy is not performed, so the call is faster but the semantics are
+//  non-standard.
+
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean strictIIOP;
 //   Enable vigorous check on incoming IIOP messages
 //
@@ -331,12 +355,39 @@ _CORBA_MODULE_VAR _core_attr timeValue clientCallTimeOutPeriod;
 //   Valid values = (n >= 0 in milliseconds) 
 //                   0 --> no timeout. Block till a reply comes back
 
+_CORBA_MODULE_VAR _core_attr timeValue clientConnectTimeOutPeriod;
+//   Connect timeout. When a client has no existing connection to
+//   communicate with a server, it must open a new connection before
+//   performing the call. If this parameter is non-zero, it sets a
+//   timeout specifically for establishing the connection. If the
+//   timeout specified here is shorter than the overall timeout for
+//   the call (set with clientCallTimeOutPeriod or per-object or
+//   per-thread timeouts), the connect timeout is used for
+//   establishing the connection, then additional time is permitted
+//   for the call to complete. If the connect timeout is longer than
+//   the normal call timeout, the deadline for the entire call is
+//   extended to match the connect timeout.
+//
+//   If this parameter is zero, the normal call timeout applies to the
+//   total time taken to perform the connect and the subsequent call.
+//
+//   Valid values = (n >= 0 in milliseconds) 
+//                   0 --> same timeout (if any) as other calls
+
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean supportPerThreadTimeOut;
 //   If true, each thread may have a timeout associated with it. This
 //   gives a performance hit due to accessing per-thread data.
 //
 //   Valid values = 0 or 1
 
+_CORBA_MODULE_VAR _core_attr CORBA::Boolean resetTimeOutOnRetries;
+//   If true, the call timeout is reset when an exception handler
+//   causes a call to be retried. If false, the timeout is not reset,
+//   and therefore applies to the call as a whole, rather than to each
+//   individual call attempt.
+//
+//   Valid values = 0 or 1
+
 _CORBA_MODULE_VAR _core_attr CORBA::String_var bootstrapAgentHostname;
 // Applies to the client side. Non-zero enables the use of Sun's bootstrap
 // agent protocol to resolve initial references. The value is the host name
