@@ -29,17 +29,26 @@
 
 /*
   $Log: omniORB.h,v $
-  Revision 1.2.2.23  2005/09/19 14:23:56  dgrisby
-  New traceFile configuration parameter.
+  Revision 1.5.2.7  2007/09/19 14:16:08  dgrisby
+  Avoid namespace clashes if IDL defines modules named CORBA.
 
-  Revision 1.2.2.22  2005/03/14 13:38:26  dgrisby
-  Add omniORB::versionString() and versionHex() functions.
+  Revision 1.5.2.6  2006/01/10 13:59:37  dgrisby
+  New clientConnectTimeOutPeriod configuration parameter.
 
-  Revision 1.2.2.21  2004/07/23 13:25:44  dgrisby
-  New traceExceptions option.
+  Revision 1.5.2.5  2005/09/19 18:26:33  dgrisby
+  Merge from omni4_0_develop again.
 
-  Revision 1.2.2.20  2004/03/02 15:31:22  dgrisby
-  Support for persistent server identifier.
+  Revision 1.5.2.4  2005/04/14 00:04:00  dgrisby
+  New traceInvocationReturns and traceTime options; remove logf function.
+
+  Revision 1.5.2.3  2005/03/30 23:36:15  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.5.2.2  2005/01/06 23:08:10  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.5.2.1  2003/03/23 21:04:13  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.2.2.19  2002/10/14 20:06:03  dgrisby
   Per objref / per thread timeouts.
@@ -261,12 +270,22 @@ _CORBA_MODULE_BEG
   //     variable.  If true, then each local and remote invocation     //
   //     will generate a trace message.                                //
   //                                                                   //
+  _CORBA_MODULE_VAR _core_attr _CORBA_Boolean traceInvocationReturns;  //
+  //                                                                   //
+  //     This value can be changed at runtime either by command-line   //
+  //     option: -ORBtraceInvocations, or by direct assignment to this //
+  //     variable.  If true, then the return of each invocation will   //
+  //     generate a trace message.                                     //
+  //                                                                   //
   _CORBA_MODULE_VAR _core_attr _CORBA_Boolean traceThreadId;           //
   //     This value can be changed at runtime either by command-line   //
   //     option: -ORBtraceThreadId, or by direct assignment to this    //
   //     variable.  If true, then the logging messages emmited due to  //
   //     the two options above will contain the thread id of the       //
   //     logging thread.                                               //
+  //                                                                   //
+  _CORBA_MODULE_VAR _core_attr _CORBA_Boolean traceTime;               //
+  //     If true, logging messages will include the current time.      //
   ///////////////////////////////////////////////////////////////////////
 
   ////////////////////////////////////////////////////////////////////////
@@ -302,6 +321,7 @@ _CORBA_MODULE_BEG
   // setClientCallTimeout()                                             //
   // setClientThreadCallTimeout()                                       //
   // setClientThreadCallDeadline()                                      //
+  // setClientConnectTimeout()                                          //
   //                                                                    //
   // Functions to set call timeouts at runtime.                         //
   //                                                                    //
@@ -313,11 +333,14 @@ _CORBA_MODULE_BEG
   // thread to the specified number of milliseconds. If the calling     //
   // thread is not an omni_thread, it throws CORBA::INITIALIZE.         //
   //                                                                    //
-  // setClientThreadCall Deadline() sets an absolute deadline for all   //
+  // setClientThreadCallDeadline() sets an absolute deadline for all    //
   // calls in the calling thread. The seconds and nanoseconds should be //
   // acquired from omni_thread::get_time(). Again throws INITIALIZE if  //
   // the calling thread is not an omni_thread.                          //
   //                                                                    //
+  // setClientConnectTimeout() sets the overriding timeout for calls    //
+  // that involve opening a new network connection.                     //
+  //                                                                    //
   _CORBA_MODULE_FN void setClientCallTimeout(CORBA::ULong millisecs);   //
   _CORBA_MODULE_FN void setClientCallTimeout(CORBA::Object_ptr obj,     //
     					     CORBA::ULong millisecs);   //
@@ -326,6 +349,7 @@ _CORBA_MODULE_BEG
                                                                         //
   _CORBA_MODULE_FN void setClientThreadCallDeadline(unsigned long secs, //
                                                     unsigned long ns);  //
+  _CORBA_MODULE_FN void setClientConnectTimeout(CORBA::ULong millisecs);//
                                                                         //
   ////////////////////////////////////////////////////////////////////////
 
@@ -455,9 +479,6 @@ _CORBA_MODULE_BEG
   };
 
 
-  _CORBA_MODULE_FN void logf(const char* fmt ...);
-  // Writes log message with prefix, and appends '\n'.
-
   _CORBA_MODULE_FN void do_logs(const char* msg);
   // internal
 
@@ -710,7 +731,7 @@ OMNI_NAMESPACE_END(omni)
 
 
 #define OMNIORB_THROW(name, minor, completion) \
-  throw CORBA::name(minor, completion)
+  throw ::CORBA::name(minor, completion)
 
 #endif
 
