@@ -28,14 +28,40 @@
 
 /*
   $Log: poa.h,v $
-  Revision 1.2.2.15  2005/06/17 10:57:26  dgrisby
-  Explicitly initialise omniServant. Thanks Thomas Richter.
+  Revision 1.4.2.11  2007/02/26 12:41:00  dgrisby
+  Fix duplicated POA enum Any operators. Thanks Thomas Richter.
 
-  Revision 1.2.2.14  2004/02/11 18:02:26  dgrisby
-  Call base constructor for pedantic compilers.
+  Revision 1.4.2.10  2006/10/23 15:08:31  dgrisby
+  Suppress GCC warnings about missing base class constructor calls.
+  Thanks Tamas Kerecsen. Somehow, this patch works on VC++ 6, where
+  before a similar change failed.
 
-  Revision 1.2.2.13  2003/07/26 22:52:22  dgrisby
-  Avoid spurious gcc warnings when sizeof pointer > sizeof int.
+  Revision 1.4.2.9  2006/03/22 15:55:14  dgrisby
+  VC++ 6 doesn't like explicit calls to default base constructors.
+
+  Revision 1.4.2.8  2005/11/09 12:22:18  dgrisby
+  Local interfaces support.
+
+  Revision 1.4.2.7  2005/08/02 09:41:13  dgrisby
+  Bug in Servant_var if assigning to itself.
+
+  Revision 1.4.2.6  2005/07/22 17:18:39  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.4.2.5  2005/01/06 23:08:11  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.4.2.4  2004/10/13 17:58:18  dgrisby
+  Abstract interfaces support; values support interfaces; value bug fixes.
+
+  Revision 1.4.2.3  2004/07/23 10:29:57  dgrisby
+  Completely new, much simpler Any implementation.
+
+  Revision 1.4.2.2  2004/02/16 10:10:28  dgrisby
+  More valuetype, including value boxes. C++ mapping updates.
+
+  Revision 1.4.2.1  2003/03/23 21:04:06  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.2.2.12  2001/11/13 14:11:44  dpg1
   Tweaks for CORBA 2.5 compliance.
@@ -112,6 +138,8 @@
 #ifndef __OMNIOBJREF_H__
 #include <omniORB4/omniObjRef.h>
 #endif
+#include <omniORB4/minorCode.h>
+
 
 OMNI_NAMESPACE_BEGIN(omni)
 
@@ -347,56 +375,8 @@ _CORBA_MODULE_BEG
     REQUEST_PROCESSING_POLICY_ID _init_in_decl_( = 22 );
 
 
-  enum ThreadPolicyValue {
-    ORB_CTRL_MODEL,  // default
-    SINGLE_THREAD_MODEL,
-    MAIN_THREAD_MODEL
-  };
+#include <omniORB4/poa_enums_defs.hh>
 
-_CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
-
-  enum LifespanPolicyValue {
-    TRANSIENT,  // default
-    PERSISTENT
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_LifespanPolicyValue;
-
-  enum IdUniquenessPolicyValue {
-    UNIQUE_ID,  // default
-    MULTIPLE_ID
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_IdUniquenessPolicyValue;
-
-  enum IdAssignmentPolicyValue {
-    USER_ID,
-    SYSTEM_ID  // default
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_IdAssignmentPolicyValue;
-
-  enum ImplicitActivationPolicyValue {
-    IMPLICIT_ACTIVATION,
-    NO_IMPLICIT_ACTIVATION  // default (but not root poa)
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ImplicitActivationPolicyValue;
-
-  enum ServantRetentionPolicyValue {
-    RETAIN,  // default
-    NON_RETAIN
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ServantRetentionPolicyValue;
-
-  enum RequestProcessingPolicyValue {
-    USE_ACTIVE_OBJECT_MAP_ONLY,  // default
-    USE_DEFAULT_SERVANT,
-    USE_SERVANT_MANAGER
-  };
-
-  _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_RequestProcessingPolicyValue;
 
   OMNIORB_DECLARE_POLICY_OBJECT(ThreadPolicy, THREAD_POLICY_ID)
   OMNIORB_DECLARE_POLICY_OBJECT(LifespanPolicy, LIFESPAN_POLICY_ID)
@@ -460,8 +440,8 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
   ///////////////////////////////// POA ////////////////////////////////
   //////////////////////////////////////////////////////////////////////
 
-  class _objref_AdapterActivator;
-  class _objref_ServantManager;
+  class AdapterActivator;
+  class ServantManager;
 
 
   class POA : public CORBA::Object {
@@ -516,6 +496,8 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
       virtual const char* _NP_repoId(int*) const;
       virtual void _NP_marshal(cdrStream&) const;
     };
+    static _dyn_attr const CORBA::TypeCode_ptr _tc_InvalidPolicy;
+
 
     // POA creation and destruction
 
@@ -552,13 +534,13 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
     virtual POA_ptr the_parent() = 0;
     virtual POAList* the_children() = 0;
     virtual POAManager_ptr the_POAManager() = 0;
-    virtual _objref_AdapterActivator* the_activator() = 0;
-    virtual void the_activator(_objref_AdapterActivator* aa) = 0;
+    virtual AdapterActivator* the_activator() = 0;
+    virtual void the_activator(AdapterActivator* aa) = 0;
 
     // Servant Manager registration
 
-    virtual _objref_ServantManager* get_servant_manager() = 0;
-    virtual void set_servant_manager(_objref_ServantManager* imgr) = 0;
+    virtual ServantManager* get_servant_manager() = 0;
+    virtual void set_servant_manager(ServantManager* imgr) = 0;
 
     // Operations for the USE_DEFAULT_SERVANT policy
 
@@ -576,7 +558,7 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
 
     virtual CORBA::Object_ptr create_reference(const char* intf) = 0;
     virtual CORBA::Object_ptr create_reference_with_id(const ObjectId& oid,
-					       const char* intf) = 0;
+						       const char* intf) = 0;
 
     // Identity mapping operations
 
@@ -686,9 +668,13 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
     virtual void _add_ref();
     virtual void _remove_ref();
 
+    virtual CORBA::ULong _refcount_value();
+
   protected:
-    inline ServantBase() {}
-    inline ServantBase(const ServantBase& os) : omniServant(os) {}
+    inline ServantBase() : _pd_refCount(1) {}
+    inline ServantBase(const ServantBase& _v) :
+      omniServant (_v), _pd_refCount (1) {}
+
     inline ServantBase& operator = (const ServantBase&) { return *this; }
 
     void* _do_this(const char* repoId);
@@ -697,71 +683,94 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
     virtual omniObjRef* _do_get_interface();
     virtual void* _downcast();
     // Overrides omniServant.
+
+    int _pd_refCount;
   };
 
   //////////////////////////////////////////////////////////////////////
   ///////////////////////// RefCountServantBase ////////////////////////
   //////////////////////////////////////////////////////////////////////
 
-  class RefCountServantBase : public virtual ServantBase {
+  struct RefCountServantBase {};
+  // Deprecated.
+
+
+  //////////////////////////////////////////////////////////////////////
+  ///////////////////////// ValueRefCountBase //////////////////////////
+  //////////////////////////////////////////////////////////////////////
+
+  class ValueRefCountBase
+    : public virtual CORBA::ValueBase,
+      public virtual ServantBase
+  {
   public:
-    virtual ~RefCountServantBase();
+    ValueRefCountBase() {}
 
     virtual void _add_ref();
     virtual void _remove_ref();
+    virtual CORBA::ULong _refcount_value();
 
   protected:
-    inline RefCountServantBase() : pd_refCount(1) {}
-    inline RefCountServantBase(const RefCountServantBase& os)
-      : omniServant(os), ServantBase(os), pd_refCount(1) {}
-    inline RefCountServantBase& operator = (const RefCountServantBase&)
-      { return *this; }
+    virtual ~ValueRefCountBase();
+    ValueRefCountBase(const ValueRefCountBase&);
 
   private:
-    int pd_refCount;
+    ValueRefCountBase& operator= (const ValueRefCountBase&);
+    // Not implemented
   };
 
+
+  //////////////////////////////////////////////////////////////////////
+  /////////////////////////// Servant_var //////////////////////////////
+  //////////////////////////////////////////////////////////////////////
+
+  template <typename T>
+  class Servant_var {
+  public:
+    inline Servant_var() : pd_data(0) {}
+    inline Servant_var(T* p) : pd_data(p) {}
+    inline Servant_var(const Servant_var& v) : pd_data(v.pd_data)
+      { if (pd_data) pd_data->_add_ref(); }
+
+    inline ~Servant_var() { if (pd_data) pd_data->_remove_ref(); }
+
+    inline Servant_var& operator= (T* p) {
+      if (pd_data != p) {
+	if (pd_data) pd_data->_remove_ref();
+	pd_data = p;
+      }
+      return *this;
+    }
+    inline Servant_var& operator= (const Servant_var& v) {
+      if (v.pd_data != pd_data) {
+	if (pd_data) pd_data->_remove_ref();
+	if ((pd_data = v.pd_data)) pd_data->_add_ref();
+      }
+      return *this;
+    }
+
+    inline operator T* () const    { return pd_data; }
+    inline T* operator -> () const { return pd_data; }
+
+    inline T* in() const { return pd_data; }
+    inline T*& inout() { return pd_data; }
+    inline T*& out() {
+      if( pd_data ) { pd_data->_remove_ref();  pd_data = 0; }
+      return pd_data;
+    }
+    inline T* _retn() { T* tmp = pd_data;  pd_data = 0;  return tmp; }
+
+  private:
+    T* pd_data;
+  };
+
+
   //////////////////////////////////////////////////////////////////////
   /////////////////////////// ServantBase_var //////////////////////////
   //////////////////////////////////////////////////////////////////////
 
-  class ServantBase_var {
-  public:
-    typedef ServantBase*    T_ptr;
-    typedef ServantBase_var T_var;
+  typedef Servant_var<ServantBase> ServantBase_var;
 
-    inline ServantBase_var() : pd_data(0) {}
-    inline ServantBase_var(T_ptr p) : pd_data(p) {}
-    inline ServantBase_var(const T_var& v) : pd_data(v.pd_data)
-      { if( pd_data )  pd_data->_add_ref(); }
-
-    inline ~ServantBase_var() { if( pd_data )  pd_data->_remove_ref(); }
-
-    inline T_var& operator = (T_ptr p) {
-      if( pd_data )  pd_data->_remove_ref();
-      pd_data = p;
-      return *this;
-    }
-    inline T_var& operator = (T_var v) {
-      if( pd_data )  pd_data->_remove_ref();
-      if( (pd_data = v.pd_data) )  pd_data->_add_ref();
-      return *this;
-    }
-
-    inline operator T_ptr () const    { return pd_data; }
-    inline T_ptr operator -> () const { return pd_data; }
-
-    inline T_ptr in() const { return pd_data; }
-    inline T_ptr& inout() { return pd_data; }
-    inline T_ptr& out() {
-      if( pd_data ) { pd_data->_remove_ref();  pd_data = 0; }
-      return pd_data;
-    }
-    inline T_ptr _retn() { T_ptr tmp = pd_data;  pd_data = 0;  return tmp; }
-
-  private:
-    T_ptr pd_data;
-  };
 
   //////////////////////////////////////////////////////////////////////
   //////////////////////// DynamicImplementation ///////////////////////
@@ -810,7 +819,8 @@ _CORBA_MODULE_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_ThreadPolicyValue;
 _CORBA_MODULE_END  // PortableServer
 
 
-#include <omniORB4/poa_operators.h>
+#include <omniORB4/poa_enums_operators.hh>
+
 #include <omniORB4/poa_poa.h>
 // This brings in the skeletons for:
 //  AdapterActivator
