@@ -29,17 +29,17 @@
 
 /*
   $Log: sslContext.h,v $
-  Revision 1.1.2.9  2005/12/30 17:41:40  dgrisby
-  Missing #undef _core_attr. Thanks Vsevolod Fedorov.
+  Revision 1.1.4.4  2006/01/10 12:24:04  dgrisby
+  Merge from omni4_0_develop pre 4.0.7 release.
 
-  Revision 1.1.2.8  2005/09/05 14:32:04  dgrisby
-  SSL transport extensions from Jon Biggar; SSL command line options.
+  Revision 1.1.4.3  2005/09/05 17:12:20  dgrisby
+  Merge again. Mainly SSL transport changes.
 
-  Revision 1.1.2.7  2004/02/11 15:44:53  dgrisby
-  Hook to set SSL verify mode. Thanks Matthew Wood.
+  Revision 1.1.4.2  2005/01/06 23:08:22  dgrisby
+  Big merge from omni4_0_develop.
 
-  Revision 1.1.2.6  2003/04/25 15:53:33  dgrisby
-  Work around OpenSSL crypt() clash.
+  Revision 1.1.4.1  2003/03/23 21:04:02  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.5  2002/09/05 14:29:01  dgrisby
   Link force mechanism wasn't working with gcc.
