@@ -30,6 +30,9 @@
 #define __TEMPLATEDECLS_H__
 
 
+// Empty class to prevent illegal objref _var widening
+class _CORBA_ObjRef_Var_base {};
+
 // Some compilers need an operator[] with an int argument when there
 // is an operator T* defined.
 
@@ -50,7 +53,7 @@ template <class T, class T_var> class _CORBA_PseudoObj_Member;
 template <class T, class T_var> class _CORBA_PseudoObj_Out;
 
 template <class T>
-class _CORBA_PseudoObj_Var {
+class _CORBA_PseudoObj_Var : public _CORBA_ObjRef_Var_base {
 public:
   typedef T* T_ptr;
   typedef _CORBA_PseudoObj_Var<T> T_var;
@@ -80,6 +83,11 @@ public:
 
 protected:
   T_ptr pd_data;
+
+private:
+  // Not implemented
+  _CORBA_PseudoObj_Var(const _CORBA_ObjRef_Var_base&);
+  T_var& operator= (const _CORBA_ObjRef_Var_base&);
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -105,6 +113,11 @@ public:
   inline operator T_ptr () const  { return _ptr; }
 
   T_ptr _ptr;
+
+private:
+  // Not implemented
+  _CORBA_PseudoObj_Member(const _CORBA_ObjRef_Var_base&);
+  T_member& operator= (const _CORBA_ObjRef_Var_base&);
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -380,17 +393,14 @@ class _CORBA_ObjRef_INOUT_arg;
 template <class T,class T_Helper>
 class _CORBA_ObjRef_OUT_arg;
 
-template <class T,class T_Helper>
-class _CORBA_ObjRef_tcDesc_arg;
-
 template <class T, class T_Helper>
-class _CORBA_ObjRef_Var {
+class _CORBA_ObjRef_Var : public _CORBA_ObjRef_Var_base {
 public:
   typedef T* ptr_t;
   typedef T* T_ptr;
-  typedef _CORBA_ObjRef_Member<T,T_Helper> T_member;
+  typedef _CORBA_ObjRef_Member<T,T_Helper>  T_member;
   typedef _CORBA_ObjRef_Element<T,T_Helper> T_element;
-  typedef _CORBA_ObjRef_Var<T,T_Helper>    T_var;
+  typedef _CORBA_ObjRef_Var<T,T_Helper>     T_var;
 
   inline _CORBA_ObjRef_Var() : pd_objref(T_Helper::_nil()) {}
   inline _CORBA_ObjRef_Var(T_ptr p) : pd_objref(p) {}
@@ -444,10 +454,13 @@ public:
 
   friend class _CORBA_ObjRef_INOUT_arg<T,T_Helper>;
   friend class _CORBA_ObjRef_OUT_arg<T,T_Helper>;
-  friend class _CORBA_ObjRef_tcDesc_arg<T,T_Helper>;
 
 private:
   T_ptr pd_objref;
+
+  // Not implemented
+  _CORBA_ObjRef_Var(const _CORBA_ObjRef_Var_base&);
+  T_var& operator= (const _CORBA_ObjRef_Var_base&);
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -458,9 +471,9 @@ template <class T,class T_Helper>
 class _CORBA_ObjRef_Member {
 public:
   typedef T* T_ptr;
-  typedef _CORBA_ObjRef_Member<T,T_Helper> T_member;
+  typedef _CORBA_ObjRef_Member<T,T_Helper>  T_member;
   typedef _CORBA_ObjRef_Element<T,T_Helper> T_element;
-  typedef _CORBA_ObjRef_Var<T,T_Helper>    T_var;
+  typedef _CORBA_ObjRef_Var<T,T_Helper>     T_var;
 
   inline _CORBA_ObjRef_Member() { 
     _ptr = T_Helper::_nil();
@@ -535,6 +548,11 @@ inline ~_CORBA_ObjRef_Member() {
   }
 
   T_ptr          _ptr;
+
+private:
+  // Not implemented
+  _CORBA_ObjRef_Member(const _CORBA_ObjRef_Var_base&);
+  T_member& operator= (const _CORBA_ObjRef_Var_base&);
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -624,6 +642,11 @@ public:
 
   _CORBA_Boolean pd_rel;
   T_ptr&         pd_data;
+
+private:
+  // Not implemented
+  _CORBA_ObjRef_Element(const _CORBA_ObjRef_Var_base&);
+  T_element& operator= (const _CORBA_ObjRef_Var_base&);
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -757,38 +780,6 @@ private:
 
 };
 
-//////////////////////////////////////////////////////////////////////
-/////////////////////// _CORBA_ObjRef_tcDesc_arg /////////////////////
-//////////////////////////////////////////////////////////////////////
-
-template <class T,class T_Helper>
-class _CORBA_ObjRef_tcDesc_arg {
-public:
-  // The sole purpose of this class is for passing as an argument
-  // to the tc build descriptor function of an interface.
-  // It provides the necessary conversions from T_var, T_member,
-  // T_element so that only one tc build description function is needed.
-  // The alternative is to have 3 overloaded build description functions
-  // which makes the stub code even more bloated.
-  typedef T* T_ptr;
-  typedef _CORBA_ObjRef_Member<T,T_Helper> T_member;
-  typedef _CORBA_ObjRef_Element<T,T_Helper> T_element;
-  typedef _CORBA_ObjRef_Var<T,T_Helper>    T_var;
-  inline _CORBA_ObjRef_tcDesc_arg(T_ptr& p, _CORBA_Boolean rel) 
-    : _data(p), _rel(rel) {}
-  inline _CORBA_ObjRef_tcDesc_arg(T_var& p) : _data(p.pd_objref), _rel(1) {}
-  inline _CORBA_ObjRef_tcDesc_arg(T_member& p) : _data(p._ptr), _rel(1) {}
-  inline _CORBA_ObjRef_tcDesc_arg(T_element p) 
-    : _data(p._NP_ref()), _rel(p._NP_release()) {}
-  inline ~_CORBA_ObjRef_tcDesc_arg() {}
-
-  T*& _data;
-  _CORBA_Boolean _rel;
-
-private:
-  _CORBA_ObjRef_tcDesc_arg();
-};
-
 
 //////////////////////////////////////////////////////////////////////
 ////////////////////// _CORBA_ConstrType_Fix_Var /////////////////////
