@@ -26,8 +26,39 @@
 #   Produce local callback functions
 #
 
-# $Id: call.py,v 1.1.4.13 2003/01/14 11:48:16 dgrisby Exp $
+# $Id: call.py,v 1.1.6.10 2007/09/19 14:16:08 dgrisby Exp $
 # $Log: call.py,v $
+# Revision 1.1.6.10  2007/09/19 14:16:08  dgrisby
+# Avoid namespace clashes if IDL defines modules named CORBA.
+#
+# Revision 1.1.6.9  2007/02/26 15:51:15  dgrisby
+# Suppress cd parameter when it is definitely unused, to avoid compiler
+# warnings.
+#
+# Revision 1.1.6.8  2005/11/09 12:22:18  dgrisby
+# Local interfaces support.
+#
+# Revision 1.1.6.7  2005/08/16 13:51:21  dgrisby
+# Problems with valuetype / abstract interface C++ mapping.
+#
+# Revision 1.1.6.6  2005/03/30 23:36:11  dgrisby
+# Another merge from omni4_0_develop.
+#
+# Revision 1.1.6.5  2005/01/06 23:09:49  dgrisby
+# Big merge from omni4_0_develop.
+#
+# Revision 1.1.6.4  2004/10/13 17:58:21  dgrisby
+# Abstract interfaces support; values support interfaces; value bug fixes.
+#
+# Revision 1.1.6.3  2003/11/06 11:56:56  dgrisby
+# Yet more valuetype. Plain valuetype and abstract valuetype are now working.
+#
+# Revision 1.1.6.2  2003/10/23 11:25:54  dgrisby
+# More valuetype support.
+#
+# Revision 1.1.6.1  2003/03/23 21:02:42  dgrisby
+# Start of omniORB 4.1.x development branch.
+#
 # Revision 1.1.4.13  2003/01/14 11:48:16  dgrisby
 # Remove warnings from gcc -Wshadow. Thanks Pablo Mejia.
 #
@@ -299,10 +330,19 @@ class CallDescriptor:
         if self.__contexts:
             impl_args.append("ctxt")
 
+        # If we have no return value, no arguments, no exceptions, and
+        # no contexts, we don't use the call descriptor argument at
+        # all, so we do not give it a name, so as to avoid warnings on
+        # some compilers.
+        if result_string or impl_args or self.__exceptions or self.__contexts:
+            cd_arg = " cd"
+        else:
+            cd_arg = ""
+
         # If we have no return value and no arguments at all then we don't
         # need to fetch the call descriptor. This suppresses a warning in gcc
         # about an unused variable.
-        if result_string != "" or impl_args != []:
+        if result_string or impl_args:
             get_cd = self.__name + "* tcd = (" + self.__name + "*)cd;"
         else:
             get_cd = ""
@@ -337,6 +377,7 @@ class CallDescriptor:
 
         stream.out(template.interface_callback,
                    local_call_descriptor = function_name,
+                   cd_arg = cd_arg,
                    get_call_descriptor = get_cd,
                    impl_fqname =interface_name.prefix("_impl_").fullyQualify(),
                    name = interface_name.fullyQualify(),
@@ -378,8 +419,12 @@ class CallDescriptor:
                 rvalue = args[n]
                 if argtype.array():
                     rvalue = "&" + rvalue + "[0]"
+                if argtype.value() or argtype.valuebox():
+                    star = "*"
+                else:
+                    star = ""
                 if h_is_ptr:
-                    rvalue = "&(" + argtype.base(environment) + "&) " \
+                    rvalue = "&(" + argtype.base(environment) + star + "&) " \
                              + rvalue
                 assign_args.append(arg_n + " = " + rvalue + ";")
 
@@ -394,7 +439,7 @@ class CallDescriptor:
 
         if self.__contexts:
             assign_context = "_call_desc.ctxt = "\
-                             "CORBA::Context::_duplicate(_ctxt);"
+                             "::CORBA::Context::_duplicate(_ctxt);"
         else:
             assign_context = ""
         
@@ -465,7 +510,10 @@ class CallDescriptor:
                               "void marshalReturnedValues(cdrStream&);\n"
         data_members = []
         n = -1
+        containsValues = 0
         for argument in self.__arguments:
+            containsValues = (containsValues or
+                              idltype.containsValueType(argument.paramType()))
             n = n + 1
             holder_n = "arg_" + str(n)
             argtype = types.Type(argument.paramType())
@@ -489,9 +537,15 @@ class CallDescriptor:
                     holder = holder + "_slice*"
                 if h_is_ptr:
                     holder = holder + "*"
+                if argtype.value() or argtype.valuebox():
+                    holder = holder + "*"
+
                 data_members.append(holder + " " + holder_n + ";")
 
         if self.__has_return_value:
+            containsValues = (containsValues or
+                              idltype.containsValueType(self.__returntype))
+
             returntype = types.Type(self.__returntype)
             ((h_is_const,h_is_ptr),\
              (s_is_holder,s_is_var)) = _arg_info(returntype,3)
@@ -507,7 +561,12 @@ class CallDescriptor:
             data_members.append(storage + " " + storage_n + ";")
 
         if self.__contexts:
-            data_members.append("CORBA::Context_var ctxt;");
+            data_members.append("::CORBA::Context_var ctxt;");
+
+        if containsValues:
+            contains_values = "containsValues(1);"
+        else:
+            contains_values = ""
         
         # Write the proxy class definition
         stream.out(template.interface_proxy_class,
@@ -515,6 +574,7 @@ class CallDescriptor:
                    call_descriptor = self.__name,
                    ctor_args = string.join(ctor_args,","),
                    base_ctor = base_ctor,
+                   contains_values = contains_values,
                    in_arguments_decl = in_arguments_decl,
                    out_arguments_decl = out_arguments_decl,
                    user_exceptions_decl = user_exceptions_decl,
@@ -599,12 +659,15 @@ class CallDescriptor:
             if s_is_var:
                 alloc = ""
                 d_type = argtype.deref(1)
+
                 if argtype.array():
                     alloc = argtype.base() + "_alloc()"
-                elif not (d_type.typecode() or \
-                          d_type.string() or \
-                          d_type.wstring() or \
-                          d_type.objref()):
+                elif not (d_type.typecode()  or
+                          d_type.string()    or
+                          d_type.wstring()   or
+                          d_type.interface() or
+                          d_type.value()     or
+                          d_type.valuebox()):
                     alloc = "new " + argtype.base()
                 if alloc != "":
                     marshal_block.out(storage_n + " = " + alloc + ";")
@@ -651,10 +714,12 @@ class CallDescriptor:
                 d_type = argtype.deref(1)
                 if argtype.array():
                     alloc = argtype.base() + "_alloc()"
-                elif not (d_type.typecode() or \
-                          d_type.string() or \
-                          d_type.wstring() or \
-                          d_type.objref()):
+                elif not (d_type.typecode()  or
+                          d_type.string()    or
+                          d_type.wstring()   or
+                          d_type.interface() or
+                          d_type.value()     or
+                          d_type.valuebox()):
                     alloc = "new " + argtype.base()
                 if alloc != "":
                     marshal_block.out(argname + " = " + alloc + ";")
@@ -678,10 +743,12 @@ class CallDescriptor:
                     alloc = ""
                     if argtype.array():
                         alloc = argtype.base() + "_alloc()"
-                    elif not (d_type.typecode() or \
-                              d_type.string() or \
-                              d_type.wstring() or \
-                              d_type.objref()):
+                    elif not (d_type.typecode()  or
+                              d_type.string()    or
+                              d_type.wstring()   or
+                              d_type.interface() or
+                              d_type.value()     or
+                              d_type.valuebox()):
                         alloc = "new " + argtype.base()
                     if alloc != "":
                         marshal_block.out(arg_n + " = " + alloc + ";")
@@ -689,8 +756,8 @@ class CallDescriptor:
                 if d_type.typecode():
                     marshal_block.out(arg_n + "_ = *" + arg_n + ";\n"+ \
                                       "*" + arg_n + " = " + \
-                                      "CORBA::TypeCode::_nil();")
-                elif d_type.objref():
+                                      "::CORBA::TypeCode::_nil();")
+                elif d_type.interface():
                     nilobjref = string.replace(d_type.base(),"_ptr","::_nil()")
                     if isinstance(d_type.type().decl(),idlast.Forward):
                         nilobjref = string.replace(nilobjref,\
@@ -703,11 +770,10 @@ class CallDescriptor:
                     marshal_block.out(arg_n + "_ = *" + arg_n + ";\n"+ \
                                       "*" + arg_n + " = " + \
                                       "(char*) _CORBA_String_helper::empty_string;")
-                elif d_type.string():
-                    # XXX the empty string constant may be wrong.
+                elif d_type.wstring():
                     marshal_block.out(arg_n + "_ = *" + arg_n + ";\n" + \
                                       "*" + arg_n + " = " + \
-                                      "(CORBA::WChar*) _CORBA_WString_helper::empty_string;")
+                                      "(::CORBA::WChar*) _CORBA_WString_helper::empty_wstring;")
                 arg_n = "*" + arg_n
             skutil.unmarshall(marshal_block, None,
                               argtype, None, arg_n, "_n")
@@ -818,6 +884,12 @@ _arg_mapping = {
     ( ((1,1),(0,1)), ((0,0),(1,1)), ((0,1),(0,1)), ((0,0),(1,1)) ),
     idltype.tk_value:
     ( ((0,0),(0,1)), ((0,0),(1,1)), ((0,1),(0,1)), ((0,0),(1,1)) ),
+    idltype.tk_value_box:
+    ( ((0,0),(0,1)), ((0,0),(1,1)), ((0,1),(0,1)), ((0,0),(1,1)) ),
+    idltype.tk_abstract_interface:
+    ( ((0,0),(0,1)), ((0,0),(1,1)), ((0,1),(0,1)), ((0,0),(1,1)) ),
+    idltype.tk_local_interface:
+    ( ((0,0),(0,1)), ((0,0),(1,1)), ((0,1),(0,1)), ((0,0),(1,1)) ),
     }
 
 _fixed = 0
