@@ -26,11 +26,38 @@
 #   
 #   Produces the instances of CORBA::TypeCode
 
-# $Id: typecode.py,v 1.16.2.9 2004/03/24 22:08:41 dgrisby Exp $
+# $Id: typecode.py,v 1.19.2.10 2006/11/28 00:07:57 dgrisby Exp $
 # $Log: typecode.py,v $
-# Revision 1.16.2.9  2004/03/24 22:08:41  dgrisby
-# TypeCodes for sequences of forward declared structs / unions used by
-# other compilation units were broken.
+# Revision 1.19.2.10  2006/11/28 00:07:57  dgrisby
+# TypeCodes for nested structures / unions were not exported.
+#
+# Revision 1.19.2.9  2005/11/09 12:22:17  dgrisby
+# Local interfaces support.
+#
+# Revision 1.19.2.8  2005/08/16 13:51:21  dgrisby
+# Problems with valuetype / abstract interface C++ mapping.
+#
+# Revision 1.19.2.7  2004/10/13 17:58:23  dgrisby
+# Abstract interfaces support; values support interfaces; value bug fixes.
+#
+# Revision 1.19.2.6  2004/07/31 23:45:44  dgrisby
+# ifdef for forward declared union typecodes.
+#
+# Revision 1.19.2.5  2004/07/04 23:53:38  dgrisby
+# More ValueType TypeCode and Any support.
+#
+# Revision 1.19.2.4  2004/04/02 13:26:22  dgrisby
+# Start refactoring TypeCode to support value TypeCodes, start of
+# abstract interfaces support.
+#
+# Revision 1.19.2.3  2004/02/16 10:10:31  dgrisby
+# More valuetype, including value boxes. C++ mapping updates.
+#
+# Revision 1.19.2.2  2003/10/23 11:25:55  dgrisby
+# More valuetype support.
+#
+# Revision 1.19.2.1  2003/03/23 21:02:39  dgrisby
+# Start of omniORB 4.1.x development branch.
 #
 # Revision 1.16.2.8  2001/11/14 17:13:43  dpg1
 # Long double support.
@@ -306,7 +333,8 @@ const CORBA::TypeCode_ptr @tc_name@ = @mangled_name@;
 def mkTypeCode(type, declarator = None, node = None):
     assert isinstance(type, types.Type)
 
-    prefix = "CORBA::TypeCode::PR_"
+    prefix   = "CORBA::TypeCode::PR_"
+    tctrack  = ", &" + config.state['Private Prefix'] + "_tcTrack"
 
     if declarator:
         assert isinstance(declarator, idlast.Declarator)
@@ -315,7 +343,7 @@ def mkTypeCode(type, declarator = None, node = None):
         post_str = ""
         for dim in dims:
             pre_str = pre_str + prefix + "array_tc(" + str(dim) + ", "
-            post_str = post_str + ")"
+            post_str = post_str + tctrack + ")"
 
         return pre_str + mkTypeCode(type, None, node) + post_str
 
@@ -348,46 +376,31 @@ def mkTypeCode(type, declarator = None, node = None):
               repr(type.kind())
 
     if isinstance(type, idltype.String):
-        return prefix + "string_tc(" + str(type.bound()) + ")"
+        return prefix + "string_tc(" + str(type.bound()) + tctrack + ")"
 
     if isinstance(type, idltype.WString):
-        return prefix + "wstring_tc(" + str(type.bound()) + ")"
+        return prefix + "wstring_tc(" + str(type.bound()) + tctrack + ")"
 
     if isinstance(type, idltype.Sequence):
         seqType = type.seqType()
         if isinstance(seqType, idltype.Declared):
             decl = seqType.decl()
-            if isinstance(decl, idlast.StructForward) or \
-               isinstance(decl, idlast.UnionForward) :
-
-                forwardUsed(decl)
-                scopedName = decl.scopedName()
-                mangled_name = mangleName(config.state['Private Prefix'] +\
-                                          "_fs_", scopedName)
-                if not alreadyDefined(mangled_name):
-                    defineName(mangled_name)
-                    tophalf.out("""\
-static CORBA::TypeCode_ptr @name@ = @prefix@forward_sequence_tc(@bound@);
-""",
-                                name=mangled_name, prefix=prefix,
-                                bound=type.bound())
-                return mangled_name
-
-            elif hasattr(decl, "recursive") and decl.recursive() and \
+            if hasattr(decl, "recursive") and decl.recursive() and \
                  currently_being_defined(decl):
 
                 depth = recursive_Depth(decl)
                 return prefix + "recursive_sequence_tc(" +\
-                       str(type.bound()) + ", " + str(depth) + ")"
+                       str(type.bound()) + ", " + str(depth) + tctrack + ")"
             
         startingNode(type)
         ret = prefix + "sequence_tc(" + str(type.bound()) + ", " +\
-              mkTypeCode(types.Type(type.seqType())) + ")"
+              mkTypeCode(types.Type(type.seqType())) + tctrack + ")"
         finishingNode()
         return ret
 
     if isinstance(type, idltype.Fixed):
-        return prefix + "fixed_tc(%d,%d)" % (type.digits(),type.scale())
+        return (prefix + "fixed_tc(%d,%d%s)" %
+                (type.digits(),type.scale(),tctrack))
 
     assert isinstance(type, idltype.Declared)
 
@@ -399,7 +412,24 @@ static CORBA::TypeCode_ptr @name@ = @prefix@forward_sequence_tc(@bound@);
         
         repoID = type.decl().repoId()
         iname = scopedName.simple()
-        return prefix + 'interface_tc("' + repoID + '", "' + iname + '")'
+        return (prefix + 'interface_tc("' + repoID + '", "' +
+                iname + '"' + tctrack + ')')
+
+    elif type.kind() == idltype.tk_abstract_interface:
+        scopedName = id.Name(type.decl().scopedName())
+        
+        repoID = type.decl().repoId()
+        iname = scopedName.simple()
+        return (prefix + 'abstract_interface_tc("' + repoID + '", "' +
+                iname + '"' + tctrack + ')')
+
+    elif type.kind() == idltype.tk_local_interface:
+        scopedName = id.Name(type.decl().scopedName())
+        
+        repoID = type.decl().repoId()
+        iname = scopedName.simple()
+        return (prefix + 'local_interface_tc("' + repoID + '", "' +
+                iname + '"' + tctrack + ')')
 
     guard_name = id.Name(type.scopedName()).guard()
 
@@ -494,10 +524,15 @@ def visitStruct(node):
     # create the static typecodes for constructed types by setting
     # the resolving_dependency flag and recursing
     save_resolving_dependency = self.__resolving_dependency
-    self.__resolving_dependency = 1
     
     for child in node.members():
         memberType = child.memberType()
+
+        if child.constrType():
+            self.__resolving_dependency = save_resolving_dependency
+        else:
+            self.__resolving_dependency = 1
+
         if isinstance(memberType, idltype.Declared):
             memberType.decl().accept(self)
         elif isinstance(memberType, idltype.Sequence):
@@ -538,21 +573,15 @@ def visitStruct(node):
         struct_name = id.Name(scopedName).simple()
 
         tophalf.out("""\
-static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_struct_tc("@repoID@", "@name@", @structmember_mangled_name@, @n@);""",
+#ifdef @mangled_name@
+#  undef @mangled_name@
+#endif
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_struct_tc("@repoID@", "@name@", @structmember_mangled_name@, @n@, &@pprefix@_tcTrack);
+""",
                     mangled_name = mangled_name,
                     structmember_mangled_name = structmember_mangled_name,
                     name = struct_name, n = str(num),
-                    repoID = repoID)   
-
-        if resolveForward(node):
-            fwname  = mangleName(config.state['Private Prefix'] +\
-                                 "_fs_", scopedName)
-            resname = mangleName(config.state['Private Prefix'] +\
-                                 "_rf_", scopedName)
-            tophalf.out("""\
-static int @resname@ = @fwname@->PR_resolve_forward(@mangled_name@);""",
-                        fwname=fwname, resname=resname,
-                        mangled_name=mangled_name)
+                    repoID = repoID, pprefix=config.state['Private Prefix'])
 
     self.__immediatelyInsideModule = insideModule
 
@@ -564,13 +593,23 @@ static int @resname@ = @fwname@->PR_resolve_forward(@mangled_name@);""",
     return
 
 def visitStructForward(node):
-    if (not node.mainFile() and
-        self.__resolving_dependency and
-        not currently_being_defined(node)):
-        
-        startingNode(node)
-        node.fullDecl().accept(self)
-        finishingNode()
+    scopedName = node.scopedName()
+    mangled_name = mangleName(config.state['Private Prefix'] +
+                              "_tc_", scopedName)
+    fmangled_name = mangleName(config.state['Private Prefix'] +
+                               "_ft_", scopedName)
+
+    if not alreadyDefined(fmangled_name):
+        defineName(fmangled_name)
+
+        tophalf.out("""\
+static CORBA::TypeCode_ptr @fmangled_name@ = CORBA::TypeCode::PR_forward_tc("@repoId@", &@pprefix@_tcTrack);
+#define @mangled_name@ @fmangled_name@
+""",
+                    mangled_name = mangled_name,
+                    fmangled_name = fmangled_name,
+                    repoId = node.repoId(),
+                    pprefix=config.state['Private Prefix'])
 
 def visitUnion(node):
     scopedName = node.scopedName()
@@ -596,7 +635,8 @@ def visitUnion(node):
     deref_switchType = switchType.deref()
     if isinstance(switchType.type(), idltype.Declared):
         save_resolving_dependency = self.__resolving_dependency
-        self.__resolving_dependency = 1
+        if not node.constrType():
+            self.__resolving_dependency = 1
         switchType.type().decl().accept(self)
         self.__resolving_dependency = save_resolving_dependency
         
@@ -610,7 +650,10 @@ def visitUnion(node):
         caseType = types.Type(c.caseType())
 
         save_resolving_dependency = self.__resolving_dependency
-        self.__resolving_dependency = 1
+
+        if not c.constrType():
+            self.__resolving_dependency = 1
+
         if isinstance(caseType.type(), idltype.Declared):
             caseType.type().decl().accept(self)
         elif caseType.sequence():
@@ -646,14 +689,19 @@ def visitUnion(node):
                                           "_unionMember_", scopedName)
     
     default_str = ""
-    if hasDefault != None:
-        default_str = ", " + str(hasDefault)
+    if hasDefault is None:
+        default_str = "-1"
+    else:
+        default_str = str(hasDefault)
 
     tophalf.out("""\
 static CORBA::PR_unionMember @unionmember_mangled_name@[] = {
   @members@
 };
-static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_union_tc("@repoID@", "@name@", @discrim_tc@, @unionmember_mangled_name@, @labels@@default_str@);""",
+#ifdef @mangled_name@
+#  undef @mangled_name@
+#endif
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_union_tc("@repoID@", "@name@", @discrim_tc@, @unionmember_mangled_name@, @labels@, @default_str@, &@pprefix@_tcTrack);""",
                 mangled_name = mangled_name,
                 repoID = repoID,
                 discrim_tc = discrim_tc,
@@ -661,22 +709,12 @@ static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_union_tc("@repoI
                 name = union_name,
                 labels = str(numlabels),
                 default_str = default_str,
-                members = string.join(array, ",\n"))
+                members = string.join(array, ",\n"),
+                pprefix = config.state['Private Prefix'])
     
     defineName(unionmember_mangled_name)
     defineName(mangled_name)
 
-    if resolveForward(node):
-        fwname  = mangleName(config.state['Private Prefix'] +\
-                             "_fs_", scopedName)
-        resname = mangleName(config.state['Private Prefix'] +\
-                             "_rf_", scopedName)
-        tophalf.out("""\
-static int @resname@ = @fwname@->PR_resolve_forward(@mangled_name@);""",
-                    fwname=fwname, resname=resname,
-                    mangled_name=mangled_name)
-
-    
     self.__immediatelyInsideModule = insideModule
 
     external_linkage(node)
@@ -688,13 +726,25 @@ static int @resname@ = @fwname@->PR_resolve_forward(@mangled_name@);""",
     finishingNode()
 
 def visitUnionForward(node):
-    if (not node.mainFile() and
-        self.__resolving_dependency and
-        not currently_being_defined(node)):
-        
-        startingNode(node)
-        node.fullDecl().accept(self)
-        finishingNode()
+    scopedName = node.scopedName()
+    mangled_name = mangleName(config.state['Private Prefix'] +
+                              "_tc_", scopedName)
+    fmangled_name = mangleName(config.state['Private Prefix'] +
+                               "_ft_", scopedName)
+
+    if not alreadyDefined(fmangled_name):
+        defineName(fmangled_name)
+
+        tophalf.out("""\
+static CORBA::TypeCode_ptr @fmangled_name@ = CORBA::TypeCode::PR_forward_tc("@repoId@", &@pprefix@_tcTrack);
+#define @mangled_name@ @fmangled_name@
+""",
+                    mangled_name = mangled_name,
+                    fmangled_name = fmangled_name,
+                    repoId = node.repoId(),
+                    pprefix=config.state['Private Prefix'])
+
+
 
 def visitEnum(node):
     scopedName = node.scopedName()
@@ -719,13 +769,14 @@ def visitEnum(node):
 
     tophalf.out("""\
 static const char* @enummember_mangled_name@[] = { @elements@ };
-static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_enum_tc("@repoID@", "@name@", @enummember_mangled_name@, @numcases@);""",
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_enum_tc("@repoID@", "@name@", @enummember_mangled_name@, @numcases@, &@pprefix@_tcTrack);""",
                 enummember_mangled_name = enummember_mangled_name,
                 mangled_name = mangled_name,
                 elements = string.join(names, ", "),
                 repoID = repoID,
                 name = enum_name,
-                numcases = str(len(names)))
+                numcases = str(len(names)),
+                pprefix = config.state['Private Prefix'])
 
     defineName(mangled_name)
     defineName(enummember_mangled_name)
@@ -757,8 +808,16 @@ def visitInterface(node):
     
     repoID = node.repoId()
     iname = id.Name(node.scopedName()).simple()
-    typecode = "CORBA::TypeCode::PR_interface_tc(\"" + repoID + "\", \"" +\
-               iname + "\")"
+
+    if node.abstract():
+        func = "PR_abstract_interface_tc"
+    elif node.local():
+        func = "PR_local_interface_tc"
+    else:
+        func = "PR_interface_tc"
+
+    typecode = 'CORBA::TypeCode::' + func + '("' + repoID + '", "' +\
+               iname + '", &' + config.state['Private Prefix'] + '_tcTrack)'
 
     node.accept(tcstring)
 
@@ -767,16 +826,18 @@ def visitInterface(node):
     finishingNode()
 
 
-def recurse(type):
+def recurse(type, constr=0):
     assert isinstance(type, types.Type)
     
     deref_type = type.deref()
     if isinstance(type.type(), idltype.Declared):
         base_decl = type.type().decl()
         save_resolving_dependency = self.__resolving_dependency
-        self.__resolving_dependency = 1
+        if not constr:
+            self.__resolving_dependency = 1
         base_decl.accept(self)
         self.__resolving_dependency = save_resolving_dependency
+
     elif deref_type.sequence():
         seqType = deref_type.type().seqType()
         if isinstance(seqType, idltype.Declared):
@@ -812,13 +873,14 @@ def visitDeclarator(declarator):
     typedef_name = id.Name(scopedName).simple()
     
     tophalf.out("""\
-static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_alias_tc("@repoID@", "@name@", @typecode@);
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_alias_tc("@repoID@", "@name@", @typecode@, &@pprefix@_tcTrack);
 
 """,
                 mangled_name = mangled_name,
                 repoID = repoID,
                 name = typedef_name,
-                typecode = typecode)
+                typecode = typecode,
+                pprefix = config.state['Private Prefix'])
     defineName(mangled_name)
 
 
@@ -827,9 +889,7 @@ static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_alias_tc("@repoI
 def visitTypedef(node):
     aliasType = types.Type(node.aliasType())
 
-    prefix = "CORBA::TypeCode::PR_"
-
-    recurse(aliasType)
+    recurse(aliasType, node.constrType())
 
     for declarator in node.declarators():
         declarator.accept(self)
@@ -857,13 +917,18 @@ def visitException(node):
     # create the static typecodes for constructed types by setting
     # the resolving_dependency flag and recursing
     save_resolving_dependency = self.__resolving_dependency
-    self.__resolving_dependency = 1
 
     insideModule = self.__immediatelyInsideModule
     self.__immediatelyInsideModule = 0
     
     for child in node.members():
         memberType = child.memberType()
+
+        if child.constrType():
+            self.__resolving_dependency = save_resolving_dependency
+        else:
+            self.__resolving_dependency = 1
+
         if isinstance(memberType, idltype.Declared):
             memberType.decl().accept(self)
 
@@ -885,11 +950,12 @@ def visitException(node):
     if num == 0:
         structmember_mangled_name = "(CORBA::PR_structMember*) 0"
     tophalf.out("""\
-static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_exception_tc("@repoID@", "@name@", @structmember_mangled_name@, @n@);""",
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_exception_tc("@repoID@", "@name@", @structmember_mangled_name@, @n@, &@pprefix@_tcTrack);""",
                 mangled_name = mangled_name,
                 name = ex_name, n = str(num),
                 structmember_mangled_name = structmember_mangled_name,
-                repoID = repoID)
+                repoID = repoID,
+                pprefix = config.state['Private Prefix'])
 
     external_linkage(node)
 
@@ -899,3 +965,220 @@ static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_exception_tc("@r
 
 
     finishingNode()
+
+
+
+
+# builds an instance of CORBA::PR_valueMember containing pointers
+# to all the TypeCodes of the value statemembers
+def buildStateMembersStructure(node):
+    struct = output.StringStream()
+    mangled_name = mangleName(config.state['Private Prefix'] + \
+                              "_valuemember_", node.scopedName())
+    if alreadyDefined(mangled_name):
+        # no need to regenerate
+        return struct
+    
+    defineName(mangled_name)
+    
+    members = node.statemembers()
+    array = []
+
+    for m in members:
+        memberType = types.Type(m.memberType())
+        access = m.memberAccess()
+        for d in m.declarators():
+            this_name = id.Name(d.scopedName()).simple()
+            typecode = mkTypeCode(memberType, d, node)
+            array.append('{"%s", %s, %d}' % (this_name, typecode, access))
+
+    struct.out("""\
+static CORBA::PR_valueMember @mangled_name@[] = {
+  @members@
+};""", members = string.join(array, ",\n"), mangled_name = mangled_name)
+
+    return struct
+
+# Convenience function to total up the number of members, treating
+# declarators separately.
+def numStateMembers(node):
+    members = node.statemembers()
+    num = 0
+    for m in members:
+        num = num + len(m.declarators())
+
+    return num
+
+
+def visitValue(node):
+    # Used for abstract value too
+    
+    startingNode(node)
+    
+    scopedName = node.scopedName()
+    mangled_name = mangleName(config.state['Private Prefix'] +
+                              "_tc_", scopedName)
+
+    visitValueForward(node)
+
+    # the key here is to redirect the bottom half to a buffer
+    # just for now
+    oldbottomhalf = self.bottomhalf
+    self.bottomhalf = output.StringStream()
+
+    insideModule = self.__immediatelyInsideModule
+    self.__immediatelyInsideModule = 0
+
+    # handle nested types
+    for n in node.declarations():
+        n.accept(self)
+
+    # create the static typecodes for constructed types by setting
+    # the resolving_dependency flag and recursing
+    save_resolving_dependency = self.__resolving_dependency
+    
+    for child in node.statemembers():
+        memberType = child.memberType()
+
+        if child.constrType():
+            self.__resolving_dependency = save_resolving_dependency
+        else:
+            self.__resolving_dependency = 1
+
+        if isinstance(memberType, idltype.Declared):
+            decl = memberType.decl()
+            if not currently_being_defined(decl):
+                decl.accept(self)
+        elif isinstance(memberType, idltype.Sequence):
+            # anonymous sequence (maybe sequence<sequence<...<T>>>)
+            # Find the ultimate base type, and if it's user declared then
+            # produce a typecode definition for it.
+            base_type = memberType.seqType()
+            while isinstance(base_type, idltype.Sequence):
+                base_type = base_type.seqType()
+
+            if isinstance(base_type, idltype.Declared):
+                decl = base_type.decl()
+                if not currently_being_defined(decl):
+                    decl.accept(self)
+                        
+    self.__resolving_dependency = save_resolving_dependency
+
+    tophalf.out(str(buildStateMembersStructure(node)))
+
+    if not alreadyDefined(mangled_name):
+        # only define the name once
+        defineName(mangled_name)
+    
+        valuemember_mangled_name = mangleName(config.state['Private Prefix'] +
+                                              "_valuemember_", scopedName)
+        assert alreadyDefined(valuemember_mangled_name), \
+               "The name \"" + valuemember_mangled_name + \
+               "\" should be defined by now"
+    
+        num = numStateMembers(node)
+        repoID = node.repoId()
+        value_name = id.Name(scopedName).simple()
+
+        # Value modifiers
+        modifierl = []
+        if isinstance(node, idlast.Value):
+            if node.custom():
+                modifierl.append("CORBA::VM_CUSTOM")
+            if node.truncatable():
+                modifierl.append("CORBA::VM_TRUNCATABLE")
+        else:
+            assert isinstance(node, idlast.ValueAbs)
+            modifierl.append("CORBA::VM_ABSTRACT")
+
+        if modifierl:
+            modifiers = string.join(modifierl, "|")
+        else:
+            modifiers = "CORBA::VM_NONE"
+
+        # Concrete base
+        inherits = node.inherits()
+        if (isinstance(node, idlast.Value) and
+            inherits and isinstance(inherits[0], idlast.Value)):
+
+            visitValueForward(inherits[0])
+            bscopedName = inherits[0].scopedName()
+            concrete_base = mangleName(config.state['Private Prefix'] +
+                                       "_tc_", bscopedName)
+        else:
+            concrete_base = "CORBA::TypeCode::PR_null_tc()"
+
+        tophalf.out("""\
+#ifdef @mangled_name@
+#  undef @mangled_name@
+#endif
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_value_tc("@repoID@", "@name@", @modifiers@, @concrete_base@, @valuemember_mangled_name@, @n@, &@pprefix@_tcTrack);
+""",
+                    mangled_name = mangled_name,
+                    modifiers = modifiers,
+                    concrete_base = concrete_base,
+                    valuemember_mangled_name = valuemember_mangled_name,
+                    name = value_name, n = str(num),
+                    repoID = repoID, pprefix=config.state['Private Prefix'])
+
+    self.__immediatelyInsideModule = insideModule
+
+    external_linkage(node)
+    # restore the old bottom half
+    oldbottomhalf.out(str(self.bottomhalf))
+    self.bottomhalf = oldbottomhalf
+    finishingNode()
+
+
+def visitValueForward(node):
+    scopedName = node.scopedName()
+    mangled_name = mangleName(config.state['Private Prefix'] +
+                              "_tc_", scopedName)
+    fmangled_name = mangleName(config.state['Private Prefix'] +
+                               "_ft_", scopedName)
+
+    if not alreadyDefined(fmangled_name):
+        defineName(fmangled_name)
+
+        tophalf.out("""\
+static CORBA::TypeCode_ptr @fmangled_name@ = CORBA::TypeCode::PR_forward_tc("@repoId@", &@pprefix@_tcTrack);
+#define @mangled_name@ @fmangled_name@
+""",
+                    mangled_name = mangled_name,
+                    fmangled_name = fmangled_name,
+                    repoId = node.repoId(),
+                    pprefix=config.state['Private Prefix'])
+
+
+def visitValueAbs(node):
+    visitValue(node)
+
+def visitValueBox(node):
+    boxedType = types.Type(node.boxedType())
+
+    recurse(boxedType, node.constrType())
+    
+    scopedName = node.scopedName()
+    mangled_name = mangleName(config.state['Private Prefix'] +\
+                              "_tc_", scopedName)
+    if alreadyDefined(mangled_name):
+        return
+    
+    repoID = node.repoId()
+    typecode = mkTypeCode(boxedType)
+
+    scopedName = node.scopedName()
+    boxed_name = id.Name(scopedName).simple()
+    
+    tophalf.out("""\
+static CORBA::TypeCode_ptr @mangled_name@ = CORBA::TypeCode::PR_value_box_tc("@repoID@", "@name@", @typecode@, &@pprefix@_tcTrack);
+
+""",
+                mangled_name = mangled_name,
+                repoID = repoID,
+                name = boxed_name,
+                typecode = typecode,
+                pprefix = config.state['Private Prefix'])
+
+    defineName(mangled_name)
+    external_linkage(node)    
