@@ -26,33 +26,62 @@
 #   
 #   C++ templates for the .hh file
 
-# $Id: template.py,v 1.5.2.27 2005/12/30 17:40:21 dgrisby Exp $
+# $Id: template.py,v 1.8.2.18 2007/09/19 14:16:07 dgrisby Exp $
 # $Log: template.py,v $
-# Revision 1.5.2.27  2005/12/30 17:40:21  dgrisby
-# Previously extern inline functions are now just inline or static
-# inline when inside classes.
+# Revision 1.8.2.18  2007/09/19 14:16:07  dgrisby
+# Avoid namespace clashes if IDL defines modules named CORBA.
 #
-# Revision 1.5.2.26  2005/12/15 12:01:36  dgrisby
-# Functions declared extern inline upset the IRIX compiler.
+# Revision 1.8.2.17  2007/05/11 09:52:27  dgrisby
+# New -Wbguard_prefix option. Thanks Austin Bingham.
 #
-# Revision 1.5.2.25  2005/07/21 15:25:09  dgrisby
-# Silence some gcc 4 warnings / errors.
+# Revision 1.8.2.16  2006/10/23 15:36:25  dgrisby
+# Undefine USE_stub_in_nt_dll at the end of header if it was not defined
+# at the start.
 #
-# Revision 1.5.2.24  2004/10/18 02:44:37  dgrisby
-# New -Wbdll_includes option.
+# Revision 1.8.2.15  2006/09/04 11:40:06  dgrisby
+# Remove crazy switch code in enum marshalling.
 #
-# Revision 1.5.2.23  2004/07/29 10:36:34  dgrisby
-# Bug with unmarshalling unions with multiple case labels.
+# Revision 1.8.2.14  2006/01/10 12:24:03  dgrisby
+# Merge from omni4_0_develop pre 4.0.7 release.
 #
-# Revision 1.5.2.22  2004/07/23 14:07:04  dgrisby
-# Subtly incorrect generated code for arrays.
+# Revision 1.8.2.13  2005/11/14 11:02:16  dgrisby
+# Local interface fixes.
 #
-# Revision 1.5.2.21  2004/04/19 09:25:21  dgrisby
-# Nil object references are bound to the lifetime of the skeleton file
-# if OMNI_UNLOADABLE_STUBS is defined.
+# Revision 1.8.2.12  2005/11/09 12:22:17  dgrisby
+# Local interfaces support.
 #
-# Revision 1.5.2.20  2004/02/11 11:49:57  dgrisby
-# Generate _out types for simple typedefs.
+# Revision 1.8.2.11  2005/08/16 13:51:21  dgrisby
+# Problems with valuetype / abstract interface C++ mapping.
+#
+# Revision 1.8.2.10  2005/07/22 17:18:37  dgrisby
+# Another merge from omni4_0_develop.
+#
+# Revision 1.8.2.9  2005/01/06 23:10:06  dgrisby
+# Big merge from omni4_0_develop.
+#
+# Revision 1.8.2.8  2005/01/06 16:35:18  dgrisby
+# Narrowing for abstract interfaces.
+#
+# Revision 1.8.2.7  2004/10/13 17:58:24  dgrisby
+# Abstract interfaces support; values support interfaces; value bug fixes.
+#
+# Revision 1.8.2.6  2004/07/31 23:46:27  dgrisby
+# Correct constness of exception Any insertion operator.
+#
+# Revision 1.8.2.5  2004/07/23 10:29:59  dgrisby
+# Completely new, much simpler Any implementation.
+#
+# Revision 1.8.2.4  2004/07/04 23:53:39  dgrisby
+# More ValueType TypeCode and Any support.
+#
+# Revision 1.8.2.3  2004/02/16 10:10:32  dgrisby
+# More valuetype, including value boxes. C++ mapping updates.
+#
+# Revision 1.8.2.2  2003/10/23 11:25:55  dgrisby
+# More valuetype support.
+#
+# Revision 1.8.2.1  2003/03/23 21:02:36  dgrisby
+# Start of omniORB 4.1.x development branch.
 #
 # Revision 1.5.2.19  2001/11/12 13:46:07  dpg1
 # _unchecked_narrow, improved _narrow.
@@ -195,8 +224,8 @@
 ##
 header = """\
 // This file is generated by @program@- @library@. Do not edit.
-#ifndef __@guard@_hh__
-#define __@guard@_hh__
+#ifndef @guard_prefix@__@guard@_hh__
+#define @guard_prefix@__@guard@_hh__
 """
 
 footer = """\
@@ -207,14 +236,13 @@ footer = """\
 ## Main file
 ##
 main = """\
-#ifndef USE_omniORB_logStream
-#define USE_omniORB_logStream
-#endif
-
 #ifndef __CORBA_H_EXTERNAL_GUARD__
 #include <omniORB4/CORBA.h>
 #endif
 
+#ifndef  USE_stub_in_nt_dll
+# define USE_stub_in_nt_dll_NOT_DEFINED_@guard@
+#endif
 #ifndef  USE_core_stub_in_nt_dll
 # define USE_core_stub_in_nt_dll_NOT_DEFINED_@guard@
 #endif
@@ -228,12 +256,12 @@ main = """\
 @sub_include_post@
 
 #ifdef USE_stub_in_nt_dll
-#ifndef USE_core_stub_in_nt_dll
-#define USE_core_stub_in_nt_dll
-#endif
-#ifndef USE_dyn_stub_in_nt_dll
-#define USE_dyn_stub_in_nt_dll
-#endif
+# ifndef USE_core_stub_in_nt_dll
+#  define USE_core_stub_in_nt_dll
+# endif
+# ifndef USE_dyn_stub_in_nt_dll
+#  define USE_dyn_stub_in_nt_dll
+# endif
 #endif
 
 #ifdef _core_attr
@@ -264,6 +292,8 @@ main = """\
 
 @poa@
 
+@obv@
+
 @other_tie@
 
 #undef _core_attr
@@ -273,6 +303,10 @@ main = """\
 
 @marshalling@
 
+#ifdef   USE_stub_in_nt_dll_NOT_DEFINED_@guard@
+# undef  USE_stub_in_nt_dll
+# undef  USE_stub_in_nt_dll_NOT_DEFINED_@guard@
+#endif
 #ifdef   USE_core_stub_in_nt_dll_NOT_DEFINED_@guard@
 # undef  USE_core_stub_in_nt_dll
 # undef  USE_core_stub_in_nt_dll_NOT_DEFINED_@guard@
@@ -307,19 +341,11 @@ sub_include_post = """\
 """
 
 main_include = """\
-#ifndef __@guardname@_EXTERNAL_GUARD__
-#define __@guardname@_EXTERNAL_GUARD__
+#ifndef @guard_prefix@__@guardname@_EXTERNAL_GUARD__
+#define @guard_prefix@__@guardname@_EXTERNAL_GUARD__
 #include @filename@
 #endif"""
 
-##
-## tcParser_unionHelper forward declaration
-##
-tcParser_unionHelper = """\
-// Declare helper class for union type @fqname@
-class @private_prefix@_tcParser_unionhelper_@guard_name@;
-"""
-
 ##
 ## Modules
 ##
@@ -342,6 +368,16 @@ POA_module_end = """\
 _CORBA_MODULE_END
 """
 
+OBV_module_begin = """\
+_CORBA_MODULE @OBV_prefix@@name@
+_CORBA_MODULE_BEG
+"""
+
+OBV_module_end = """\
+_CORBA_MODULE_END
+"""
+
+
 POA_interface = """\
 class @POA_name@ :
   public virtual @impl_scopedID@,
@@ -360,6 +396,7 @@ public:
 ##
 ## Interfaces
 ##
+
 interface_Helper = """\
 #ifndef __@guard@__
 #define __@guard@__
@@ -390,6 +427,7 @@ typedef _CORBA_ObjRef_OUT_arg<_objref_@name@,@name@_Helper > @name@_out;
 """
 
 interface_type = """\
+// interface @name@
 class @name@ {
 public:
   // Declarations for this interface type.
@@ -397,8 +435,9 @@ public:
   typedef @name@_var _var_type;
 
   static _ptr_type _duplicate(_ptr_type);
-  static _ptr_type _narrow(CORBA::Object_ptr);
-  static _ptr_type _unchecked_narrow(CORBA::Object_ptr);
+  static _ptr_type _narrow(::CORBA::Object_ptr);
+  static _ptr_type _unchecked_narrow(::CORBA::Object_ptr);
+  @abstract_narrows@
   static _ptr_type _nil();
 
   static inline void _marshalObjRef(_ptr_type, cdrStream&);
@@ -418,6 +457,182 @@ public:
 };
 """
 
+interface_abstract_narrows = """\
+static _ptr_type _narrow(::CORBA::AbstractBase_ptr);
+static _ptr_type _unchecked_narrow(::CORBA::AbstractBase_ptr);
+"""
+
+
+##
+## Abstract Interfaces
+##
+
+abstract_interface_Helper = """\
+#ifndef __@guard@__
+#define __@guard@__
+
+class @name@;
+class _objref_@name@;
+typedef @name@* @name@_ptr;
+typedef @name@_ptr @name@Ref;
+
+class @name@_Helper {
+public:
+  typedef @name@_ptr _ptr_type;
+
+  static _ptr_type _nil();
+  static _CORBA_Boolean is_nil(_ptr_type);
+  static void release(_ptr_type);
+  static void duplicate(_ptr_type);
+  static void marshalObjRef(_ptr_type, cdrStream&);
+  static _ptr_type unmarshalObjRef(cdrStream&);
+};
+
+typedef _CORBA_ObjRef_Var<@name@, @name@_Helper> @name@_var;
+typedef _CORBA_ObjRef_OUT_arg<@name@,@name@_Helper > @name@_out;
+
+#endif
+"""
+
+abstract_interface_type = """\
+// abstract interface @name@
+class @name@ :
+  @inherits@
+{
+public:
+  // Declarations for this interface type.
+  typedef @name@_ptr _ptr_type;
+  typedef @name@_var _var_type;
+
+  static _ptr_type _duplicate(_ptr_type);
+  static _ptr_type _narrow(::CORBA::AbstractBase_ptr);
+  static _ptr_type _unchecked_narrow(::CORBA::AbstractBase_ptr);
+  static _ptr_type _nil();
+
+  static inline void _marshalObjRef(_ptr_type, cdrStream&);
+
+  static inline _ptr_type _unmarshalObjRef(cdrStream& s) {
+    _CORBA_Boolean b = s.unmarshalBoolean();
+    if (b) {
+      omniObjRef* o = omniObjRef::_unMarshal(_PD_repoId,s);
+      if (o)
+        return (_ptr_type) o->_ptrToObjRef(_PD_repoId);
+      else
+        return _nil();
+    }
+    else {
+      ::CORBA::ValueBase* v = ::CORBA::ValueBase::_NP_unmarshal(s);
+      if (v)
+        return (_ptr_type) v->_ptrToValue(_PD_repoId);
+      else
+        return 0;
+    }
+  }
+
+  static _core_attr const char* _PD_repoId;
+
+  // Other IDL defined within this scope.
+  @Other_IDL@
+
+  // Operations declared in this abstract interface
+  @operations@  
+};
+"""
+
+##
+## Local Interfaces
+##
+
+local_interface_Helper = """\
+#ifndef __@guard@__
+#define __@guard@__
+
+class @name@;
+typedef @name@* @name@_ptr;
+typedef @name@_ptr @name@Ref;
+
+class @name@_Helper {
+public:
+  typedef @name@_ptr _ptr_type;
+
+  static _ptr_type _nil();
+  static _CORBA_Boolean is_nil(_ptr_type);
+  static void release(_ptr_type);
+  static void duplicate(_ptr_type);
+  static void marshalObjRef(_ptr_type, cdrStream&);
+  static _ptr_type unmarshalObjRef(cdrStream&);
+};
+
+typedef _CORBA_ObjRef_Var<@name@, @name@_Helper> @name@_var;
+typedef _CORBA_ObjRef_OUT_arg<@name@,@name@_Helper > @name@_out;
+
+#endif
+"""
+
+local_interface_type = """\
+// local interface @name@
+class @name@ :
+  @inherits@
+{
+public:
+  // Declarations for this interface type.
+  typedef @name@_ptr _ptr_type;
+  typedef @name@_var _var_type;
+
+  static _ptr_type _duplicate(_ptr_type);
+  static _ptr_type _narrow(::CORBA::Object_ptr);
+  static _ptr_type _unchecked_narrow(::CORBA::Object_ptr);
+  @abstract_narrows@
+  static _ptr_type _nil();
+
+  static inline void _marshalObjRef(_ptr_type, cdrStream& s) {
+    OMNIORB_THROW(MARSHAL, _OMNI_NS(MARSHAL_LocalObject),
+                  (::CORBA::CompletionStatus)s.completion());
+  }
+  static inline _ptr_type _unmarshalObjRef(cdrStream& s) {
+    OMNIORB_THROW(MARSHAL, _OMNI_NS(MARSHAL_LocalObject),
+                  (::CORBA::CompletionStatus)s.completion());
+#ifdef NEED_DUMMY_RETURN
+    return 0;
+#endif
+  }
+
+  static _core_attr const char* _PD_repoId;
+
+  // Other IDL defined within this scope.
+  @Other_IDL@
+
+  // Operations declared in this local interface
+  @operations@  
+
+private:
+  virtual void* _ptrToObjRef(const char*);
+
+protected:
+  @name@();
+  virtual ~@name@();
+};
+
+class _nil_@name@ :
+  @nil_inherits@
+  public virtual @name@
+{
+  public:
+    @nil_operations@
+
+    inline _nil_@name@() { _PR_setobj(0); }
+
+  protected:
+    virtual ~_nil_@name@();
+};
+"""
+
+
+
+##
+## Object reference
+##
+
 interface_objref = """\
 class _objref_@name@ :
   @inherits@
@@ -449,6 +664,10 @@ _impl_@name@* _shortcut;
 const _CORBA_Boolean* _invalid;\
 """
 
+##
+## Proxy Object Factory
+##
+
 interface_pof = """\
 class _pof_@name@ : public _OMNI_NS(proxyObjectFactory) {
 public:
@@ -460,6 +679,10 @@ public:
 };
 """
 
+##
+## Interface Impl class
+##
+
 interface_impl = """\
 class _impl_@name@ :
   @inherits@
@@ -475,10 +698,22 @@ public:  // Really protected, workaround for xlC
 private:
   virtual void* _ptrToInterface(const char*);
   virtual const char* _mostDerivedRepoId();
+  @abstract@
 };
 
 """
 
+interface_impl_abstract = """\
+virtual void _interface_is_abstract() = 0;"""
+
+interface_impl_not_abstract = """\
+virtual void _interface_is_abstract();"""
+
+
+##
+## Old BOA skeleton class
+##
+
 interface_sk = """\
 class _sk_@name@ :
   public virtual _impl_@name@,
@@ -495,6 +730,10 @@ public:
 };
 """
 
+##
+## Objref marshal function
+##
+
 interface_marshal_forward = """\
 inline void
 @name@::_marshalObjRef(::@name@_ptr obj, cdrStream& s) {
@@ -503,6 +742,30 @@ inline void
 
 """
 
+abstract_interface_marshal_forward = """\
+inline void
+@name@::_marshalObjRef(::@name@_ptr obj, cdrStream& s) {
+  if (obj) {
+    ::CORBA::ValueBase* v = obj->_NP_to_value();
+    if (v) {
+      s.marshalBoolean(0);
+      ::CORBA::ValueBase::_NP_marshal(v,s);
+      return;
+    }
+    ::CORBA::Object_ptr o = obj->_NP_to_object();
+    if (o) {
+      s.marshalBoolean(1);
+      omniObjRef::_marshal(o->_PR_getobj(),s);
+      return;
+    }
+  }
+  s.marshalBoolean(0);
+  ::CORBA::ValueBase::_NP_marshal(0, s);
+}
+"""
+
+
+
 ##
 ## Typedefs
 ##
@@ -522,25 +785,25 @@ typedef @base@_forany @derived@_forany;
 
 typedef_simple_string = """\
 typedef char* @name@;
-typedef CORBA::String_var @name@_var;
-typedef CORBA::String_out @name@_out;
+typedef ::CORBA::String_var @name@_var;
+typedef ::CORBA::String_out @name@_out;
 """
 
 typedef_simple_wstring = """\
-typedef CORBA::WChar* @name@;
-typedef CORBA::WString_var @name@_var;
-typedef CORBA::WString_out @name@_out;
+typedef ::CORBA::WChar* @name@;
+typedef ::CORBA::WString_var @name@_var;
+typedef ::CORBA::WString_out @name@_out;
 """
 
 typedef_simple_typecode = """\
-typedef CORBA::TypeCode_ptr @name@_ptr;
-typedef CORBA::TypeCode_var @name@_var;
+typedef ::CORBA::TypeCode_ptr @name@_ptr;
+typedef ::CORBA::TypeCode_var @name@_var;
 """
 
 typedef_simple_any = """\
-typedef CORBA::Any @name@;
-typedef CORBA::Any_var @name@_var;
-typedef CORBA::Any_out @name@_out;
+typedef ::CORBA::Any @name@;
+typedef ::CORBA::Any_var @name@_var;
+typedef ::CORBA::Any_out @name@_out;
 """
 
 typedef_simple_fixed = """\
@@ -732,6 +995,7 @@ public:
   @subscript_operator@
 
   inline @name@* operator -> () { return _pd_seq; }
+  inline const @name@* operator -> () const { return _pd_seq; }
 #if defined(__GNUG__)
   inline operator @name@& () const { return *_pd_seq; }
 #else
@@ -821,6 +1085,10 @@ struct_variable_out_type = """\
 typedef _CORBA_ConstrType_Variable_OUT_arg< @name@,@name@_var > @name@_out;
 """
 
+struct_array_declarator = """\
+typedef @memtype@ @prefix@_@cxx_id@@dims@;
+typedef @memtype@ _@cxx_id@_slice@tail_dims@;
+"""
 
 struct_nonarray_sequence = """\
 typedef @memtype@ _@cxx_id@_seq;
@@ -840,7 +1108,7 @@ struct @name@;
 ##
 
 exception = """\
-class @name@ : public CORBA::UserException {
+class @name@ : public ::CORBA::UserException {
 public:
   @Other_IDL@
   @members@
@@ -854,9 +1122,9 @@ public:
   @name@& operator=(const @name@&);
   virtual ~@name@();
   virtual void _raise() const;
-  static @name@* _downcast(CORBA::Exception*);
-  static const @name@* _downcast(const CORBA::Exception*);
-  static inline @name@* _narrow(CORBA::Exception* _e) {
+  static @name@* _downcast(::CORBA::Exception*);
+  static const @name@* _downcast(const ::CORBA::Exception*);
+  static inline @name@* _narrow(::CORBA::Exception* _e) {
     return _downcast(_e);
   }
   
@@ -866,7 +1134,7 @@ public:
   static _core_attr insertExceptionToAny    insertToAnyFn;
   static _core_attr insertExceptionToAnyNCP insertToAnyFnNCP;
 
-  virtual CORBA::Exception* _NP_duplicate() const;
+  virtual ::CORBA::Exception* _NP_duplicate() const;
 
   static _core_attr const char* _PD_repoId;
   static _core_attr const char* _PD_typeId;
@@ -881,7 +1149,7 @@ private:
 
 exception_array_declarator = """\
 typedef @memtype@ @private_prefix@_@cxx_id@@dims@;
-typedef @memtype@ _@cxx_id@_slice;
+typedef @memtype@ _@cxx_id@_slice@tail_dims@;
 """
 
 exception_member = """\
@@ -961,12 +1229,10 @@ public:
   void operator>>= (cdrStream&) const;
   void operator<<= (cdrStream&);
 
-  @tcParser_unionHelper@
-
 private:
   @discrimtype@ _pd__d;
-  CORBA::Boolean _pd__default;
-  CORBA::Boolean _pd__initialised;
+  _CORBA_Boolean _pd__default;
+  _CORBA_Boolean _pd__initialised;
 
   @union@
   @outsideUnion@
@@ -992,14 +1258,14 @@ union {
 union_d_fn_body = """\
 // illegal to set discriminator before making a member active
 if (!_pd__initialised)
-  OMNIORB_THROW(BAD_PARAM,_OMNI_NS(BAD_PARAM_InvalidUnionDiscValue),CORBA::COMPLETED_NO);
+  OMNIORB_THROW(BAD_PARAM,_OMNI_NS(BAD_PARAM_InvalidUnionDiscValue),::CORBA::COMPLETED_NO);
 
 if (_value == _pd__d) return; // no change
 
 @switch@
 
 fail:
-OMNIORB_THROW(BAD_PARAM,_OMNI_NS(BAD_PARAM_InvalidUnionDiscValue),CORBA::COMPLETED_NO);
+OMNIORB_THROW(BAD_PARAM,_OMNI_NS(BAD_PARAM_InvalidUnionDiscValue),::CORBA::COMPLETED_NO);
 
 """
 
@@ -1019,14 +1285,6 @@ void _default()
 }
 """
 
-union_tcParser_friend = """\
-#if defined(__GNUG__) && (__GNUG__ < 4) || defined(__DECCXX) && (__DECCXX_VER < 60000000)
-friend class @private_prefix@_tcParser_unionhelper_@name@;
-#else
-friend class ::@private_prefix@_tcParser_unionhelper_@name@;
-#endif
-"""
-
 union_proxy_float = """
 #ifdef USING_PROXY_FLOAT
   @type@ _pd_@name@@dims@;
@@ -1067,20 +1325,20 @@ void @name@ (const @type@& _value) {
 """
 
 union_typecode = """\
-CORBA::TypeCode_ptr @name@ () const { return _pd_@name@._ptr; }
-void @name@(CORBA::TypeCode_ptr _value) {
+::CORBA::TypeCode_ptr @name@ () const { return _pd_@name@._ptr; }
+void @name@(::CORBA::TypeCode_ptr _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
-  _pd_@name@ = CORBA::TypeCode::_duplicate(_value);
+  _pd_@name@ = ::CORBA::TypeCode::_duplicate(_value);
 }
-void @name@(const CORBA::TypeCode_member& _value) {
+void @name@(const ::CORBA::TypeCode_member& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::TypeCode_var& _value) {
+void @name@(const ::CORBA::TypeCode_var& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
@@ -1112,13 +1370,13 @@ void @name@(const char*  _value) {
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::String_var& _value) {
+void @name@(const ::CORBA::String_var& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::String_member& _value) {
+void @name@(const ::CORBA::String_member& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
@@ -1127,28 +1385,28 @@ void @name@(const CORBA::String_member& _value) {
 """
 
 union_wstring = """\
-const CORBA::WChar * @name@ () const {
-    return (const CORBA::WChar*) _pd_@name@;
+const ::CORBA::WChar * @name@ () const {
+    return (const ::CORBA::WChar*) _pd_@name@;
 }
-void @name@(CORBA::WChar* _value) {
+void @name@(::CORBA::WChar* _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::WChar*  _value) {
+void @name@(const ::CORBA::WChar*  _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::WString_var& _value) {
+void @name@(const ::CORBA::WString_var& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
   _pd_@name@ = _value;
 }
-void @name@(const CORBA::WString_member& _value) {
+void @name@(const ::CORBA::WString_member& _value) {
   _pd__initialised = 1;
   _pd__d = @discrimvalue@;
   _pd__default = @isDefault@;
@@ -1203,6 +1461,18 @@ void @member@ (const _@member@_seq& _value) {
 }
 """
 
+union_value = """\
+@type@* @member@() const { return _pd_@member@.in(); }
+void @member@(@type@* _value) {
+  _pd__initialised = 1;
+  _pd__d = @discrimvalue@;
+  _pd__default = @isDefault@;
+  ::CORBA::add_ref(_value);
+  _pd_@member@ = _value;
+}
+"""
+
+
 union_member = """\
 @type@ _pd_@name@@dims@;
 """
@@ -1239,7 +1509,7 @@ _CORBA_@where@_VAR _core_attr const @type@ @name@;
 ## Typecode_ptr
 ##
 typecode = """\
-@qualifier@ _dyn_attr const CORBA::TypeCode_ptr _tc_@name@;
+@qualifier@ _dyn_attr const ::CORBA::TypeCode_ptr _tc_@name@;
 """
 
 
@@ -1247,66 +1517,69 @@ typecode = """\
 ## Operators
 ##
 any_struct = """\
-extern void operator<<=(CORBA::Any& _a, const @fqname@& _s);
-extern void operator<<=(CORBA::Any& _a, @fqname@* _sp);
-extern CORBA::Boolean operator>>=(const CORBA::Any& _a, @fqname@*& _sp);
-extern CORBA::Boolean operator>>=(const CORBA::Any& _a, const @fqname@*& _sp);
-"""
-
-any_union = """\
-void operator<<=(CORBA::Any& _a, const @fqname@& _s);
-void operator<<=(CORBA::Any& _a, @fqname@* _sp);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, const @fqname@*& _sp);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, @fqname@*& _sp);
-"""
-
-any_enum = """\
-void operator<<=(CORBA::Any& _a, @name@ _s);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, @name@& _s);
-"""
-
-any_interface = """\
-void operator<<=(CORBA::Any& _a, @fqname@_ptr _s);
-void operator<<=(CORBA::Any& _a, @fqname@_ptr* _s);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, @fqname@_ptr& _s);
-"""
-
-any_array_declarator = """\
-void operator<<=(CORBA::Any& _a, const @fqname@_forany& _s);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, @fqname@_forany& _s);
-"""
-
-any_sequence = """\
-extern void operator <<= (CORBA::Any& _a, const @fqname@& _s);
-inline void operator <<= (CORBA::Any& _a, @fqname@* _sp) {
-  _a <<= *_sp;
-  delete _sp;
-}
-extern _CORBA_Boolean operator >>= (const CORBA::Any& _a, @fqname@*& _sp);
-extern _CORBA_Boolean operator >>= (const CORBA::Any& _a, const @fqname@*& _sp);
+extern void operator<<=(::CORBA::Any& _a, const @fqname@& _s);
+extern void operator<<=(::CORBA::Any& _a, @fqname@* _sp);
+extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@*& _sp);
+extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const @fqname@*& _sp);
 """
 
 any_exception = """\
-void operator<<=(CORBA::Any& _a, const @fqname@& _s);
-void operator<<=(CORBA::Any& _a, const @fqname@* _sp);
-CORBA::Boolean operator>>=(const CORBA::Any& _a, const @fqname@*& _sp);
+void operator<<=(::CORBA::Any& _a, const @fqname@& _s);
+void operator<<=(::CORBA::Any& _a, const @fqname@* _sp);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const @fqname@*& _sp);
 """
 
+any_union = """\
+void operator<<=(::CORBA::Any& _a, const @fqname@& _s);
+void operator<<=(::CORBA::Any& _a, @fqname@* _sp);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const @fqname@*& _sp);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@*& _sp);
+"""
+
+any_enum = """\
+void operator<<=(::CORBA::Any& _a, @name@ _s);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @name@& _s);
+"""
+
+any_interface = """\
+void operator<<=(::CORBA::Any& _a, @fqname@_ptr _s);
+void operator<<=(::CORBA::Any& _a, @fqname@_ptr* _s);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@_ptr& _s);
+"""
+
+any_array_declarator = """\
+void operator<<=(::CORBA::Any& _a, const @fqname@_forany& _s);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@_forany& _s);
+"""
+
+any_sequence = """\
+void operator<<=(::CORBA::Any& _a, const @fqname@& _s);
+void operator<<=(::CORBA::Any& _a, @fqname@* _sp);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@*& _sp);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const @fqname@*& _sp);
+"""
+
+any_value = """\
+void operator<<=(::CORBA::Any& _a, @fqname@* _s);
+void operator<<=(::CORBA::Any& _a, @fqname@** _s);
+_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, @fqname@*& _s);
+"""
+
+
 enum_operators = """\
 inline void operator >>=(@name@ _e, cdrStream& s) {
-  ::operator>>=((CORBA::ULong)_e, s);
+  ::operator>>=((::CORBA::ULong)_e, s);
 }
 
 inline void operator <<= (@name@& _e, cdrStream& s) {
-  CORBA::ULong @private_prefix@_e;
+  ::CORBA::ULong @private_prefix@_e;
   ::operator<<=(@private_prefix@_e,s);
-  switch (@private_prefix@_e) {
-    @cases@
+  if (@private_prefix@_e <= @last_item@) {
     _e = (@name@) @private_prefix@_e;
-    break;
-  default:
+  }
+  else {
     OMNIORB_THROW(MARSHAL,_OMNI_NS(MARSHAL_InvalidEnumValue),
-                  (CORBA::CompletionStatus)s.completion());
+                  (::CORBA::CompletionStatus)s.completion());
   }
 }
 """
@@ -1322,14 +1595,14 @@ class @tie_name@ : public virtual @inherits@
 public:
   @tie_name@(_omniT& t)
     : pd_obj(&t), pd_poa(0), pd_rel(0) {}
-  @tie_name@(_omniT& t, PortableServer::POA_ptr p)
+  @tie_name@(_omniT& t, ::PortableServer::POA_ptr p)
     : pd_obj(&t), pd_poa(p), pd_rel(0) {}
-  @tie_name@(_omniT* t, CORBA::Boolean r=1)
+  @tie_name@(_omniT* t, _CORBA_Boolean r=1)
     : pd_obj(t), pd_poa(0), pd_rel(r) {}
-  @tie_name@(_omniT* t, PortableServer::POA_ptr p,CORBA::Boolean r=1)
+  @tie_name@(_omniT* t, ::PortableServer::POA_ptr p,_CORBA_Boolean r=1)
     : pd_obj(t), pd_poa(p), pd_rel(r) {}
   ~@tie_name@() {
-    if( pd_poa )  CORBA::release(pd_poa);
+    if( pd_poa )  ::CORBA::release(pd_poa);
     if( pd_rel )  delete pd_obj;
   }
 
@@ -1341,31 +1614,31 @@ public:
     pd_rel = 0;
   }
 
-  void _tied_object(_omniT* t, CORBA::Boolean r=1) {
+  void _tied_object(_omniT* t, _CORBA_Boolean r=1) {
     if( pd_rel )  delete pd_obj;
     pd_obj = t;
     pd_rel = r;
   }
 
-  CORBA::Boolean _is_owner()        { return pd_rel; }
-  void _is_owner(CORBA::Boolean io) { pd_rel = io;   }
+  _CORBA_Boolean _is_owner()        { return pd_rel; }
+  void _is_owner(_CORBA_Boolean io) { pd_rel = io;   }
 
-  PortableServer::POA_ptr _default_POA() {
-    if( !pd_poa )  return PortableServer::POA::_the_root_poa();
-    else           return PortableServer::POA::_duplicate(pd_poa);
+  ::PortableServer::POA_ptr _default_POA() {
+    if( !pd_poa )  return ::PortableServer::POA::_the_root_poa();
+    else           return ::PortableServer::POA::_duplicate(pd_poa);
   }
 
   @callables@
 
 private:
-  _omniT*                 pd_obj;
-  PortableServer::POA_ptr pd_poa;
-  CORBA::Boolean          pd_rel;
+  _omniT*                   pd_obj;
+  ::PortableServer::POA_ptr pd_poa;
+  _CORBA_Boolean            pd_rel;
 };
 """
 
 tie_template_old = """\
-template <class _omniT, CORBA::Boolean release>
+template <class _omniT, _CORBA_Boolean release>
 class @tie_name@ : public virtual @inherits@
 {
 public:
@@ -1381,7 +1654,7 @@ public:
 
 private:
   _omniT*                  pd_obj;
-  CORBA::Boolean           pd_rel;
+  _CORBA_Boolean           pd_rel;
 };
 """
 
@@ -1391,7 +1664,7 @@ private:
 tcstring = """\
 #if !defined(___tc_string_@n@__) && !defined(DISABLE_Unnamed_Bounded_String_TC)
 #define ___tc_string_@n@__
-_CORBA_GLOBAL_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_string_@n@;
+_CORBA_GLOBAL_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_string_@n@;
 #endif
 """
 
@@ -1401,6 +1674,6 @@ _CORBA_GLOBAL_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_string_@n@;
 tcwstring = """\
 #if !defined(___tc_wstring_@n@__) && !defined(DISABLE_Unnamed_Bounded_WString_TC)
 #define ___tc_wstring_@n@__
-_CORBA_GLOBAL_VAR _dyn_attr const CORBA::TypeCode_ptr _tc_wstring_@n@;
+_CORBA_GLOBAL_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_wstring_@n@;
 #endif
 """
