@@ -26,10 +26,52 @@
 #   
 #   Code associated with IDL interfaces
 
-# $Id: iface.py,v 1.1.4.13 2003/07/25 16:03:06 dgrisby Exp $
+# $Id: iface.py,v 1.1.6.14 2007/09/19 14:16:08 dgrisby Exp $
 # $Log: iface.py,v $
-# Revision 1.1.4.13  2003/07/25 16:03:06  dgrisby
-# Initialise base classes in correct order.
+# Revision 1.1.6.14  2007/09/19 14:16:08  dgrisby
+# Avoid namespace clashes if IDL defines modules named CORBA.
+#
+# Revision 1.1.6.13  2006/01/24 11:46:03  dgrisby
+# Interfaces inheriting from a typedef caused an omniidl compiler error.
+# Thanks Renzo Tomaselli.
+#
+# Revision 1.1.6.12  2005/11/18 18:23:06  dgrisby
+# New -Wbimpl_mapping option.
+#
+# Revision 1.1.6.11  2005/11/14 11:02:16  dgrisby
+# Local interface fixes.
+#
+# Revision 1.1.6.10  2005/11/09 12:22:17  dgrisby
+# Local interfaces support.
+#
+# Revision 1.1.6.9  2005/09/19 15:36:35  dgrisby
+# Refcount shortcut now throws INV_OBJREF when the servant is
+# deactivated, rather than deactivating the shortcut, which could lead
+# to a race condition.
+#
+# Revision 1.1.6.8  2005/09/05 17:22:09  dgrisby
+# Reference counted local call shortcut.
+#
+# Revision 1.1.6.7  2005/08/16 13:51:21  dgrisby
+# Problems with valuetype / abstract interface C++ mapping.
+#
+# Revision 1.1.6.6  2005/01/06 23:09:50  dgrisby
+# Big merge from omni4_0_develop.
+#
+# Revision 1.1.6.5  2005/01/06 16:35:18  dgrisby
+# Narrowing for abstract interfaces.
+#
+# Revision 1.1.6.4  2004/10/13 17:58:22  dgrisby
+# Abstract interfaces support; values support interfaces; value bug fixes.
+#
+# Revision 1.1.6.3  2003/11/06 11:56:56  dgrisby
+# Yet more valuetype. Plain valuetype and abstract valuetype are now working.
+#
+# Revision 1.1.6.2  2003/10/23 11:25:54  dgrisby
+# More valuetype support.
+#
+# Revision 1.1.6.1  2003/03/23 21:02:41  dgrisby
+# Start of omniORB 4.1.x development branch.
 #
 # Revision 1.1.4.12  2003/03/03 15:02:30  dgrisby
 # -Wbvirtual_objref option went astray. Thanks Malge Nishant.
@@ -141,11 +183,17 @@ class Interface:
     return self._callables
 
   def inherits(self):
-    return map(lambda x:Interface(x), self._node.inherits())
+    return map(lambda x:Interface(x), self._node.fullDecl().inherits())
 
   def allInherits(self):
     return map(lambda x:Interface(x), ast.allInherits(self._node))
 
+  def local(self):
+    return self._node.fullDecl().local()
+
+  def abstract(self):
+    return self._node.fullDecl().abstract()
+
   def name(self):
     return self._node_name
     
@@ -190,7 +238,6 @@ class Class(cxx.Class):
 class _objref_Method(cxx.Method):
   def __init__(self, callable, parent_class):
     assert isinstance(callable, call.Callable)
-    assert isinstance(parent_class, cxx.Class)
     self._callable = callable
     self._parent_class = parent_class
     self.from_Callable()
@@ -198,7 +245,8 @@ class _objref_Method(cxx.Method):
   def callable(self): return self._callable
 
   def from_Callable(self):
-    self._from_Callable(use_out = 1)
+    use_out = not config.state["Impl Mapping"]
+    self._from_Callable(use_out = use_out)
 
   def _from_Callable(self, use_out):
     # Grab the IDL environment
@@ -218,7 +266,7 @@ class _objref_Method(cxx.Method):
 
       # Special ugly case. If the IDL says something like (in foo::bar
       # bar), the parameter name may be the same as the relative type
-      # name. We mangly the parameter name if this happens.
+      # name. We mangle the parameter name if this happens.
 
       typeBase = pType.base(environment)
       ident    = id.mapID(p.identifier())
@@ -230,7 +278,7 @@ class _objref_Method(cxx.Method):
       
     # an operation has optional context
     if self.callable().contexts() != []:
-      param_types.append("CORBA::Context_ptr")
+      param_types.append("::CORBA::Context_ptr")
       param_names.append("_ctxt")
 
     self._arg_types = param_types
@@ -256,7 +304,15 @@ class I_Helper(Class):
     if config.state['BOA Skeletons']:
       class_sk_name = "class " + \
                       self.interface().name().prefix("_sk_").simple() + ";"
-    stream.out(omniidl_be.cxx.header.template.interface_Helper,
+
+    if self.interface().abstract():
+      helper_tpl = omniidl_be.cxx.header.template.abstract_interface_Helper
+    elif self.interface().local():
+      helper_tpl = omniidl_be.cxx.header.template.local_interface_Helper
+    else:
+      helper_tpl = omniidl_be.cxx.header.template.interface_Helper
+
+    stream.out(helper_tpl,
                class_sk_name = class_sk_name,
                name = self.interface().name().simple(),
                guard = self.interface().name().guard())
@@ -266,30 +322,173 @@ class I_Helper(Class):
                name = self.interface().name().fullyQualify())
 
 
+class I(Class):
+  def __init__(self, i, other_idl):
+    Class.__init__(self, i)
+    self._other_idl = other_idl
+
+  def hh(self, stream):
+    if self.interface().abstract():
+
+      for callable in self.interface().callables():
+        method = _impl_Method(callable, self)
+        self._methods.append(method)
+        self._callables[method] = callable
+
+      methodl = []
+      for method in self.methods():
+          methodl.append(method.hh(virtual = 1, pure = 1))
+
+      inh = self.interface().inherits()
+      if inh:
+        inheritl = []
+        for i in inh:
+          iname = i.name().unambiguous(self._environment)
+          inheritl.append("public virtual " + iname)
+      else:
+        inheritl = [ "public virtual ::CORBA::AbstractBase" ]
+
+      stream.out(omniidl_be.cxx.header.template.abstract_interface_type,
+                 name=self.interface().name().simple(),
+                 Other_IDL = self._other_idl,
+                 inherits = string.join(inheritl, ",\n"),
+                 operations = string.join(methodl, "\n"))
+
+    elif self.interface().local():
+      abstract_base = 0
+      for i in self.interface().allInherits():
+        if i.abstract():
+          abstract_base = 1
+          break
+
+      if abstract_base:
+        abstract_narrows = omniidl_be.cxx.header.template.\
+                           interface_abstract_narrows
+      else:
+        abstract_narrows = ""
+
+      for callable in self.interface().callables():
+        method = _impl_Method(callable, self)
+        self._methods.append(method)
+        self._callables[method] = callable
+
+      # Override methods from inherited non-local interfaces
+      for i in self.interface().allInherits():
+        if not i.local():
+          for c in i.callables():
+            # Make new callable with ref to this interface instead of
+            # inherited interface
+            callable = call.Callable(self.interface(),
+                                     c.operation_name(),
+                                     c.method_name(),
+                                     c.returnType(),
+                                     c.parameters(),
+                                     c.oneway(),
+                                     c.raises(),
+                                     c.contexts())
+
+            method = _impl_Method(callable, self)
+            self._methods.append(method)
+            self._callables[method] = callable
+
+      bmethodl = []
+      nmethodl = []
+      for method in self.methods():
+          bmethodl.append(method.hh(virtual = 1, pure = 1))
+          nmethodl.append(method.hh(virtual = 1, pure = 0))
+
+      local_base = 0
+      inheritl   = []
+      ninheritl  = []
+
+      for i in self.interface().inherits():
+        if i.local():
+          local_base = 1
+          iname = i.name().unambiguous(self._environment)
+          niname = i.name().prefix("_nil_").unambiguous(self._environment)
+          ninheritl.append(niname)
+        else:
+          iname = i.name().prefix("_objref_").unambiguous(self._environment)
+
+        inheritl.append("public virtual " + iname)
+
+      if not local_base:
+        inheritl.append("public virtual ::CORBA::LocalObject")
+
+      if ninheritl:
+        nil_inherits = string.join(ninheritl, ",\n") + ","
+      else:
+        nil_inherits = ""
+
+      stream.out(omniidl_be.cxx.header.template.local_interface_type,
+                 name=self.interface().name().simple(),
+                 abstract_narrows = abstract_narrows,
+                 Other_IDL = self._other_idl,
+                 inherits = string.join(inheritl, ",\n"),
+                 operations = string.join(bmethodl, "\n"),
+                 nil_operations = string.join(nmethodl, "\n"),
+                 nil_inherits = nil_inherits)
+
+    else:
+      abstract_base = 0
+      for i in self.interface().allInherits():
+        if i.abstract():
+          abstract_base = 1
+          break
+
+      if abstract_base:
+        abstract_narrows = omniidl_be.cxx.header.template.\
+                           interface_abstract_narrows
+      else:
+        abstract_narrows = ""
+
+      stream.out(omniidl_be.cxx.header.template.interface_type,
+                 name=self.interface().name().simple(),
+                 abstract_narrows = abstract_narrows,
+                 Other_IDL = self._other_idl)
+
+
+
 class _objref_I(Class):
   def __init__(self, I):
     Class.__init__(self, I)
     self._name = self._name.prefix("_objref_")
 
+    # In abstract interfaces, signatures are like _impl classes, not
+    # normal _objref classes.
+    if self.interface().abstract():
+      cls = _impl_Method
+    else:
+      cls = _objref_Method
+
     for callable in self.interface().callables():
-      method = _objref_Method(callable, self)
+      method = cls(callable, self)
       self._methods.append(method)
       self._callables[method] = callable
 
 
   def hh(self, stream):
     # build the inheritance list
+
     objref_inherits = []
-    for i in self.interface().inherits():
+
+    inh = self.interface().inherits()
+    if inh:
+      for i in inh:
         objref_inherited_name = i.name().prefix("_objref_")
         uname = objref_inherited_name.unambiguous(self._environment)
         objref_inherits.append("public virtual " + uname)
-
-    # if already inheriting, the base class will be present
-    # (transitivity of the inherits-from relation)
-    if self.interface().inherits() == []:
-        objref_inherits = [ "public virtual CORBA::Object, " + \
+    else:
+      if self.interface().abstract():
+        objref_inherits = [ "public virtual ::CORBA::_omni_AbstractBaseObjref",
                             "public virtual omniObjRef" ]
+      else:
+        objref_inherits = [ "public virtual ::CORBA::Object",
+                            "public virtual omniObjRef" ]
+
+    if self.interface().abstract():
+      objref_inherits.append("public virtual " +
+                             self.interface().name().simple())
 
     methods = []
     for method in self.methods():
@@ -297,7 +496,7 @@ class _objref_I(Class):
         methods.append(method.hh(virtual = 1, pure = 0))
       else:
         methods.append(method.hh())
-            
+
     if config.state['Shortcut']:
       shortcut = output.StringStream()
       shortcut.out(omniidl_be.cxx.header.template.interface_shortcut,
@@ -318,15 +517,31 @@ class _objref_I(Class):
   def cc(self, stream):
 
     def _ptrToObjRef_ptr(self = self, stream = stream):
+      has_abstract = self.interface().abstract()
+
       for i in self.interface().allInherits():
+        if i.abstract(): has_abstract = 1
+
         stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_ptr,
                    inherits_fqname = i.name().fullyQualify())
 
+      if has_abstract:
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_ptr,
+                   inherits_fqname = "::CORBA::AbstractBase")
+
     def _ptrToObjRef_str(self = self, stream = stream):
+      has_abstract = self.interface().abstract()
+
       for i in self.interface().allInherits():
+        if i.abstract(): has_abstract = 1
+
         stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_str,
                    inherits_fqname = i.name().fullyQualify())
 
+      if has_abstract:
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_str,
+                   inherits_fqname = "::CORBA::AbstractBase")
+
     # build the inherits list
     
     inherits_str_list = []
@@ -369,6 +584,12 @@ class _objref_I(Class):
     else:
       init_shortcut = ""
 
+    if config.state['Shortcut'] == 2:
+      release_shortcut = omniidl_be.cxx.skel.template.\
+                         interface_release_refcount_shortcut
+    else:
+      release_shortcut = ""
+
     stream.out(omniidl_be.cxx.skel.template.interface_objref,
                name = self.interface().name().fullyQualify(),
                fq_objref_name = self.name().fullyQualify(),
@@ -377,9 +598,12 @@ class _objref_I(Class):
                comma = comma,
                _ptrToObjRef_ptr = _ptrToObjRef_ptr,
                _ptrToObjRef_str = _ptrToObjRef_str,
-               init_shortcut = init_shortcut)
+               init_shortcut = init_shortcut,
+               release_shortcut = release_shortcut)
 
-    if config.state['Shortcut']:
+    shortcut_mode = config.state['Shortcut']
+
+    if shortcut_mode:
       inherited = output.StringStream()
       for i in self.interface().inherits():
         objref_name = i.name().prefix("_objref_")
@@ -391,8 +615,13 @@ class _objref_I(Class):
 
         inherited.out(omniidl_be.cxx.skel.template.interface_shortcut_inh,
                       parent=objref_str)
+
+      if shortcut_mode == 1:
+        tmpl = omniidl_be.cxx.skel.template.interface_shortcut
+      else:
+        tmpl = omniidl_be.cxx.skel.template.interface_shortcut_refcount
         
-      stream.out(omniidl_be.cxx.skel.template.interface_shortcut,
+      stream.out(tmpl,
                  name = self.interface().name().fullyQualify(),
                  basename = self.interface().name().simple(),
                  fq_objref_name = self.name().fullyQualify(),
@@ -436,8 +665,15 @@ class _objref_I(Class):
         objref_class = method.parent_class()
         interface = objref_class.interface()
         implname = interface.name().prefix("_impl_").unambiguous(self._environment)
+
+        if config.state['Shortcut'] == 2:
+          tmpl = omniidl_be.cxx.skel.template.\
+                 interface_operation_shortcut_refcount
+        else:
+          tmpl = omniidl_be.cxx.skel.template.\
+                 interface_operation_shortcut
         
-        body.out(omniidl_be.cxx.skel.template.interface_operation_shortcut,
+        body.out(tmpl,
                  impl_type = implname,
                  callreturn = callreturn,
                  voidreturn = voidreturn,
@@ -452,6 +688,103 @@ class _objref_I(Class):
                                      self._environment)
       method.cc(stream, body)
 
+
+class _nil_I(Class):
+  def __init__(self, I):
+    Class.__init__(self, I)
+    self._name = self._name.prefix("_nil_")
+
+    for callable in self.interface().callables():
+      method = _impl_Method(callable, self)
+      self._methods.append(method)
+      self._callables[method] = callable
+
+    for i in self.interface().allInherits():
+      if not i.local():
+        for c in i.callables():
+          # Make new callable with ref to this interface instead of
+          # inherited interface
+          callable = call.Callable(self.interface(),
+                                   c.operation_name(),
+                                   c.method_name(),
+                                   c.returnType(),
+                                   c.parameters(),
+                                   c.oneway(),
+                                   c.raises(),
+                                   c.contexts())
+
+          method = _impl_Method(callable, self)
+          self._methods.append(method)
+          self._callables[method] = callable
+
+  def hh(self, stream):
+    pass
+
+  def cc(self, stream):
+
+    def _ptrToObjRef_ptr(self = self, stream = stream):
+      has_abstract = 0
+
+      for i in self.interface().allInherits():
+        if i.abstract(): has_abstract = 1
+
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_ptr,
+                   inherits_fqname = i.name().fullyQualify())
+
+      stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_ptr,
+                 inherits_fqname = "::CORBA::LocalObject")
+
+      if has_abstract:
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_ptr,
+                   inherits_fqname = "::CORBA::AbstractBase")
+
+    def _ptrToObjRef_str(self = self, stream = stream):
+      has_abstract = 0
+
+      for i in self.interface().allInherits():
+        if i.abstract(): has_abstract = 1
+
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_str,
+                   inherits_fqname = i.name().fullyQualify())
+
+      stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_str,
+                 inherits_fqname = "::CORBA::LocalObject")
+
+      if has_abstract:
+        stream.out(omniidl_be.cxx.skel.template.interface_objref_repoID_str,
+                   inherits_fqname = "::CORBA::AbstractBase")
+
+    stream.out(omniidl_be.cxx.skel.template.local_interface_objref,
+               name = self.interface().name().fullyQualify(),
+               sname = self.interface().name().simple(),
+               fq_nil_name = self.name().fullyQualify(),
+               nil_name = self.name().simple(),
+               _ptrToObjRef_ptr = _ptrToObjRef_ptr,
+               _ptrToObjRef_str = _ptrToObjRef_str)
+
+    for method in self.methods():
+      callable = self._callables[method]
+
+      # signature is a text string form of the complete operation signature
+      signature = callable.signature()
+
+      # produce member function for this operation/attribute.
+      body = output.StringStream()
+
+      argnames = method.arg_names()
+
+      body.out(omniidl_be.cxx.skel.template.local_interface_nil_operation)
+
+      if not method.return_type().void():
+        body.out(omniidl_be.cxx.skel.template.local_interface_nil_dummy_return,
+                 method = method.name(),
+                 args   = string.join(method.arg_names(),", "))
+      
+      method.cc(stream, body)
+      stream.out("\n")
+
+
+
 class _pof_I(Class):
   def __init__(self, I):
     Class.__init__(self, I)
@@ -493,7 +826,12 @@ class _impl_I(Class):
     # build the inheritance list
     environment = self._environment
     impl_inherits = []
+    inherit_abstract = 0
+    
     for i in self.interface().inherits():
+      if i.abstract():
+        inherit_abstract = 1
+
       impl_inherited_name = i.name().prefix("_impl_")
       uname = impl_inherited_name.unambiguous(environment)
       impl_inherits.append("public virtual " + uname)
@@ -503,15 +841,22 @@ class _impl_I(Class):
     if self.interface().inherits() == []:
       impl_inherits   = [ "public virtual omniServant" ]
 
-
     methods = []
     for method in self.methods():
         methods.append(method.hh(virtual = 1, pure = 1))
+
+    if self.interface().abstract():
+      abstract = omniidl_be.cxx.header.template.interface_impl_abstract
+    elif inherit_abstract:
+      abstract = omniidl_be.cxx.header.template.interface_impl_not_abstract
+    else:
+      abstract = ""
         
     stream.out(omniidl_be.cxx.header.template.interface_impl,
                name = self.interface().name().simple(),
                inherits = string.join(impl_inherits, ",\n"),
-               operations = string.join(methods, "\n"))
+               operations = string.join(methods, "\n"),
+               abstract = abstract)
 
   def cc(self, stream):
 
@@ -545,11 +890,13 @@ class _impl_I(Class):
       for i in self.interface().allInherits():
         inherited_name = i.name()
         impl_inherited_name = inherited_name.prefix("_impl_")
-        inherited_str = inherited_name.unambiguous(self._environment)
-        impl_inherited_str = impl_inherited_name.unambiguous(self._environment)
-        if inherited_name.needFlatName(self._environment):
-          inherited_str = inherited_name.flatName()
-          impl_inherited_str = impl_inherited_name.flatName()
+
+        # HERE: using the fully scoped name may fail on old MSVC
+        # versions, but it is required by newer MSVC versions.
+        # Marvellous.
+        inherited_str      = inherited_name.fullyQualify()
+        impl_inherited_str = impl_inherited_name.fullyQualify()
+
         stream.out(omniidl_be.cxx.skel.template.interface_impl_repoID_ptr,
                    inherited_name = inherited_str,
                    impl_inherited_name = impl_inherited_str)
@@ -558,11 +905,10 @@ class _impl_I(Class):
       for i in self.interface().allInherits():
         inherited_name = i.name()
         impl_inherited_name = inherited_name.prefix("_impl_")
-        inherited_str = inherited_name.unambiguous(self._environment)
-        impl_inherited_str = impl_inherited_name.unambiguous(self._environment)
-        if inherited_name.needFlatName(self._environment):
-          inherited_str = inherited_name.flatName()
-          impl_inherited_str = impl_inherited_name.flatName()
+
+        inherited_str      = inherited_name.fullyQualify()
+        impl_inherited_str = impl_inherited_name.fullyQualify()
+
         stream.out(omniidl_be.cxx.skel.template.interface_impl_repoID_str,
                    inherited_name = inherited_str,
                    impl_inherited_name = impl_inherited_str)
@@ -583,6 +929,17 @@ class _impl_I(Class):
                _ptrToInterface_ptr = _ptrToInterface_ptr,
                _ptrToInterface_str = _ptrToInterface_str,
                name = node_name.fullyQualify())
+
+    if not self.interface().abstract():
+      # Are we derived from an abstract interface?
+      inherit_abstract = 0
+      for i in self.interface().inherits():
+        if i.abstract():
+          inherit_abstract = 1
+          break
+      if inherit_abstract:
+        stream.out(omniidl_be.cxx.skel.template.interface_impl_not_abstract,
+                   impl_fqname = impl_name.fullyQualify())
           
 
 class _sk_I(Class):
