@@ -28,6 +28,18 @@
 
 /*
   $Log: cs-UTF-8.cc,v $
+  Revision 1.1.4.4  2006/04/28 18:40:46  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.4.3  2005/12/08 14:22:31  dgrisby
+  Better string marshalling performance; other minor optimisations.
+
+  Revision 1.1.4.2  2005/01/06 23:10:14  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.4.1  2003/03/23 21:02:20  dgrisby
+  Start of omniORB 4.1.x development branch.
+
   Revision 1.1.2.13  2001/10/17 16:47:09  dpg1
   New minor codes
 
@@ -108,7 +120,7 @@ public:
 
   // Unicode based marshalling
   void marshalChar  (cdrStream& stream, omniCodeSet::UniChar uc);
-  void marshalString(cdrStream& stream,
+  void marshalString(cdrStream& stream, _CORBA_ULong bound,
 		     _CORBA_ULong len, const omniCodeSet::UniChar* us);
 
   omniCodeSet::UniChar unmarshalChar(cdrStream& stream);
@@ -138,9 +150,9 @@ public:
 				     _CORBA_ULong&       length,
 				     char*&              s);
 
-  TCS_C_UTF_8()
+  TCS_C_UTF_8(GIOP::Version v)
     : omniCodeSet::TCS_C(omniCodeSet::ID_UTF_8, "UTF-8",
-			 omniCodeSet::CS_Other, omniCodeSetUtil::GIOP12)
+			 omniCodeSet::CS_Other, v)
   { }
 
   virtual ~TCS_C_UTF_8() {}
@@ -277,9 +289,7 @@ NCS_C_UTF_8::marshalString(cdrStream& stream, omniCodeSet::TCS_C* tcs,
     // processors. Cases 3, 2 and 1 should drop through with no
     // branching code.
     switch (bytes) {
-    case 6: OMNIORB_THROW(DATA_CONVERSION, 
-			  DATA_CONVERSION_BadInput,
-			  (CORBA::CompletionStatus)stream.completion());
+    case 6:
     case 5:
     case 4: OMNIORB_THROW(DATA_CONVERSION, 
 			  DATA_CONVERSION_BadInput,
@@ -309,8 +319,9 @@ NCS_C_UTF_8::marshalString(cdrStream& stream, omniCodeSet::TCS_C* tcs,
     }
   }
   // Null terminator
+  len = ub.length();
   ub.insert(0);
-  tcs->marshalString(stream, ub.length() - 1, ub.buffer());
+  tcs->marshalString(stream, bound, len, ub.buffer());
 }
 
 
@@ -395,8 +406,8 @@ NCS_C_UTF_8::unmarshalString(cdrStream& stream, omniCodeSet::TCS_C* tcs,
     }
     else if (uc < 0xe000) {
       // Second half of surrogate pair not allowed on its own
-      OMNIORB_THROW(BAD_INV_ORDER, 
-		    BAD_INV_ORDER_CodeSetNotKnownYet,
+      OMNIORB_THROW(DATA_CONVERSION,
+		    DATA_CONVERSION_BadInput,
 		    (CORBA::CompletionStatus)stream.completion());
     }
     else {
@@ -431,6 +442,7 @@ TCS_C_UTF_8::marshalChar(cdrStream& stream, omniCodeSet::UniChar uc)
 
 void
 TCS_C_UTF_8::marshalString(cdrStream& stream,
+			   _CORBA_ULong bound,
 			   _CORBA_ULong len,
 			   const omniCodeSet::UniChar* us)
 {
@@ -488,6 +500,11 @@ TCS_C_UTF_8::marshalString(cdrStream& stream,
     }
   }
   _CORBA_ULong mlen = b.length();
+
+  if (bound && mlen-1 > bound)
+    OMNIORB_THROW(MARSHAL, MARSHAL_StringIsTooLong, 
+		  (CORBA::CompletionStatus)stream.completion());
+
   mlen >>= stream;
   stream.put_octet_array((const _CORBA_Octet*)b.buffer(), mlen);
 }
@@ -517,19 +534,16 @@ TCS_C_UTF_8::unmarshalString(cdrStream& stream,
 
   if (len == 0) {
     if (orbParameters::strictIIOP) {
-      if (omniORB::trace(1)) {
-	omniORB::logger l;
-	l << "Error: received an invalid zero length string.\n";
-      }
+      omniORB::logs(1, "Error: received an invalid zero length string.");
       OMNIORB_THROW(MARSHAL, MARSHAL_StringNotEndWithNull, 
 		    (CORBA::CompletionStatus)stream.completion());
     }
     else {
-      if (omniORB::trace(1)) {
-	omniORB::logger l;
-	l << "Warning: received an invalid zero length string."
-	  << " Substituted with a proper empty string.\n";
-      }
+      omniORB::logs(1, "Warning: received an invalid zero length string. "
+		    "Substituted with a proper empty string.");
+      us = omniCodeSetUtil::allocU(1);
+      us[0] = 0;
+      return 0;
     }
   }
 
@@ -624,14 +638,22 @@ TCS_C_UTF_8::fastMarshalString(cdrStream&          stream,
 			       const char*         s)
 {
   if (ncs->id() == id()) { // Null transformation
-    if (bound && len > bound)
-      OMNIORB_THROW(MARSHAL, MARSHAL_StringIsTooLong, 
-		    (CORBA::CompletionStatus)stream.completion());
+    if (len == 0) {
+      len = stream.marshalRawString(s);
 
+      if (bound && len-1 > bound)
+	OMNIORB_THROW(MARSHAL, MARSHAL_StringIsTooLong, 
+		      (CORBA::CompletionStatus)stream.completion());
+    }
+    else {
+      if (bound && len > bound)
+	OMNIORB_THROW(MARSHAL, MARSHAL_StringIsTooLong, 
+		      (CORBA::CompletionStatus)stream.completion());
 
-    _CORBA_ULong mlen = len + 1;
-    mlen >>= stream;
-    stream.put_octet_array((const _CORBA_Octet*)s, mlen);
+      _CORBA_ULong mlen = len + 1;
+      mlen >>= stream;
+      stream.put_octet_array((const _CORBA_Octet*)s, mlen);
+    }
     return 1;
   }
   else if (ncs->kind() == omniCodeSet::CS_8bit) { // Simple 8 bit code set
@@ -706,19 +728,16 @@ TCS_C_UTF_8::fastUnmarshalString(cdrStream&          stream,
 
     if (mlen == 0) {
       if (orbParameters::strictIIOP) {
-	if (omniORB::trace(1)) {
-	  omniORB::logger l;
-	  l << "Error: received an invalid zero length string.\n";
-	}
+	omniORB::logs(1, "Error: received an invalid zero length string.");
 	OMNIORB_THROW(MARSHAL, MARSHAL_StringNotEndWithNull, 
 		      (CORBA::CompletionStatus)stream.completion());
       }
       else {
-	if (omniORB::trace(1)) {
-	  omniORB::logger l;
-	  l << "Warning: received an invalid zero length string."
-	    << " Substituted with a proper empty string.\n";
-	}
+	omniORB::logs(1, "Warning: received an invalid zero length string. "
+		      "Substituted with a proper empty string.");
+	s = omniCodeSetUtil::allocC(1);
+	s[0] = '\0';
+	return 0;
       }
     }
 
@@ -750,19 +769,16 @@ TCS_C_UTF_8::fastUnmarshalString(cdrStream&          stream,
 
     if (mlen == 0) {
       if (orbParameters::strictIIOP) {
-	if (omniORB::trace(1)) {
-	  omniORB::logger l;
-	  l << "Error: received an invalid zero length string.\n";
-	}
+	omniORB::logs(1, "Error: received an invalid zero length string.");
 	OMNIORB_THROW(MARSHAL, MARSHAL_StringNotEndWithNull, 
 		      (CORBA::CompletionStatus)stream.completion());
       }
       else {
-	if (omniORB::trace(1)) {
-	  omniORB::logger l;
-	  l << "Warning: received an invalid zero length string."
-	    << " Substituted with a proper empty string.\n";
-	}
+	omniORB::logs(1, "Warning: received an invalid zero length string. "
+		      "Substituted with a proper empty string.");
+	s = omniCodeSetUtil::allocC(1);
+	s[0] = '\0';
+	return 0;
       }
     }
 
@@ -837,13 +853,15 @@ TCS_C_UTF_8::fastUnmarshalString(cdrStream&          stream,
 //
 
 static NCS_C_UTF_8 _NCS_C_UTF_8;
-static TCS_C_UTF_8 _TCS_C_UTF_8;
+static TCS_C_UTF_8 _TCS_C_UTF_8_11(omniCodeSetUtil::GIOP11);
+static TCS_C_UTF_8 _TCS_C_UTF_8_12(omniCodeSetUtil::GIOP12);
 
 class CS_UTF_8_init {
 public:
   CS_UTF_8_init() {
     omniCodeSet::registerNCS_C(&_NCS_C_UTF_8);
-    omniCodeSet::registerTCS_C(&_TCS_C_UTF_8);
+    omniCodeSet::registerTCS_C(&_TCS_C_UTF_8_11);
+    omniCodeSet::registerTCS_C(&_TCS_C_UTF_8_12);
   }
 };
 
