@@ -29,8 +29,26 @@
 
 /*
   $Log: giopImpl11.cc,v $
-  Revision 1.1.4.20  2003/07/25 16:07:18  dgrisby
-  Incorrect COMM_FAILURE with GIOP 1.2 CloseConnection.
+  Revision 1.1.6.7  2006/09/20 13:36:31  dgrisby
+  Descriptive logging for connection and GIOP errors.
+
+  Revision 1.1.6.6  2006/09/17 23:23:16  dgrisby
+  Wrong offsets with indirections spanning GIOP fragments.
+
+  Revision 1.1.6.5  2006/06/05 11:28:04  dgrisby
+  Change clientSendRequest interceptor members to a single GIOP_C.
+
+  Revision 1.1.6.4  2005/12/08 14:22:31  dgrisby
+  Better string marshalling performance; other minor optimisations.
+
+  Revision 1.1.6.3  2005/04/11 12:09:42  dgrisby
+  Another merge.
+
+  Revision 1.1.6.2  2005/01/06 23:10:15  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.6.1  2003/03/23 21:02:15  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.4.19  2003/01/22 11:40:12  dgrisby
   Correct serverSendException interceptor use.
@@ -162,13 +180,15 @@ public:
 
   static void inputSkipWholeMessage(giopStream* g);
 
-  static void inputTerminalProtocolError(giopStream* g);
+  static void inputTerminalProtocolError(giopStream* g,
+					 const char* file, int line,
+					 const char* message);
   // Helper function.  Call this function to indicate that a protocol
   // voilation was detected.  This function *always* raise a
   // giopStream::CommFailure exception.  Therefore the caller should not
   // expect this function to return.
 
-  static void inputRaiseCommFailure(giopStream* g);
+  static void inputRaiseCommFailure(giopStream* g, const char* message);
 
   static void outputNewMessage(giopStream* g);
 
@@ -222,6 +242,7 @@ giopImpl11::inputMessageBegin(giopStream* g,
       // This is a GIOP 1.0 message, switch to the implementation of giop 1.0
       // and dispatch again.
       GIOP::Version v = { 1, 0 };
+      ((giopStrand &)*g).version = v;
       g->impl(giopStreamImpl::matchVersion(v));
       OMNIORB_ASSERT(g->impl());
       g->impl()->inputMessageBegin(g,g->impl()->unmarshalWildCardRequestHeader);
@@ -229,7 +250,8 @@ giopImpl11::inputMessageBegin(giopStream* g,
     }
     // We accept a CloseConnection message with any GIOP version.
     if ((GIOP::MsgType)hdr[7] != GIOP::CloseConnection) {
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Invalid GIOP message version");
       // never reaches here.
     }
   }
@@ -290,17 +312,20 @@ giopImpl11::inputReplyBegin(giopStream* g,
     {
       CORBA::ULong minor;
       CORBA::Boolean retry;
+      g->pd_strand->orderly_closed = 1;
       g->notifyCommFailure(0,minor,retry);
       g->pd_strand->state(giopStrand::DYING);
-      g->pd_strand->orderly_closed = 1;
       giopStream::CommFailure::_raise(minor,
 				      CORBA::COMPLETED_NO,
-				      retry,__FILE__,__LINE__);
+				      retry,__FILE__,__LINE__,
+				      "Orderly connection shutdown",
+				      g->pd_strand);
       // never reach here.
       break;
     }
   default:
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Unknown GIOP message type");
     // never reaches here.
   }
 
@@ -311,13 +336,15 @@ giopImpl11::inputReplyBegin(giopStream* g,
     if (mtype == GIOP::LocateReply && 
 	unmarshalHeader != unmarshalLocateReply) {
 
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Unexpected LocateReply");
       // never reach here
     }
     else if (mtype == GIOP::Reply &&
 	     unmarshalHeader != unmarshalReplyHeader) {
 
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Unexpected Reply");
       // never reach here
     }
 
@@ -344,7 +371,8 @@ giopImpl11::inputReplyBegin(giopStream* g,
 
 	if (target->inputMatchedId()) {
 	  // a reply has already been received!
-	  inputTerminalProtocolError(g);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__,
+				     "Duplicate Reply");
 	}
 
 	target->pd_input = source->pd_input;
@@ -436,7 +464,8 @@ giopImpl11::inputMessageEnd(giopStream* g,CORBA::Boolean disgard) {
 	disgard = 1;
       }
       else {
-	inputTerminalProtocolError(g);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Garbage left at end of input message");
 	// never reach here.
       }
     }
@@ -466,7 +495,9 @@ giopImpl11::unmarshalReplyHeader(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::Reply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "Reply");
     // Never reach here.
   }
 
@@ -507,7 +538,8 @@ giopImpl11::unmarshalReplyHeader(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in Reply");
     // Never reach here.
     break;
   }
@@ -529,7 +561,9 @@ giopImpl11::unmarshalLocateReply(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::LocateReply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "LocateReply");
     // Never reach here.
   }
 
@@ -551,7 +585,8 @@ giopImpl11::unmarshalLocateReply(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in LocateReply");
     // Never reach here.
     break;
   }
@@ -574,10 +609,11 @@ giopImpl11::unmarshalWildCardRequestHeader(giopStream* g) {
   case GIOP::CancelRequest:
     break;
   case GIOP::CloseConnection:
-    inputRaiseCommFailure(g);
+    inputRaiseCommFailure(g, "Orderly connection shutdown");
     break;
   default:
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Unknown GIOP message type");
     // Never reach here.
     break;
   }
@@ -707,15 +743,26 @@ giopImpl11::inputNewFragment(giopStream* g) {
                      g->pd_currentInputBuffer->start;
 
   if (hdr[4] != 1 || hdr[5] != 1) {
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Received a non GIOP 1.1 message when "
+			       "expecting a GIOP 1.1 Fragment");
     // never reach here.
   }
       
   CORBA::Boolean bswap = (((hdr[6] & 0x1) == _OMNIORB_HOST_BYTE_ORDER_)
 			  ? 0 : 1 );
 
-  if (hdr[7] != (char)GIOP::Fragment || bswap != g->pd_unmarshal_byte_swap) {
-    inputTerminalProtocolError(g);
+  if (hdr[7] != (char)GIOP::Fragment) {
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "Fragment");
+    // never reach here
+  }
+    
+  if (bswap != g->pd_unmarshal_byte_swap) {
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Fragment has different byte ordering to "
+			       "initial message");
     // never reach here
   }
 
@@ -760,7 +807,9 @@ giopImpl11::getInputData(giopStream* g,omni::alignment_t align,size_t sz) {
 
 	if (g->inputExpectAnotherFragment()) {
 	  // The incoming message is fragmented at the wrong boundary!!!
-	  inputTerminalProtocolError(g);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__,
+				     "Message fragmented at incorrect "
+				     "boundary");
 	  // never reach here
 	}
 	// Very bad. Should never happen given our invariant.
@@ -875,7 +924,9 @@ giopImpl11::copyInputData(giopStream* g,void* b, size_t sz,
 
       if (g->inputExpectAnotherFragment()) {
 	// The incoming message is fragmented at the wrong boundary!!!
-	inputTerminalProtocolError(g);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Message fragmented at incorrect "
+				   "boundary (bulk copy)");
 	// never reach here
       }
       // Very bad. Should never happen given our invariant.
@@ -1002,21 +1053,24 @@ giopImpl11::currentInputPtr(const giopStream* g) {
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl11::inputTerminalProtocolError(giopStream* g) {
+giopImpl11::inputTerminalProtocolError(giopStream* g,
+				       const char* file, int line,
+				       const char* message) {
 
   // XXX We may choose to send a message error to the other end.
   if (omniORB::trace(1)) {
     omniORB::logger l;
     l << "From endpoint: " << g->pd_strand->connection->peeraddress()
       <<". Detected GIOP 1.1 protocol error in input message. "
+      << omniExHelper::strip(file) << ":" << line
       << "Connection is closed.\n";
   }
-  inputRaiseCommFailure(g);
+  inputRaiseCommFailure(g, message);
 }
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl11::inputRaiseCommFailure(giopStream* g) {
+giopImpl11::inputRaiseCommFailure(giopStream* g, const char* message) {
 
   CORBA::ULong minor;
   CORBA::Boolean retry;
@@ -1024,7 +1078,7 @@ giopImpl11::inputRaiseCommFailure(giopStream* g) {
   g->pd_strand->state(giopStrand::DYING);
   giopStream::CommFailure::_raise(minor,
 				  (CORBA::CompletionStatus)g->completion(),
-				  0,__FILE__,__LINE__);
+				  0,__FILE__,__LINE__,message,g->pd_strand);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -1158,11 +1212,7 @@ giopImpl11::marshalRequestHeader(giopStream* g) {
   omniCallDescriptor& calldesc = *giop_c.calldescriptor();
   CORBA::Boolean response_expected = !calldesc.is_oneway();
 
-  omniInterceptors::clientSendRequest_T::info_T info(*g,
-				                     *(giop_c.ior()),
-						     calldesc.op(),
-						     response_expected,
-						     !response_expected);
+  omniInterceptors::clientSendRequest_T::info_T info(giop_c);
   omniInterceptorP::visit(info);
 
   // service context
@@ -1176,11 +1226,11 @@ giopImpl11::marshalRequestHeader(giopStream* g) {
 
   // object key
   giop_c.keysize() >>= s;
-  s.put_octet_array(giop_c.key(),giop_c.keysize());
+  s.put_small_octet_array(giop_c.key(),giop_c.keysize());
 
   // operation
   operator>>= ((CORBA::ULong) calldesc.op_len(),s);
-  s.put_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
+  s.put_small_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
 
   // principal
   omni::myPrincipalID >>= s;
@@ -1215,7 +1265,7 @@ giopImpl11::sendLocateRequest(giopStream* g) {
   
   // object key
   giop_c.keysize() >>= s;
-  s.put_octet_array(giop_c.key(),giop_c.keysize());
+  s.put_small_octet_array(giop_c.key(),giop_c.keysize());
 
   outputMessageEnd(g);
 }
@@ -1270,12 +1320,17 @@ giopImpl11::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
     giop_s.notifyCommFailure(0,minor,retry);
     giopStream::CommFailure::_raise(minor,(CORBA::CompletionStatus)
 				    giop_s.completion(),
-				    retry,__FILE__,__LINE__);
+				    retry,__FILE__,__LINE__,
+				    "System Exception occurred while "
+				    "marshalling reply. Sending a "
+				    "MessageError", g->pd_strand);
   }
 
   int repoid_size;
   const char* repoid = ex._NP_repoId(&repoid_size);
 
+  outputNewMessage(g);
+
   char* hdr = (char*) g->pd_currentOutputBuffer + 
                       g->pd_currentOutputBuffer->start;
 
@@ -1319,7 +1374,7 @@ giopImpl11::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
 
   // system exception value
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
   ex.minor() >>= s;
   CORBA::ULong(ex.completed()) >>= s;
 
@@ -1378,7 +1433,7 @@ giopImpl11::sendUserException(giopStream* g,const CORBA::UserException& ex) {
   rc >>= s;
 
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
   ex._NP_marshal(s);
 
   outputMessageEnd(g);
@@ -1542,7 +1597,7 @@ giopImpl11::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
   else {
     CORBA::Long msz = g->outputMessageSize();
     if (msz) {
-      g->outputMessageSize(msz+fsz);
+      g->outputMessageSize(msz+fsz-12);
     }
     else {
       g->outputMessageSize(fsz-12);
@@ -1586,7 +1641,14 @@ giopImpl11::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
     avail = (avail ? avail - 4 : 0);
 
     omni::ptr_arith_t newmkr = (omni::ptr_arith_t) g->pd_outb_mkr + avail;
-    if (newmkr < (omni::ptr_arith_t)g->pd_outb_end) {
+
+    // If the new position is inside the buffer, set the end pointer.
+    // Note that if avail is very large, newmkr may wrap around and be
+    // < pd_outb_mkr.
+
+    if ((newmkr >= (omni::ptr_arith_t)g->pd_outb_mkr &&
+	 newmkr <  (omni::ptr_arith_t)g->pd_outb_end)) {
+
       g->pd_outb_end = (void*) newmkr;
     }
   }
@@ -1703,12 +1765,7 @@ giopImpl11::currentOutputPtr(const giopStream* g) {
                      ((omni::ptr_arith_t) g->pd_currentOutputBuffer + 
 		      g->pd_currentOutputBuffer->start);
 
-  if (g->outputMessageSize()) {
-    return fsz + g->outputMessageSize();
-  }
-  else {
-    return fsz - 12;
-  }
+  return g->outputMessageSize() + fsz - 12;
 }
 
 
