@@ -29,36 +29,42 @@
 
 /*
   $Log: giopImpl12.cc,v $
-  Revision 1.1.4.30  2005/10/17 15:49:10  dgrisby
-  Missing newline at end of log message.
+  Revision 1.1.6.12  2007/04/05 15:06:45  dgrisby
+  Handle CancelRequest without starting a new giopWorker.
 
-  Revision 1.1.4.29  2005/04/10 22:17:19  dgrisby
-  Fixes to connection management. Thanks Jon Biggar.
+  Revision 1.1.6.11  2006/11/09 15:37:45  dgrisby
+  Remove duplicate calls to retrieve addressing mode.
 
-  Revision 1.1.4.28  2005/02/02 00:21:07  dgrisby
-  Memory leak with CloseConnection or error received in a queued message.
+  Revision 1.1.6.10  2006/09/20 13:36:31  dgrisby
+  Descriptive logging for connection and GIOP errors.
 
-  Revision 1.1.4.27  2004/10/18 00:23:54  dgrisby
-  Bug in trying to set up bidirectional GIOP with GIOP 1.0 and 1.1.
+  Revision 1.1.6.9  2006/09/17 23:23:16  dgrisby
+  Wrong offsets with indirections spanning GIOP fragments.
 
-  Revision 1.1.4.26  2004/07/01 19:16:23  dgrisby
-  Client call interceptor oneway and response expected flipped. Thanks
-  John Fardo.
+  Revision 1.1.6.8  2006/09/01 14:11:52  dgrisby
+  Avoid potential deadlock with call buffering; do not force worker
+  creation when a request is fully buffered.
 
-  Revision 1.1.4.25  2004/06/18 14:45:24  dgrisby
-  Obscure and rare bug in GIOP 1.2 system exception sending.
+  Revision 1.1.6.7  2006/06/05 11:28:04  dgrisby
+  Change clientSendRequest interceptor members to a single GIOP_C.
 
-  Revision 1.1.4.24  2004/05/25 14:02:22  dgrisby
-  Properly close bidirectional connections.
+  Revision 1.1.6.6  2005/12/08 14:22:31  dgrisby
+  Better string marshalling performance; other minor optimisations.
 
-  Revision 1.1.4.23  2004/02/06 16:17:45  dgrisby
-  Properly handle large giopMaxMsgSize settings.
+  Revision 1.1.6.5  2005/11/17 17:03:26  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.4.22  2003/11/19 10:42:09  dgrisby
-  Locking bug with comm failure in bidirectional GIOP.
+  Revision 1.1.6.4  2005/04/11 12:09:42  dgrisby
+  Another merge.
 
-  Revision 1.1.4.21  2003/07/16 14:22:38  dgrisby
-  Speed up oneway handling a little. More tracing for split messages.
+  Revision 1.1.6.3  2005/03/02 12:39:18  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.6.2  2005/01/06 23:10:16  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.6.1  2003/03/23 21:02:15  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.4.20  2003/01/22 11:40:12  dgrisby
   Correct serverSendException interceptor use.
@@ -199,13 +205,14 @@ public:
   static void inputSkipWholeMessage(giopStream* g);
 
   static void inputTerminalProtocolError(giopStream* g,
-					 const char* file, int line);
+					 const char* file, int line,
+					 const char* message);
   // Helper function.  Call this function to indicate that a protocol
   // voilation was detected.  This function *always* raise a
   // giopStream::CommFailure exception.  Therefore the caller should not
   // expect this function to return.
 
-  static void inputRaiseCommFailure(giopStream* g);
+  static void inputRaiseCommFailure(giopStream* g, const char* message);
 
   static void outputNewMessage(giopStream* g);
 
@@ -221,6 +228,15 @@ private:
   giopImpl12& operator=(const giopImpl12&);
 };
 
+////////////////////////////////////////////////////////////////////////
+CORBA::Boolean
+giopImpl12::outputHasReachedLimit(giopStream* g) {
+
+  return ( (omni::ptr_arith_t) g->pd_outb_end == 
+          ((omni::ptr_arith_t) g->pd_currentOutputBuffer + 
+		  	       g->pd_currentOutputBuffer->start + 12) );
+}
+
 ////////////////////////////////////////////////////////////////////////
 void
 giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
@@ -248,7 +264,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     // We accept a CloseConnection message with any GIOP version.
     if ((GIOP::MsgType)hdr[7] != GIOP::CloseConnection) {
       giopStream_Buffer::deleteBuffer(b);
-      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Invalid GIOP message version");
       // never reach here
     }
   }
@@ -269,7 +286,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
   }
   else if ( mtype == GIOP::MessageError) {
     giopStream_Buffer::deleteBuffer(b);
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Received a MessageError message");
     // never reach here
   }
   else if ( g->pd_strand->isClient() || g->pd_strand->biDir) {
@@ -282,12 +300,15 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     g->pd_strand->state(giopStrand::DYING);
     giopStream::CommFailure::_raise(minor,
 				    CORBA::COMPLETED_NO,
-				    retry,__FILE__,__LINE__);
+				    retry,__FILE__,__LINE__,
+				    "Orderly connection shutdown",
+				    g->pd_strand);
     // never reach here
   }
   else {
     giopStream_Buffer::deleteBuffer(b);
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Orderly connection shutdown on server");
     // never reach here
   }
 
@@ -303,7 +324,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     if (!(g->pd_strand->isClient() || g->pd_strand->biDir)) {
       omniTransportLock->unlock();
       giopStream_Buffer::deleteBuffer(b);
-      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Server received an invalid reply message");
       // never reach here
     }
     // falls through
@@ -324,7 +346,9 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    // a reply has already been received!
 	    omniTransportLock->unlock();
 	    giopStream_Buffer::deleteBuffer(b);
-	    inputTerminalProtocolError(g, __FILE__, __LINE__);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__,
+				       "Message claims to belong to a "
+				       "completed reply");
 	    // never reach here
 	  }
 
@@ -333,7 +357,9 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	      // already got the header
 	      omniTransportLock->unlock();
 	      giopStream_Buffer::deleteBuffer(b);
-	      inputTerminalProtocolError(g, __FILE__, __LINE__);
+	      inputTerminalProtocolError(g, __FILE__, __LINE__,
+					 "Reply header has already been "
+					 "received");
 	      // never reach here
 	    }
 	  }
@@ -341,7 +367,9 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    // receive body before the header
 	    omniTransportLock->unlock();
 	    giopStream_Buffer::deleteBuffer(b);
-	    inputTerminalProtocolError(g, __FILE__, __LINE__);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__,
+				       "Reply header not seen for message "
+				       "fragment");
 	    // never reach here
 	  }
 	  else {
@@ -367,7 +395,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	  omniTransportLock->unlock();
 	  giopStream_Buffer::deleteBuffer(b);
-	  inputTerminalProtocolError(g, __FILE__, __LINE__);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__,
+				     "Request message received by client");
 	  // never reach here
 	}
 	// Make sure this request id has not been seen before
@@ -383,7 +412,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    // already have a request with the same id.
 	    omniTransportLock->unlock();
 	    giopStream_Buffer::deleteBuffer(b);
-	    inputTerminalProtocolError(g, __FILE__, __LINE__);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__,
+				       "Duplicate request id");
 	    // never reach here
 	  }
 	}
@@ -401,7 +431,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	omniTransportLock->unlock();
 	giopStream_Buffer::deleteBuffer(b);
-	inputTerminalProtocolError(g, __FILE__, __LINE__);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Client received a CancelRequest message");
 	// never reach here
       }
       // falls through
@@ -422,7 +453,9 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	      // a reply has already been received!
 	      omniTransportLock->unlock();
 	      giopStream_Buffer::deleteBuffer(b);
-	      inputTerminalProtocolError(g, __FILE__, __LINE__);
+	      inputTerminalProtocolError(g, __FILE__, __LINE__,
+					 "Message claims to belong to a "
+					 "completed request");
 	      // never reach here
 	    }
 
@@ -435,7 +468,6 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       break;
     }
   }
-
   omniTransportLock->unlock();
 
   if (matched_target) {
@@ -467,26 +499,38 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     }
       
     CORBA::Boolean isfull = ((hdr[6] & 0x2) ? 0 : 1);
-    if (mtype == GIOP::CancelRequest) isfull = 1;
-    if (isfull) {
-      omni_tracedmutex_lock sync(*omniTransportLock);
-      matched_target->inputFullyBuffered(isfull);
 
+    if (mtype == GIOP::CancelRequest) {
+      if (!matched_target_is_client)
+	((GIOP_S*)matched_target)->handleCancelRequest();
+    }
+    else if (isfull) {
+      {
+	omni_tracedmutex_lock sync(*omniTransportLock);
+	matched_target->inputFullyBuffered(isfull);
+
+	if (!matched_target_is_client)
+	  ((GIOP_S*)matched_target)->state(IOP_S::InputFullyBuffered);
+
+	giopStream::wakeUpRdLock(g->pd_strand);
+      }
       if (!matched_target_is_client) {
-
-	((GIOP_S*)matched_target)->state(IOP_S::InputFullyBuffered);
-	omniORB::logs(40, "Changed GIOP_S to InputFullyBuffered");
+	omniORB::logs(25, "Changed GIOP_S to InputFullyBuffered");
 	if (!g->pd_strand->isClient()) {
 	  g->pd_strand->server->notifyCallFullyBuffered(g->pd_strand->connection);
 	}
       }
-      giopStream::wakeUpRdLock(g->pd_strand);
     }
     return;
   }
 
   // reach here if we have no match.
   {
+    if (omniORB::trace(25)) {
+      omniORB::logger log;
+      log << "Discarding message for non-existent request id "
+	  << reqid << ".\n";
+    }
     CORBA::ULong fetchsz = b->size - (b->last - b->start);
     giopStream_Buffer::deleteBuffer(b);
     while (fetchsz) {
@@ -511,7 +555,8 @@ giopImpl12::inputNewServerMessage(giopStream* g) {
                                        g->pd_currentInputBuffer->start;
 
   if (hdr[4] != 1 || hdr[5] > 2 || hdr[7] > (unsigned char) GIOP::Fragment) {
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid message version/type");
     // never reach here
   }
 
@@ -527,7 +572,8 @@ giopImpl12::inputNewServerMessage(giopStream* g) {
       break;
     }
     else {
-      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Server received an invalid reply message");
       // Never reach here.
     }
   case GIOP::Fragment:
@@ -572,7 +618,8 @@ giopImpl12::inputNewFragment(giopStream* g) {
       throw GIOP_S::terminateProcessing();
     }
     else {
-      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Client received a CancelRequest message");
       // never reach here.
     }
   }
@@ -581,7 +628,9 @@ giopImpl12::inputNewFragment(giopStream* g) {
 			  ? 0 : 1 );
 
   if (bswap != g->pd_unmarshal_byte_swap) {
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Fragment has different byte ordering to "
+			       "initial message");
     // never reach here
   }
 
@@ -638,7 +687,10 @@ giopImpl12::inputReplyBegin(giopStream* g,
 	giopStream::CommFailure::_raise(minor,
 					status,
 					retry,
-					__FILE__,__LINE__);
+					__FILE__,__LINE__,
+					"Connection closed while waiting for "
+					"a reply on a bidirectional "
+					"connection", g->pd_strand);
 	// never reaches here.
       }
     }
@@ -819,7 +871,8 @@ giopImpl12::inputMessageEnd(giopStream* g,CORBA::Boolean disgard) {
 	disgard = 1;
       }
       else {
-	inputTerminalProtocolError(g, __FILE__, __LINE__);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Garbage left at end of input message");
 	// never reach here.
       }
     }
@@ -849,7 +902,9 @@ giopImpl12::unmarshalReplyHeader(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::Reply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "Reply");
     // Never reach here.
   }
 
@@ -873,7 +928,8 @@ giopImpl12::unmarshalReplyHeader(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in Reply");
     // Never reach here.
   }
 
@@ -911,7 +967,9 @@ giopImpl12::unmarshalLocateReply(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::LocateReply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "LocateReply");
     // Never reach here.
   }
 
@@ -935,7 +993,8 @@ giopImpl12::unmarshalLocateReply(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in LocateReply");
     // never reach here.
   }
   giop_c.locateStatus((GIOP::LocateStatusType)v);
@@ -958,6 +1017,7 @@ giopImpl12::unmarshalWildCardRequestHeader(giopStream* g) {
     CORBA::ULong reqid;
     reqid <<= (cdrStream&)(*g);
     ((GIOP_S*)g)->requestId(reqid);
+    // HERE: should check if request id has been seen before
     break;
   case GIOP::CloseConnection:
     if (g->pd_strand->biDir && g->pd_strand->isClient()) {
@@ -969,10 +1029,11 @@ giopImpl12::unmarshalWildCardRequestHeader(giopStream* g) {
       }
       g->pd_strand->startIdleCounter();
     }
-    inputRaiseCommFailure(g);
+    inputRaiseCommFailure(g, "Orderly connection shutdown");
     break;
   default:
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Unknown GIOP message type");
     // Never reach here.
     break;
   }
@@ -1079,7 +1140,9 @@ giopImpl12::getInputData(giopStream* g,omni::alignment_t align,size_t sz) {
 
 	if (g->inputExpectAnotherFragment()) {
 	  // The incoming message is fragmented at the wrong boundary!!!
-	  inputTerminalProtocolError(g, __FILE__, __LINE__);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__,
+				     "Message fragmented at incorrect "
+				     "boundary");
 	  // never reach here
 	}
 	// Very bad. Should never happen given our invariant.
@@ -1169,7 +1232,9 @@ giopImpl12::copyInputData(giopStream* g,void* b, size_t sz,
 
       if (g->inputExpectAnotherFragment()) {
 	// The incoming message is fragmented at the wrong boundary!!!
-	inputTerminalProtocolError(g, __FILE__, __LINE__);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Message fragmented at incorrect "
+				   "boundary (bulk copy)");
 	// never reach here
       }
       // Very bad. Should never happen given our invariant.
@@ -1201,10 +1266,10 @@ giopImpl12::copyInputData(giopStream* g,void* b, size_t sz,
       g->pd_inb_mkr = (void*)((omni::ptr_arith_t)g->pd_inb_mkr + avail);
     }
     sz -= avail;
-    if (b) b = (void*)((omni::ptr_arith_t)b + avail);
-
     if (!sz) break;
 
+    if (b) b = (void*)((omni::ptr_arith_t)b + avail);
+
     if (g->pd_inb_mkr == g->pd_inb_end) {
 
       if (g->inputFragmentToCome() < sz && !g->inputExpectAnotherFragment()) {
@@ -1273,31 +1338,33 @@ giopImpl12::currentInputPtr(const giopStream* g) {
 ////////////////////////////////////////////////////////////////////////
 void
 giopImpl12::inputTerminalProtocolError(giopStream* g,
-				       const char* file, int line)
+				       const char* file, int line,
+				       const char* message)
 {
   // XXX We may choose to send a message error to the other end.
   if (omniORB::trace(1)) {
     omniORB::logger l;
     l << "From endpoint: " << g->pd_strand->connection->peeraddress()
       << ". Detected GIOP 1.2 protocol error in input message. "
-      << file << ":" << line
+      << omniExHelper::strip(file) << ":" << line
       << ". Connection is closed.\n";
   }
 
-  inputRaiseCommFailure(g);
+  inputRaiseCommFailure(g, message);
 }
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl12::inputRaiseCommFailure(giopStream* g) {
+giopImpl12::inputRaiseCommFailure(giopStream* g, const char* message) {
 
   CORBA::ULong minor;
   CORBA::Boolean retry;
   g->notifyCommFailure(0,minor,retry);
   g->pd_strand->state(giopStrand::DYING);
+
   giopStream::CommFailure::_raise(minor,
 				  (CORBA::CompletionStatus)g->completion(),
-				  0,__FILE__,__LINE__);
+				  0,__FILE__,__LINE__,message,g->pd_strand);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -1452,25 +1519,34 @@ giopImpl12::marshalRequestHeader(giopStream* g) {
   CORBA::Boolean response_expected = !calldesc.is_oneway();
   
 
-  omniInterceptors::clientSendRequest_T::info_T info(*g,
-				                     *(giop_c.ior()),
-						     calldesc.op(),
-						     !response_expected,
-						     response_expected);
+  omniInterceptors::clientSendRequest_T::info_T info(giop_c);
   omniInterceptorP::visit(info);
 
   CORBA::Octet v = 0;
-  {
-    // calculate the request header size
-    cdrCountingStream cs(g->TCS_C(),g->TCS_W(),12);
-    operator>>= ((CORBA::ULong)0,cs);
-    cs.marshalOctet(v);
-    cs.marshalOctet(v);
-    cs.marshalOctet(v);
-    cs.marshalOctet(v);
+  CORBA::ULong sc_len = info.service_contexts.length();
+
+  if (sc_len == 0 && giop_c.ior()->addr_mode() == GIOP::KeyAddr) {
+
+    // Simple fast case to calculate request header size
+    //
+    // GIOP header (12) + req_id (4) + flags (1) + pad(3) +
+    // KeyAddr(2) + pad(2) + key_len (4) + key (n) + pad (<=3) +
+    // op_len (4) + op (n) + pad (<=3) + service_ctx list len (4)
+
+    CORBA::ULong total;
+    total = 12 + 4 + 1 + 3 + 2 + 2 + 4 + giop_c.keysize();
+    total = (total + 3) & ~((CORBA::ULong)3);
+    total = (total + 4 + calldesc.op_len() + 3) & ~((CORBA::ULong)3);
+    total = total + 4;
+
+    *((CORBA::ULong*)(hdr+8)) = total;
+  }
+  else {
+    // Calculate the request header size
+    cdrCountingStream cs(g->TCS_C(),g->TCS_W(),12 + 4 + 1 + 3);
 
     if (giop_c.ior()->addr_mode() == GIOP::KeyAddr) {
-      giop_c.ior()->addr_mode() >>= cs;
+      GIOP::KeyAddr >>= cs;
       giop_c.keysize() >>= cs;
       cs.put_octet_array(giop_c.key(),giop_c.keysize());
     }
@@ -1481,6 +1557,7 @@ giopImpl12::marshalRequestHeader(giopStream* g) {
     operator>>= ((CORBA::ULong) calldesc.op_len(),cs);
     cs.put_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
     info.service_contexts >>= cs;
+
     *((CORBA::ULong*)(hdr+8)) = cs.total();
   }
 
@@ -1499,9 +1576,9 @@ giopImpl12::marshalRequestHeader(giopStream* g) {
 
   // Target address
   if (giop_c.ior()->addr_mode() == GIOP::KeyAddr) {
-    giop_c.ior()->addr_mode() >>= s;
+    GIOP::KeyAddr >>= s;
     giop_c.keysize() >>= s;
-    s.put_octet_array(giop_c.key(),giop_c.keysize());
+    s.put_small_octet_array(giop_c.key(),giop_c.keysize());
   }
   else {
     giop_c.ior()->marshalIORAddressingInfo(s);
@@ -1509,10 +1586,13 @@ giopImpl12::marshalRequestHeader(giopStream* g) {
 
   // operation
   operator>>= ((CORBA::ULong) calldesc.op_len(),s);
-  s.put_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
+  s.put_small_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
 
   // service context
-  info.service_contexts >>= s;
+  if (sc_len)
+    info.service_contexts >>= s;
+  else
+    CORBA::ULong(0) >>= s;
 
   s.alignOutput(omni::ALIGN_8);
 }
@@ -1561,8 +1641,13 @@ giopImpl12::marshalReplyHeader(giopStream* g) {
   GIOP_S& giop_s = *(GIOP_S*)g;
   cdrStream& s = (cdrStream&) *g;
 
+  CORBA::ULong sc_len = giop_s.service_contexts().length();
   CORBA::ULong rc = GIOP::NO_EXCEPTION;
-  {
+  if (sc_len == 0) {
+    CORBA::ULong total = 12 + 4 + 4 + 4;
+    *((CORBA::ULong*)(hdr+8)) = total;
+  }
+  else {
     // calculate the reply header size
     cdrCountingStream cs(g->TCS_C(),g->TCS_W(),12);
     operator>>= ((CORBA::ULong)0,cs);
@@ -1578,8 +1663,11 @@ giopImpl12::marshalReplyHeader(giopStream* g) {
   rc >>= s;
 
   // Service context
-  giop_s.service_contexts() >>= s;
-  
+  if (sc_len)
+    giop_s.service_contexts() >>= s;
+  else
+    CORBA::ULong(0) >>= s;
+
   s.alignOutput(omni::ALIGN_8);
 }
 
@@ -1600,7 +1688,10 @@ giopImpl12::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
     giop_s.notifyCommFailure(0,minor,retry);
     giopStream::CommFailure::_raise(minor,(CORBA::CompletionStatus)
 				    giop_s.completion(),
-				    retry,__FILE__,__LINE__);
+				    retry,__FILE__,__LINE__,
+				    "System Exception occurred while "
+				    "marshalling reply. Sending a "
+				    "MessageError", g->pd_strand);
   }
 
   int repoid_size;
@@ -1664,7 +1755,7 @@ giopImpl12::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
 
   // RepoId
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
 
   // system exception value
   ex.minor() >>= s;
@@ -1726,7 +1817,7 @@ giopImpl12::sendUserException(giopStream* g,const CORBA::UserException& ex) {
 
   // RepoId
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
 
   // Exception value
   ex._NP_marshal(s);
@@ -1847,7 +1938,7 @@ giopImpl12::sendLocateReply(giopStream* g,GIOP::LocateStatusType rc,
   }
   else if (rc == GIOP::LOC_SYSTEM_EXCEPTION) {
     CORBA::ULong(repoid_size) >>= s;
-    s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+    s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
     p->minor() >>= s;
     operator>>= ((CORBA::ULong)p->completed(),s);
   }
@@ -1879,7 +1970,7 @@ giopImpl12::outputRemaining(const giopStream* g) {
 void
 giopImpl12::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
 
-  // Note: g->outputFragmentSize() == 0 implies that the full message
+  // Note: g->outputFragmentSize() != 0 implies that the full message
   //       size has been pre-calculated and no GIOP Fragment should be
   //       sent! This also means that the message size limit has been
   //       checked and there is no need to check against
@@ -1890,7 +1981,7 @@ giopImpl12::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
 				    g->pd_currentOutputBuffer + 
 				    g->pd_currentOutputBuffer->start);
 
-  CORBA::ULong fsz =   (omni::ptr_arith_t) g->pd_outb_mkr - outbuf_begin;
+  CORBA::ULong fsz = (omni::ptr_arith_t) g->pd_outb_mkr - outbuf_begin;
 
   if (!g->outputFragmentSize()) {
     ((char*)outbuf_begin)[6] |= 0x2;
@@ -1902,7 +1993,15 @@ giopImpl12::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
     else {
       sz = *((CORBA::ULong*)((omni::ptr_arith_t)outbuf_begin + 8));
     }
-    g->outputMessageSize(g->outputMessageSize()+sz);
+    CORBA::Long msz = g->outputMessageSize();
+    if (msz) {
+      // Message has increased by 4 bytes less than the fragment size,
+      // due to the message id at the start of the fragment.
+      g->outputMessageSize(msz + sz - 4);
+    }
+    else {
+      g->outputMessageSize(sz);
+    }
     if (g->outputMessageSize() > orbParameters::giopMaxMsgSize) {
       OMNIORB_THROW(MARSHAL,MARSHAL_MessageSizeExceedLimitOnClient,
 		    (CORBA::CompletionStatus)g->completion());
@@ -1911,7 +2010,7 @@ giopImpl12::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
   else {
     CORBA::Long msz = g->outputMessageSize();
     if (msz) {
-      g->outputMessageSize(msz+fsz);
+      g->outputMessageSize(msz+fsz-16);
     }
     else {
       g->outputMessageSize(fsz-12);
@@ -2083,10 +2182,18 @@ giopImpl12::currentOutputPtr(const giopStream* g) {
                      ((omni::ptr_arith_t) g->pd_currentOutputBuffer + 
 		      g->pd_currentOutputBuffer->start);
 
-  if (g->outputMessageSize()) {
-    return fsz + g->outputMessageSize();
+  // Output offset is the sent message so far, plus the current buffer
+  // minus 12 byte header.
+  CORBA::Long msz = g->outputMessageSize();
+  if (msz) {
+    // At least one fragment has been output already; message has
+    // been extended by the current fragment size minus its 16 byte
+    // header (12 for GIOP header, plus 4 for request id).
+    return msz + fsz - 16;
   }
   else {
+    // This is the first fragment -- message size is the fragment
+    // size minus 12 byte GIOP header.
     return fsz - 12;
   }
 }
@@ -2102,15 +2209,6 @@ giopImpl12::outputSetFragmentSize(giopStream* g,CORBA::ULong msz) {
   g->outputFragmentSize(msz);
 }
 
-////////////////////////////////////////////////////////////////////////
-CORBA::Boolean
-giopImpl12::outputHasReachedLimit(giopStream* g) {
-
-  return ( (omni::ptr_arith_t) g->pd_outb_end == 
-          ((omni::ptr_arith_t) g->pd_currentOutputBuffer + 
-		  	       g->pd_currentOutputBuffer->start + 12) );
-}
-
 ////////////////////////////////////////////////////////////////////////
 static giopStreamImpl* giop_1_2_singleton = 0;
 
