@@ -29,78 +29,58 @@
 
 /*
   $Log: giopServer.cc,v $
-  Revision 1.22.2.46  2005/11/20 15:33:03  dgrisby
-  New connection cleanup meant some connection state was not completely
-  deleted.
+  Revision 1.25.2.17  2006/10/30 14:17:22  dgrisby
+  Cast in log message for 64 bit Windows.
 
-  Revision 1.22.2.45  2005/11/17 11:35:52  dgrisby
-  In thread pool mode, bidir workers become wedged waiting for incoming
-  calls, so they must be closed in giopServer::deactivate.
+  Revision 1.25.2.16  2006/10/09 09:47:12  dgrisby
+  Only delete giopServer if all threads are successfully shut down.
 
-  Revision 1.22.2.44  2005/11/16 17:35:25  dgrisby
-  New connectionWatchPeriod and connectionWatchImmediate parameters.
+  Revision 1.25.2.15  2006/09/01 14:11:52  dgrisby
+  Avoid potential deadlock with call buffering; do not force worker
+  creation when a request is fully buffered.
 
-  Revision 1.22.2.43  2005/11/15 11:07:56  dgrisby
-  More shutdown cleanup.
+  Revision 1.25.2.14  2006/06/05 13:34:31  dgrisby
+  Make connection thread limit a per-connection value.
 
-  Revision 1.22.2.42  2005/11/14 15:20:08  dgrisby
-  Race between giopServer destruction and closure of connections in
-  thread pool mode.
+  Revision 1.25.2.13  2006/04/28 18:40:46  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.22.2.41  2005/11/14 10:58:23  dgrisby
-  Better connection / thread shutdown behaviour.
+  Revision 1.25.2.12  2006/04/09 19:52:31  dgrisby
+  More IPv6, endPointPublish parameter.
 
-  Revision 1.22.2.40  2005/11/04 14:26:02  dgrisby
-  Open connections would prevent shutdown on Windows.
+  Revision 1.25.2.11  2005/11/21 11:02:57  dgrisby
+  Another merge.
 
-  Revision 1.22.2.39  2005/10/13 11:38:16  dgrisby
-  Dump CloseConnection messages.
+  Revision 1.25.2.10  2005/11/17 17:03:26  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.22.2.38  2005/05/03 10:10:48  dgrisby
-  Avoid deadlock caused by trying to deactivation SSL rendezvouser more
-  than once.
+  Revision 1.25.2.9  2005/06/08 09:41:06  dgrisby
+  Update comment.
 
-  Revision 1.22.2.37  2005/04/10 22:17:19  dgrisby
-  Fixes to connection management. Thanks Jon Biggar.
+  Revision 1.25.2.8  2005/05/10 22:07:33  dgrisby
+  Merge again.
 
-  Revision 1.22.2.36  2005/03/10 11:28:29  dgrisby
-  Race condition between setSelectable / clearSelectable.
+  Revision 1.25.2.7  2005/04/11 12:09:41  dgrisby
+  Another merge.
 
-  Revision 1.22.2.35  2005/02/23 12:27:31  dgrisby
-  Another race in setSelectable with connection shutdown. Thanks Peter
-  Klotz.
+  Revision 1.25.2.6  2005/03/30 23:36:10  dgrisby
+  Another merge from omni4_0_develop.
 
-  Revision 1.22.2.34  2005/02/13 20:52:51  dgrisby
-  Change threadPoolWatchConnection parameter to be an integer rather
-  than a boolean. Value is the count of threads that can be handling a
-  connection when one decides to watch it.
+  Revision 1.25.2.5  2005/03/02 12:39:17  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.22.2.33  2004/12/20 18:17:55  dgrisby
-  Avoid race condition on connection disconnection.
+  Revision 1.25.2.4  2005/03/02 12:10:49  dgrisby
+  setSelectable / Peek fixes.
 
-  Revision 1.22.2.32  2004/10/17 20:14:32  dgrisby
-  Updated support for OpenVMS. Many thanks to Bruce Visscher.
+  Revision 1.25.2.3  2005/01/13 21:09:59  dgrisby
+  New SocketCollection implementation, using poll() where available and
+  select() otherwise. Windows specific version to follow.
 
-  Revision 1.22.2.31  2004/09/13 10:03:13  dgrisby
-  Fix for giopServer shutdown issue. Thanks Serguei Kolos.
+  Revision 1.25.2.2  2005/01/06 23:10:27  dgrisby
+  Big merge from omni4_0_develop.
 
-  Revision 1.22.2.30  2004/04/08 10:02:20  dgrisby
-  In thread pool mode, close connections that will not be selectable.
-
-  Revision 1.22.2.29  2004/03/02 15:32:07  dgrisby
-  Fix locking in connection shutdown.
-
-  Revision 1.22.2.28  2004/02/27 12:40:09  dgrisby
-  Fix race conditions in connection shutdown. Thanks Serguei Kolos.
-
-  Revision 1.22.2.27  2003/10/20 16:11:12  dgrisby
-  Race condition in connection closure.
-
-  Revision 1.22.2.26  2003/07/16 14:22:38  dgrisby
-  Speed up oneway handling a little. More tracing for split messages.
-
-  Revision 1.22.2.25  2003/05/22 13:47:40  dgrisby
-  Failed to setSelectable in some cases.
+  Revision 1.25.2.1  2003/03/23 21:02:15  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.22.2.24  2003/02/17 01:46:23  dgrisby
   Pipe to kick select thread (on Unix).
@@ -282,43 +262,69 @@ giopServer::~giopServer()
 
 
 ////////////////////////////////////////////////////////////////////////////
-const char*
-giopServer::instantiate(const char* uri,
-			CORBA::Boolean no_publish,
-			CORBA::Boolean no_listen)
+CORBA::Boolean
+giopServer::instantiate(const char*              uri,
+			CORBA::Boolean           no_publish,
+			orbServer::EndpointList& listening_endpoints)
 {
   ASSERT_OMNI_TRACEDMUTEX_HELD(pd_lock,0);
   omni_tracedmutex_lock sync(pd_lock);
 
   ensureNotInFlux();
 
-  if (!no_listen) {
+  giopEndpoint* ept = giopEndpoint::str2Endpoint(uri);
+  if (!ept) return 0;
 
-    giopEndpoint* ept = giopEndpoint::str2Endpoint(uri);
-    if (!ept) return 0;
+  OMNIORB_ASSERT(pd_state != ZOMBIE);
 
-    OMNIORB_ASSERT(pd_state != ZOMBIE);
+  if (no_publish)
+    ept->set_no_publish();
 
-    if (ept->Bind()) {
-      pd_endpoints.push_back(ept);
-      if (pd_state == ACTIVE) activate();
-      uri =  ept->address();
-    }
-    else {
-      delete ept;
-      return 0;
+  if (ept->Bind()) {
+    pd_endpoints.push_back(ept);
+
+    // Add to listening_endpoints
+    const orbServer::EndpointList* addrs = ept->addresses();
+    CORBA::ULong i = 0;
+    CORBA::ULong j = listening_endpoints.length();
+
+    listening_endpoints.length(j + addrs->length());
+    for (; i < addrs->length(); ++i, ++j) {
+      listening_endpoints[j] = (*addrs)[i];
     }
+    if (pd_state == ACTIVE) activate();
   }
-  else if ( !giopEndpoint::strIsValidEndpoint(uri) ) {
+  else {
+    delete ept;
     return 0;
   }
-
-  if (!no_publish) {
-    (void)giopEndpoint::addToIOR(uri);
-  }
-  return uri;
+  return 1;
 }
 
+////////////////////////////////////////////////////////////////////////////
+CORBA::Boolean
+giopServer::publish(const orbServer::PublishSpecs& publish_specs,
+		    CORBA::Boolean           	   all_specs,
+		    CORBA::Boolean           	   all_eps,
+		    orbServer::EndpointList& 	   published_endpoints)
+{
+  omni_tracedmutex_lock sync(pd_lock);
+
+  ensureNotInFlux();
+
+  CORBA::Boolean result = 0;
+  CORBA::Boolean ok;
+
+  giopEndpointList::iterator i;
+  for (i = pd_endpoints.begin(); i != pd_endpoints.end(); ++i) {
+
+    ok = (*i)->publish(publish_specs, all_specs, all_eps, published_endpoints);
+    result |= ok;
+  }
+  return result;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////
 CORBA::Boolean
 giopServer::addBiDirStrand(giopStrand* s,giopActiveCollection* watcher) {
@@ -390,10 +396,8 @@ giopServer::start()
 
   switch (pd_state) {
   case IDLE:
-    {
-      pd_state = ACTIVE;
-      activate();
-    }
+    pd_state = ACTIVE;
+    activate();
     break;
   default:
     break;
@@ -410,9 +414,7 @@ giopServer::stop()
 
   switch (pd_state) {
   case ACTIVE:
-    {
-      deactivate();
-    }
+    deactivate();
     break;
   default:
     break;
@@ -423,31 +425,35 @@ giopServer::stop()
 void
 giopServer::remove()
 {
-  omni_tracedmutex_lock sync(pd_lock);
+  CORBA::Boolean do_delete = 1;
+  {
+    omni_tracedmutex_lock sync(pd_lock);
 
-  ensureNotInFlux();
+    ensureNotInFlux();
 
-  switch (pd_state) {
-  case ACTIVE:
-    {
+    switch (pd_state) {
+    case ACTIVE:
       deactivate();
+      break;
+    case TIMEDOUT:
+      do_delete = 0;
+      break;
+    default:
+      break;
     }
-  case IDLE:
-    {
-      pd_state = ZOMBIE;
+
+    pd_state = ZOMBIE;
+
+    giopEndpointList::iterator i;
+    i = pd_endpoints.begin();
+
+    while (i != pd_endpoints.end()) {
+      (*i)->Shutdown();
+      pd_endpoints.erase(i);
     }
-    break;
-  default:
-    return;
-  }
-
-  giopEndpointList::iterator i;
-  i = pd_endpoints.begin();
-
-  while (i != pd_endpoints.end()) {
-    (*i)->Shutdown();
-    pd_endpoints.erase(i);
   }
+  if (do_delete)
+    delete this;
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -458,7 +464,7 @@ giopServer::activate()
   ASSERT_OMNI_TRACEDMUTEX_HELD(pd_lock, 1);
 
   giopEndpointList::iterator i;
-  i    = pd_endpoints.begin();
+  i = pd_endpoints.begin();
 
   while (i != pd_endpoints.end()) {
     giopRendezvouser* task = new giopRendezvouser(*i,this);
@@ -582,8 +588,8 @@ giopServer::deactivate()
 
   pd_state = INFLUX;
 
-  CORBA::Boolean waitforcompletion;
   CORBA::Boolean rendezvousers_terminated = 0;
+  CORBA::Boolean timed_out = 0;
 
   omniORB::logs(25, "giopServer deactivate...");
 
@@ -633,6 +639,8 @@ giopServer::deactivate()
       while (go && pd_n_dedicated_workers) {
 	go = pd_cond.timedwait(s, ns);
       }
+      timed_out |= !go;
+
       if (omniORB::trace(25)) {
 	omniORB::logger l;
 	if (!go)
@@ -655,10 +663,13 @@ giopServer::deactivate()
     }
 
     omni_thread::get_time(&s, &ns, timeout);
+
     int go = 1;
     while (go && pd_rendezvousers.next != & pd_rendezvousers) {
       go = pd_cond.timedwait(s, ns);
     }
+    timed_out |= !go;
+
     if (omniORB::trace(25)) {
       omniORB::logger l;
       if (go)
@@ -682,6 +693,8 @@ giopServer::deactivate()
     while (go && !Link::is_empty(pd_bidir_monitors)) {
       go = pd_cond.timedwait(s, ns);
     }
+    timed_out |= !go;
+
     if (omniORB::trace(25)) {
       omniORB::logger l;
       if (go)
@@ -732,6 +745,8 @@ giopServer::deactivate()
     while (go && pd_n_temporary_workers) {
       go = pd_cond.timedwait(s, ns);
     }
+    timed_out |= !go;
+
     if (omniORB::trace(25)) {
       omniORB::logger l;
       if (!go)
@@ -745,7 +760,7 @@ giopServer::deactivate()
   if (pd_bidir_strands.size()) {
     if (omniORB::trace(25)) {
       omniORB::logger l;
-      l << "Close " << pd_bidir_strands.size()
+      l << "Close " << (CORBA::ULong)pd_bidir_strands.size()
 	<< " bidirectional connections...\n";
     }
 
@@ -757,7 +772,11 @@ giopServer::deactivate()
       g->deleteStrandAndConnection();
     }
   }
-  pd_state = IDLE;
+  if (timed_out)
+    pd_state = TIMEDOUT;
+  else
+    pd_state = IDLE;
+
   pd_cond.broadcast();
   omniORB::logs(25, "giopServer deactivated.");
 }
@@ -884,6 +903,7 @@ giopServer::csInsert(giopConnection* conn)
   }
 
   conn->pd_has_dedicated_thread = pd_thread_per_connection;
+  conn->pd_max_workers = orbParameters::maxServerThreadPerConnection;
 
   return cs;
 }
@@ -917,6 +937,7 @@ giopServer::csInsert(giopStrand* s)
   }
 
   conn->pd_has_dedicated_thread = pd_thread_per_connection;
+  conn->pd_max_workers = orbParameters::maxServerThreadPerConnection;
 
   {
     omni_tracedmutex_lock sync(*omniTransportLock);
@@ -1027,7 +1048,7 @@ giopServer::notifyRzDone(giopRendezvouser* r, CORBA::Boolean exit_on_error)
 
   delete r;
 
-  if (exit_on_error) {
+  if (exit_on_error && pd_state != INFLUX) {
     if (omniORB::trace(1)) {
       omniORB::logger log;
       log << "Unrecoverable error for this endpoint: ";
@@ -1044,6 +1065,7 @@ giopServer::notifyRzDone(giopRendezvouser* r, CORBA::Boolean exit_on_error)
 
   if (pd_state == INFLUX) {
     if (Link::is_empty(pd_rendezvousers)) {
+      omniORB::logs(25, "No remaining rendezvousers.");
       pd_cond.broadcast();
     }
   }
@@ -1064,8 +1086,7 @@ giopServer::notifyRzReadable(giopConnection* conn,
     {
       if (conn->pd_dying) return;
 
-      if (!force_create &&
-	  conn->pd_n_workers >= (int)orbParameters::maxServerThreadPerConnection) {
+      if (!force_create && conn->pd_n_workers >= conn->pd_max_workers) {
 	conn->pd_has_hit_n_workers_limit = 1;
 	return;
       }
@@ -1093,7 +1114,7 @@ giopServer::notifyRzReadable(giopConnection* conn,
 void
 giopServer::notifyCallFullyBuffered(giopConnection* conn)
 {
-  notifyRzReadable(conn,1);
+  notifyRzReadable(conn);
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -1129,8 +1150,9 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     pd_lock.lock();
 
     int workers;
+    CORBA::Boolean singleshot = w->singleshot();
 
-    if (w->singleshot())
+    if (singleshot)
       workers = --pd_n_temporary_workers;
     else
       workers = --pd_n_dedicated_workers;
@@ -1146,6 +1168,12 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     }
 
     if (pd_state == INFLUX) {
+      if (omniORB::trace(25)) {
+	omniORB::logger l;
+	l << "removeConnectionAndWorker for "
+	  << (singleshot ? "temporary" : "dedicated")
+	  << " worker. " << workers << " remaining.\n";
+      }
       if (workers == 0) {
 	pd_cond.broadcast();
       }
@@ -1207,8 +1235,11 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       delete w;
       conn->pd_n_workers--;
       pd_n_temporary_workers--;
-      if (pd_state == INFLUX && pd_n_temporary_workers == 0)
-	pd_cond.broadcast();
+      if (pd_state == INFLUX) {
+	omniORB::logs(25, "Temporary additional worker finishing.");
+	if (pd_n_temporary_workers == 0)
+	  pd_cond.broadcast();
+      }
       return 0;
     }
   }
@@ -1240,14 +1271,10 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
 
     if (!select_and_return) {
       // Call Peek(). This thread will be used for a short time to
-      // monitor the connection. If the connection is available for
-      // reading, the callback function peekCallBack is called. We can
-      // probably afford to call Peek() here because this thread is
-      // otherwise idle.
+      // monitor the connection. We can probably afford to call Peek()
+      // here because this thread is otherwise idle.
 
-      CORBA::Boolean readable = 0;
-      conn->Peek(peekCallBack,(void*)&readable);
-      if (readable) {
+      if (conn->Peek()) {
 	// There is data to be read. Tell the worker to go around again.
 	return 1;
       }
@@ -1310,7 +1337,7 @@ giopServer::notifyWkPreUpCall(giopWorker* w, CORBA::Boolean data_in_buffer) {
 	omni_tracedmutex_lock sync(pd_lock);
 	conn->pd_dedicated_thread_in_upcall = 1;
       }
-      if (orbParameters::maxServerThreadPerConnection > 1) {
+      if (conn->pd_max_workers > 1) {
 	// If only one thread per connection is allowed, there is no
 	// need to setSelectable, since we won't be able to act on any
 	// interleaved calls that arrive.
@@ -1369,6 +1396,7 @@ giopServer::notifyMrDone(giopMonitor* m, CORBA::Boolean exit_on_error)
   delete m;
   if (pd_state == INFLUX) {
     if (Link::is_empty(pd_bidir_monitors)) {
+      omniORB::logs(25, "No remaining bidir monitors.");
       pd_cond.broadcast();
     }
   }
