@@ -28,27 +28,39 @@
 
 /*
   $Log: giopStrand.cc,v $
-  Revision 1.1.4.25  2005/11/14 10:58:23  dgrisby
-  Better connection / thread shutdown behaviour.
+  Revision 1.1.6.10  2007/02/05 18:39:53  dgrisby
+  Log scavenger connections closures at traceLevel 25 instead of 30.
 
-  Revision 1.1.4.24  2005/10/13 11:38:16  dgrisby
-  Dump CloseConnection messages.
+  Revision 1.1.6.9  2007/02/05 17:38:13  dgrisby
+  Scavenger did not cope with client strands with no connection. This
+  can happen if a Python client uses invalid argument types, for
+  example.
 
-  Revision 1.1.4.23  2005/04/10 22:17:18  dgrisby
-  Fixes to connection management. Thanks Jon Biggar.
+  Revision 1.1.6.8  2006/07/18 16:21:21  dgrisby
+  New experimental connection management extension; ORB core support
+  for it.
 
-  Revision 1.1.4.22  2005/03/16 09:16:59  dgrisby
-  Previous change to CancelRequest handling accidentally broke
-  server-side connection shutdown.
+  Revision 1.1.6.7  2006/07/02 22:52:05  dgrisby
+  Store self thread in task objects to avoid calls to self(), speeding
+  up Current. Other minor performance tweaks.
 
-  Revision 1.1.4.21  2004/10/17 21:48:40  dgrisby
-  Support CancelRequest better.
+  Revision 1.1.6.6  2006/06/22 13:53:49  dgrisby
+  Add flags to strand.
 
-  Revision 1.1.4.20  2004/03/30 14:26:36  dgrisby
-  Better fix for scavenger thread restarting after shutdown.
+  Revision 1.1.6.5  2005/11/17 17:03:26  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.4.19  2004/03/30 13:14:47  dgrisby
-  Allow scavenger thread to restart.
+  Revision 1.1.6.4  2005/04/11 12:09:41  dgrisby
+  Another merge.
+
+  Revision 1.1.6.3  2005/03/30 23:36:10  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.1.6.2  2005/01/06 23:10:29  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.6.1  2003/03/23 21:02:14  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.4.18  2002/09/04 23:29:30  dgrisby
   Avoid memory corruption with multiple list removals.
@@ -118,6 +130,7 @@
 #include <giopStreamImpl.h>
 #include <giopRope.h>
 #include <giopStrand.h>
+#include <giopStrandFlags.h>
 #include <GIOP_S.h>
 #include <GIOP_C.h>
 #include <initialiser.h>
@@ -207,9 +220,9 @@ sendCloseConnection(giopStrand* s)
 giopStrand::giopStrand(const giopAddress* addr) :
   pd_safelyDeleted(0),
   idlebeats(-1),
-  biDir(0), address(addr), connection(0), server(0),
-  gatekeeper_checked(0),first_use(1),orderly_closed(0),
-  biDir_initiated(0), biDir_has_callbacks(0),
+  address(addr), connection(0), server(0), flags(0),
+  biDir(0), gatekeeper_checked(0), first_use(1), first_call(1),
+  orderly_closed(0), biDir_initiated(0), biDir_has_callbacks(0),
   tcs_selected(0), tcs_c(0), tcs_w(0), giopImpl(0),
   rdcond(omniTransportLock), rd_nwaiting(0), rd_n_justwaiting(0),
   wrcond(omniTransportLock), wr_nwaiting(0),
@@ -225,9 +238,9 @@ giopStrand::giopStrand(const giopAddress* addr) :
 giopStrand::giopStrand(giopConnection* conn, giopServer* serv) :
   pd_safelyDeleted(0),
   idlebeats(-1),
-  biDir(0), address(0), connection(conn), server(serv),
-  gatekeeper_checked(0),first_use(0),orderly_closed(0),
-  biDir_initiated(0), biDir_has_callbacks(0),
+  address(0), connection(conn), server(serv), flags(0),
+  biDir(0), gatekeeper_checked(0), first_use(0), first_call(0),
+  orderly_closed(0), biDir_initiated(0), biDir_has_callbacks(0),
   tcs_selected(0), tcs_c(0), tcs_w(0), giopImpl(0),
   rdcond(omniTransportLock), rd_nwaiting(0), rd_n_justwaiting(0),
   wrcond(omniTransportLock), wr_nwaiting(0),
@@ -630,6 +643,11 @@ giopStrand::startIdleCounter() {
     // The idle counter is already active or has already expired.
     return 0;
   }
+  if (flags & GIOPSTRAND_HOLD_OPEN) {
+    // Flags say we should hold the connection open.
+    idlebeats = -1;
+    return 1;
+  }
   if (isClient()) {
     idlebeats = (idleOutgoingBeats) ? (CORBA::Long)idleOutgoingBeats : -1;
   }
@@ -732,15 +750,17 @@ Scavenger::execute()
 	p = p->next;
 	s->StrandList::remove();
 	s->state(giopStrand::DYING);
-	if (omniORB::trace(30)) {
+	if (omniORB::trace(25)) {
 	  omniORB::logger log;
-	  log << "Scavenger close connection to " << s->address->address() << "\n";
-	}
-	if ( s->version.minor >= 2 ) {
-	  // GIOP 1.2 or above requires the client send a closeconnection
-	  // message.
-	  sendCloseConnection(s);
+          log << "Scavenger close "
+              << (s->connection ? "connection" : "unconnected strand")
+              << " to " << s->address->address() << "\n";
 	}
+        if ( s->version.minor >= 2 && s->connection ) {
+          // GIOP 1.2 or above requires the client send a CloseConnection
+          // message.
+          sendCloseConnection(s);
+        }
 	s->safeDelete(1);
       }
     }
@@ -756,7 +776,7 @@ Scavenger::execute()
 	p = p->next;
 	s->StrandList::remove();
 	s->state(giopStrand::DYING);
-	if (omniORB::trace(30)) {
+	if (omniORB::trace(25)) {
 	  omniORB::logger log;
 	  log << "Scavenger close connection from " 
 	      << s->connection->peeraddress() << "\n";
@@ -979,14 +999,16 @@ public:
 	s->state(giopStrand::DYING);
 	if (omniORB::trace(25)) {
 	  omniORB::logger log;
-	  log << "Shutdown close connection to "
+	  log << "Shutdown close "
+              << (s->connection ? "connection" : "unconnected strand")
+              << " to "
 	      << s->address->address() << "\n";
 	}
-	if ( s->version.minor >= 2 ) {
-	  // GIOP 1.2 or above requires the client send a closeconnection
-	  // message.
-	  sendCloseConnection(s);
-	}
+        if ( s->version.minor >= 2 && s->connection ) {
+          // GIOP 1.2 or above requires the client send a closeconnection
+          // message.
+          sendCloseConnection(s);
+        }
 	s->safeDelete(1);
       }
     }
@@ -1036,33 +1058,6 @@ static omni_giopStrand_initialiser initialiser;
 
 omniInitialiser& omni_giopStrand_initialiser_ = initialiser;
 
-////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////
-void
-giopStreamList::insert(giopStreamList& head)
-{
-  next = head.prev->next;
-  head.prev->next = this;
-  prev = head.prev;
-  head.prev = this;
-}
-
-////////////////////////////////////////////////////////////////////////////
-void
-giopStreamList::remove()
-{
-  prev->next = next;
-  next->prev = prev;
-  next = prev = this;
-}
-
-////////////////////////////////////////////////////////////////////////////
-CORBA::Boolean
-giopStreamList::is_empty(giopStreamList& head)
-{
-  return (head.next == &head);
-}
-
 OMNI_NAMESPACE_END(omni)
 
 
