@@ -29,11 +29,20 @@
 
 /*
   $Log: ior.cc,v $
-  Revision 1.10.2.25  2003/11/20 13:39:59  dgrisby
-  corbaloc handler for Unix sockets. Uses omniunix scheme name.
+  Revision 1.12.2.5  2006/09/17 23:24:18  dgrisby
+  Remove hard-coded hostname length.
 
-  Revision 1.10.2.24  2003/07/26 22:52:22  dgrisby
-  Avoid spurious gcc warnings when sizeof pointer > sizeof int.
+  Revision 1.12.2.4  2006/05/21 17:44:04  dgrisby
+  Remove unnecessary uses of cdrCountingStream.
+
+  Revision 1.12.2.3  2006/04/09 19:52:31  dgrisby
+  More IPv6, endPointPublish parameter.
+
+  Revision 1.12.2.2  2005/01/06 23:10:30  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.12.2.1  2003/03/23 21:02:13  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.10.2.23  2003/02/03 16:53:14  dgrisby
   Force type in constructor argument to help confused compilers.
@@ -159,6 +168,7 @@
 
 #include <omniORB4/CORBA.h>
 #include <omniORB4/omniInterceptors.h>
+#include <omniORB4/omniURI.h>
 #include <exceptiondefs.h>
 #include <initialiser.h>
 #include <giopBiDir.h>
@@ -291,29 +301,8 @@ IIOP::encodeProfile(const IIOP::ProfileBody& body,IOP::TaggedProfile& profile)
 {
   profile.tag = IOP::TAG_INTERNET_IOP;
 
-  CORBA::ULong bufsize;
   {
-    cdrCountingStream s(orbParameters::anyCharCodeSet,
-			orbParameters::anyWCharCodeSet);
-    s.marshalOctet(omni::myByteOrder);
-    s.marshalOctet(body.version.major);
-    s.marshalOctet(body.version.minor);
-    s.marshalRawString(body.address.host);
-    body.address.port >>= s;
-    body.object_key >>= s;
-
-    if (body.version.minor > 0) {
-      CORBA::ULong total = body.components.length();
-      total >>= s;
-      for (CORBA::ULong index=0; index < total; index++) {
-	body.components[index] >>= s;
-      }
-    }
-    bufsize = s.total();
-  }
-
-  {
-    cdrEncapsulationStream s(bufsize,1);
+    cdrEncapsulationStream s((CORBA::ULong)0,1);
     s.marshalOctet(body.version.major);
     s.marshalOctet(body.version.minor);
     s.marshalRawString(body.address.host);
@@ -342,23 +331,8 @@ IIOP::encodeMultiComponentProfile(const IOP::MultipleComponentProfile& body,
 {
   profile.tag = IOP::TAG_MULTIPLE_COMPONENTS;
 
-  CORBA::ULong bufsize;
   {
-    cdrCountingStream s(orbParameters::anyCharCodeSet,
-			orbParameters::anyWCharCodeSet);
-    s.marshalOctet(omni::myByteOrder);
-    CORBA::ULong total = body.length();
-    if (total) {
-      total >>= s;
-      for (CORBA::ULong index=0; index < total; index++) {
-	body[index] >>= s;
-      }
-    }
-    bufsize = s.total();
-  }
-
-  {
-    cdrEncapsulationStream s(bufsize,1);
+    cdrEncapsulationStream s((CORBA::ULong)0,1);
     CORBA::ULong total = body.length();
     if (total) {
       total >>= s;
@@ -672,14 +646,11 @@ omniIOR::unmarshal_TAG_SSL_SEC_TRANS(const IOP::TaggedComponent& c ,
   }
   if (tcpaddr == 0) return;
 
-  const char* host = strchr(tcpaddr,':') + 1; host = strchr(host,':') + 1;
-  CORBA::ULong hostlen = strchr(host,':') - host;
-  CORBA::String_var copyhost(CORBA::string_alloc(hostlen));
-  strncpy(copyhost,host,hostlen);
-  ((char*)copyhost)[hostlen] = '\0';
+  CORBA::UShort     tcp_port;
+  CORBA::String_var tcp_host = omniURI::extractHostPort(tcpaddr+9, tcp_port);
 
   IIOP::Address ssladdr;
-  ssladdr.host = copyhost._retn();
+  ssladdr.host = tcp_host._retn();
   ssladdr.port = port;
   giopAddress* address = giopAddress::fromSslAddress(ssladdr);
   // If we do not have ssl transport linked the return value will be 0
@@ -807,8 +778,8 @@ omniIOR::unmarshal_TAG_OMNIORB_UNIX_TRANS(const IOP::TaggedComponent& c ,
   filename = e.unmarshalRawString();
 
   // Check if we are on the same host and hence can use unix socket.
-  char self[64];
-  if (gethostname(&self[0],64) == RC_SOCKET_ERROR) {
+  char self[OMNIORB_HOSTNAME_MAX];
+  if (gethostname(&self[0],OMNIORB_HOSTNAME_MAX) == RC_SOCKET_ERROR) {
     self[0] = '\0';
     omniORB::logs(1, "Cannot get the name of this host.");
   }
@@ -849,6 +820,114 @@ omniIOR::dump_TAG_OMNIORB_UNIX_TRANS(const IOP::TaggedComponent& c) {
 
 }
 
+//////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////////////
+void
+omniIOR::unmarshal_TAG_OMNIORB_PERSISTENT_ID(const IOP::TaggedComponent& c,
+					     omniIOR& ior)
+{
+  OMNIORB_ASSERT(c.tag == IOP::TAG_OMNIORB_PERSISTENT_ID);
+
+  CORBA::ULong len = orbParameters::persistentId.length();
+
+  if (len && len == c.component_data.length()) {
+
+    const CORBA::Octet* a = c.component_data.get_buffer();
+    const CORBA::Octet* b = orbParameters::persistentId.get_buffer();
+    for (CORBA::ULong i=0; i < len; i++) {
+      if (*a++ != *b++)
+	return;
+    }
+    
+    omniIOR::IORExtraInfoList& extra = ior.pd_iorInfo->extraInfo();
+    CORBA::ULong index = extra.length();
+    extra.length(index+1);
+    extra[index] = new IORExtraInfo(IOP::TAG_OMNIORB_PERSISTENT_ID);
+  }
+}
+
+
+char*
+omniIOR::dump_TAG_OMNIORB_PERSISTENT_ID(const IOP::TaggedComponent& c)
+{
+  OMNIORB_ASSERT(c.tag == IOP::TAG_OMNIORB_PERSISTENT_ID);
+
+  const char* prefix = "TAG_OMNIORB_PERSISTENT_ID ";
+
+  CORBA::String_var outstr;
+
+  CORBA::ULong prefix_len = strlen(prefix);
+  outstr = CORBA::string_alloc(prefix_len + c.component_data.length() * 2);
+
+  strcpy((char*)outstr, prefix);
+
+  char* o = (char*)outstr + prefix_len;
+
+  int d, n;
+  for (CORBA::ULong i=0; i < c.component_data.length(); i++) {
+    d = c.component_data[i];
+    n = (d & 0xf0) >> 4;
+    if (n >= 10)
+      *o++ = 'a' + n - 10;
+    else
+      *o++ = '0' + n;
+
+    n = d & 0xf;
+    if (n >= 10)
+      *o++ = 'a' + n - 10;
+    else
+      *o++ = '0' + n;
+  }
+  *o = '\0';
+
+  return outstr._retn();
+}
+
+static void
+logPersistentIdentifier()
+{
+  omniORB::logger l;
+  l << "Persistent server identifier: ";
+
+  int c, n;
+  for (CORBA::ULong i=0; i < orbParameters::persistentId.length(); i++) {
+    c = orbParameters::persistentId[i];
+    n = (c & 0xf0) >> 4;
+    if (n >= 10)
+      l << (char)('a' + n - 10);
+    else
+      l << (char)('0' + n);
+
+    n = c & 0xf;
+    if (n >= 10)
+      l << (char)('a' + n - 10);
+    else
+      l << (char)('0' + n);
+  }
+  l << "\n";
+}  
+
+
+void
+omniORB::
+setPersistentServerIdentifier(const _CORBA_Unbounded_Sequence_Octet& id)
+{
+  if (orbParameters::persistentId.length()) {
+    // Once set, it must not be changed
+    OMNIORB_THROW(INITIALIZE, INITIALIZE_FailedLoadLibrary,
+		  CORBA::COMPLETED_NO);
+  }
+
+  orbParameters::persistentId = id;
+
+  if (omniORB::trace(10)) {
+    logPersistentIdentifier();
+  }
+}
+
+
+
+
 OMNI_NAMESPACE_BEGIN(omni)
 
 //////////////////////////////////////////////////////////////////////////
@@ -918,6 +997,10 @@ static struct {
     omniIOR::unmarshal_TAG_OMNIORB_UNIX_TRANS,
     omniIOR::dump_TAG_OMNIORB_UNIX_TRANS },
 
+  { IOP::TAG_OMNIORB_PERSISTENT_ID,
+    omniIOR::unmarshal_TAG_OMNIORB_PERSISTENT_ID,
+    omniIOR::dump_TAG_OMNIORB_PERSISTENT_ID },
+
   { 0xffffffff, 0, 0 }
 };
 
@@ -1010,6 +1093,8 @@ static _CORBA_Unbounded_Sequence<_CORBA_Unbounded_Sequence_Octet> my_alternative
 static _CORBA_Unbounded_Sequence<_CORBA_Unbounded_Sequence_Octet> my_ssl_addr;
 static _CORBA_Unbounded_Sequence<_CORBA_Unbounded_Sequence_Octet> my_unix_addr;
 
+_CORBA_Unbounded_Sequence_Octet orbParameters::persistentId;
+
 OMNI_NAMESPACE_END(omni)
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1088,8 +1173,8 @@ omniIOR::add_TAG_OMNIORB_UNIX_TRANS(const char* filename) {
 
   OMNIORB_ASSERT(filename && strlen(filename) != 0);
 
-  char self[64];
-  if (gethostname(&self[0],64) == RC_SOCKET_ERROR) {
+  char self[OMNIORB_HOSTNAME_MAX];
+  if (gethostname(&self[0],OMNIORB_HOSTNAME_MAX) == RC_SOCKET_ERROR) {
     omniORB::logs(1, "Cannot get the name of this host.");
     self[0] = '\0';
   }
@@ -1193,6 +1278,17 @@ CORBA::Boolean insertSupportedComponents(omniInterceptors::encodeIOR_T::info_T&
     }
   }
 
+  if (v.major > 1 || v.minor >= 1) {
+    // 1.1 or later, insert omniORB persistent id
+    if (orbParameters::persistentId.length()) {
+      IOP::TaggedComponent& c = omniIOR::newIIOPtaggedComponent(cs);
+      c.tag = IOP::TAG_OMNIORB_PERSISTENT_ID;
+      c.component_data.replace(orbParameters::persistentId.maximum(),
+			       orbParameters::persistentId.length(),
+			       orbParameters::persistentId.get_buffer(), 0);
+    }
+  }
+
   return (info.default_only ? 0 : 1);
 }
 
@@ -1261,6 +1357,10 @@ public:
     _CORBA_Octet* p; CORBA::ULong max,len; s.getOctetStream(p,max,len);
     my_orb_type.replace(max,len,p,1);
 
+    if (omniORB::trace(10) && orbParameters::persistentId.length()) {
+      logPersistentIdentifier();
+    }
+
   }
 
   void detach() {
