@@ -29,42 +29,55 @@
 
 /*
   $Log: omniInternal.cc,v $
-  Revision 1.2.2.41  2005/12/01 23:14:24  dgrisby
-  Remove unsafe log message from native exception handler.
+  Revision 1.5.2.16  2007/04/14 17:56:52  dgrisby
+  Identity downcasting mechanism was broken by VC++ 8's
+  over-enthusiastic optimiser.
 
-  Revision 1.2.2.40  2005/11/28 14:57:51  dgrisby
-  New abortOnNativeException parameter for Windows.
+  Revision 1.5.2.15  2006/07/19 09:27:18  dgrisby
+  Properly pass hints when handling persistent id.
 
-  Revision 1.2.2.39  2005/09/19 14:23:56  dgrisby
-  New traceFile configuration parameter.
+  Revision 1.5.2.14  2006/07/18 16:21:21  dgrisby
+  New experimental connection management extension; ORB core support
+  for it.
 
-  Revision 1.2.2.38  2005/04/08 00:06:14  dgrisby
-  Remove all remaining uses of logf.
+  Revision 1.5.2.13  2006/06/02 12:48:32  dgrisby
+  Small code cleanups.
 
-  Revision 1.2.2.37  2005/03/29 14:41:33  dgrisby
-  Use omniORB logger instead of logf. May avoid a compiler / platform
-  bug on VxWorks.
+  Revision 1.5.2.12  2006/04/28 18:40:46  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.2.2.36  2004/10/17 20:14:33  dgrisby
-  Updated support for OpenVMS. Many thanks to Bruce Visscher.
+  Revision 1.5.2.11  2006/02/22 14:56:36  dgrisby
+  New endPointPublishHostname and endPointResolveNames parameters.
 
-  Revision 1.2.2.35  2004/07/23 13:25:44  dgrisby
-  New traceExceptions option.
+  Revision 1.5.2.10  2006/01/23 16:05:41  dgrisby
+  Another merge from omni4_0_develop.
 
-  Revision 1.2.2.34  2004/03/02 15:31:22  dgrisby
-  Support for persistent server identifier.
+  Revision 1.5.2.9  2006/01/10 12:24:03  dgrisby
+  Merge from omni4_0_develop pre 4.0.7 release.
 
-  Revision 1.2.2.33  2003/08/06 20:36:49  dgrisby
-  Shorten assertion failure message.
+  Revision 1.5.2.8  2005/12/08 14:22:31  dgrisby
+  Better string marshalling performance; other minor optimisations.
 
-  Revision 1.2.2.32  2003/07/02 11:01:06  dgrisby
-  Race condition in POA destruction.
+  Revision 1.5.2.7  2005/09/19 18:26:33  dgrisby
+  Merge from omni4_0_develop again.
 
-  Revision 1.2.2.31  2003/05/28 09:59:25  dgrisby
-  HPUX fixes broke some platforms.
+  Revision 1.5.2.6  2005/04/14 00:03:58  dgrisby
+  New traceInvocationReturns and traceTime options; remove logf function.
 
-  Revision 1.2.2.30  2003/05/22 13:41:41  dgrisby
-  HPUX patches.
+  Revision 1.5.2.5  2005/04/08 00:35:46  dgrisby
+  Merging again.
+
+  Revision 1.5.2.4  2005/03/30 23:36:09  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.5.2.3  2005/01/06 23:10:36  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.5.2.2  2003/05/20 16:53:17  dgrisby
+  Valuetype marshalling support.
+
+  Revision 1.5.2.1  2003/03/23 21:02:09  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.2.2.29  2003/01/16 11:08:26  dgrisby
   Patches to support Digital Mars C++. Thanks Christof Meerwald.
@@ -244,7 +257,11 @@
 #include <omniORB4/objTracker.h>
 #include <corbaOrb.h>
 
-#ifdef __WIN32__
+#if defined(__WIN32__) && !defined(__GNUC__)
+#  define WIN32_EXCEPTION_HANDLING
+#endif
+
+#ifdef WIN32_EXCEPTION_HANDLING
 #  include <eh.h>
 #endif
 
@@ -275,11 +292,23 @@ CORBA::Boolean  omniORB::traceInvocations = 0;
 //
 //    Valid values = 0 or 1
 
+CORBA::Boolean  omniORB::traceInvocationReturns = 0;
+//    If true, then each local and remote invocation will generate a trace 
+//    message when it returns.
+//
+//    Valid values = 0 or 1
+
 CORBA::Boolean  omniORB::traceThreadId = 0;
 //    If true, then the log messages will contain the thread id.
 //
 //    Valid values = 0 or 1
 
+CORBA::Boolean  omniORB::traceTime = 0;
+//    If true, then the log messages will contain the time.
+//
+//    Valid values = 0 or 1
+
+
 
 const CORBA::Char                omni::myByteOrder = _OMNIORB_HOST_BYTE_ORDER_;
 omni_tracedmutex*                omni::internalLock = 0;
@@ -449,19 +478,6 @@ omniObjTable::resize()
 //////////////////////////////// omni ////////////////////////////////
 //////////////////////////////////////////////////////////////////////
 
-_CORBA_ULong
-omni::hash(const CORBA::Octet* key, int keysize)
-{
-  //?? This is from Knuth.  We may be able to do better!
-
-  _CORBA_ULong n = 0;
-
-  while( keysize-- )  n = ((n << 5) ^ (n >> 27)) ^ *key++;
-
-  return n;
-}
-
-
 omni_tracedmutex&
 omni::nilRefLock()
 {
@@ -838,20 +854,18 @@ omniObjTableEntry::loseRef(omniObjRef* objref)
   delete this;
 }
 
+
 void*
-omniObjTableEntry::thisClassCompare(omniIdentity* id, void* vfn)
+omniObjTableEntry::ptrToClass(int* cptr)
 {
-  classCompare_fn fn = (classCompare_fn)vfn;
-
-  if (fn == omniObjTableEntry::thisClassCompare)
-    return (omniObjTableEntry*)id;
-
-  if (fn == omniLocalIdentity::thisClassCompare)
-    return (omniLocalIdentity*)id;
-
+  if (cptr == &omniObjTableEntry::_classid) return (omniObjTableEntry*)this;
+  if (cptr == &omniLocalIdentity::_classid) return (omniLocalIdentity*)this;
+  if (cptr == &omniIdentity     ::_classid) return (omniIdentity*)     this;
   return 0;
 }
 
+int omniObjTableEntry::_classid;
+
 
 
 omniIdentity*
@@ -912,8 +926,7 @@ omni::createIdentity(omniIOR* ior, const char* target, CORBA::Boolean locked)
   CORBA::Boolean is_local = 0;
   Rope* rope;
 
-  if (_OMNI_NS(giopRope)::selectRope(ior->getIORInfo()->addresses(),
-				     info,rope,is_local) == 0) {
+  if (giopRope::selectRope(info->addresses(), info, rope, is_local) == 0) {
     return 0;
   }
 
@@ -961,13 +974,6 @@ omni::createInProcessIdentity(const _CORBA_Octet* key, int keysize) {
 }
 
 
-omniIdentity*
-omni::createLoopBackIdentity(omniIOR* ior,const _CORBA_Octet* key,int keysize){
-  Rope* rope = omniObjAdapter::defaultLoopBack();
-  rope->incrRefCount();
-  return new omniRemoteIdentity(ior,key,keysize,rope);
-}
-
 omniObjRef*
 omni::createObjRef(const char* targetRepoId,
 		   omniIOR* ior,
@@ -1073,11 +1079,12 @@ omni::createObjRef(const char* targetRepoId,
 	  omniORB::logs(15, "Re-write local persistent object reference.");
 
 	  omniObjRef* new_objref;
+	  omniIORHints hints(0);
 	  {
 	    omni_optional_lock sync(*internalLock, locked, locked);
 
 	    omniIOR* new_ior = new omniIOR(ior->repositoryID(),
-					   id->key(), id->keysize());
+					   id->key(), id->keysize(), hints);
 
 	    new_objref = createObjRef(targetRepoId, new_ior, 1, 0);
 	  }
@@ -1094,7 +1101,8 @@ omni::createObjRef(const char* targetRepoId,
 omniObjRef*
 omni::createLocalObjRef(const char* mostDerivedRepoId,
 			const char* targetRepoId,
-			omniObjTableEntry* entry)
+			omniObjTableEntry* entry,
+			const omniIORHints& hints)
 {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*internalLock, 1);
   OMNIORB_ASSERT(targetRepoId);
@@ -1134,7 +1142,7 @@ omni::createLocalObjRef(const char* mostDerivedRepoId,
   }
   // Reach here if we have to create a new objref.
   omniIOR* ior = new omniIOR(mostDerivedRepoId,
-			     entry->key(), entry->keysize());
+			     entry->key(), entry->keysize(), hints);
 
   return createObjRef(targetRepoId,ior,1,entry);
 }
@@ -1142,7 +1150,8 @@ omni::createLocalObjRef(const char* mostDerivedRepoId,
 omniObjRef*
 omni::createLocalObjRef(const char* mostDerivedRepoId,
 			const char* targetRepoId,
-			const _CORBA_Octet* key, int keysize)
+			const _CORBA_Octet* key, int keysize,
+			const omniIORHints& hints)
 {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*internalLock, 1);
   OMNIORB_ASSERT(targetRepoId);
@@ -1155,9 +1164,9 @@ omni::createLocalObjRef(const char* mostDerivedRepoId,
 							hashv, 0);
 
   if (entry)
-    return createLocalObjRef(mostDerivedRepoId, targetRepoId, entry);
+    return createLocalObjRef(mostDerivedRepoId, targetRepoId, entry, hints);
 
-  omniIOR* ior = new omniIOR(mostDerivedRepoId, key, keysize);
+  omniIOR* ior = new omniIOR(mostDerivedRepoId, key, keysize, hints);
 
   return createObjRef(targetRepoId,ior,1,entry);
 }
@@ -1278,8 +1287,8 @@ omni::assertFail(const char* file, int line, const char* expr)
 {
   if( omniORB::trace(1) ) {
     omniORB::logger l;
-    l << "Assertion failed.  This indicates a bug in the application using\n"
-      "omniORB, or maybe in omniORB itself.\n"
+    l << "Assertion failed.  This indicates a bug in the application\n"
+      "using omniORB, or maybe in omniORB itself.\n"
       " file: " << file << "\n"
       " line: " << line << "\n"
       " info: " << expr << "\n";
@@ -1394,6 +1403,35 @@ public:
 
 static traceInvocationsHandler traceInvocationsHandler_;
 
+/////////////////////////////////////////////////////////////////////////////
+class traceInvocationReturnsHandler : public orbOptions::Handler {
+public:
+
+  traceInvocationReturnsHandler() : 
+    orbOptions::Handler("traceInvocationReturns",
+			"traceInvocationReturns = 0 or 1",
+			1,
+			"-ORBtraceInvocationReturns < 0 | 1 >") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+
+    CORBA::Boolean v;
+    if (!orbOptions::getBoolean(value,v)) {
+      throw orbOptions::BadParam(key(),value,
+				 orbOptions::expect_boolean_msg);
+    }
+    omniORB::traceInvocationReturns = v;
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVBoolean(key(),omniORB::traceInvocationReturns,
+			     result);
+  }
+};
+
+static traceInvocationReturnsHandler traceInvocationReturnsHandler_;
+
 /////////////////////////////////////////////////////////////////////////////
 class traceThreadIdHandler : public orbOptions::Handler {
 public:
@@ -1423,6 +1461,35 @@ public:
 
 static traceThreadIdHandler traceThreadIdHandler_;
 
+/////////////////////////////////////////////////////////////////////////////
+class traceTimeHandler : public orbOptions::Handler {
+public:
+
+  traceTimeHandler() : 
+    orbOptions::Handler("traceTime",
+			"traceTime = 0 or 1",
+			1,
+			"-ORBtraceTime < 0 | 1 >") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+
+    CORBA::Boolean v;
+    if (!orbOptions::getBoolean(value,v)) {
+      throw orbOptions::BadParam(key(),value,
+				 orbOptions::expect_boolean_msg);
+    }
+    omniORB::traceTime = v;
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVBoolean(key(),omniORB::traceTime,
+			     result);
+  }
+};
+
+static traceTimeHandler traceTimeHandler_;
+
 /////////////////////////////////////////////////////////////////////////////
 class traceFileHandler : public orbOptions::Handler {
 public:
@@ -1533,7 +1600,7 @@ public:
 static abortOnNativeExceptionHandler abortOnNativeExceptionHandler_;
 
 
-#ifdef __WIN32__
+#ifdef WIN32_EXCEPTION_HANDLING
 extern "C" void omniORB_rethrow_exception(unsigned, EXCEPTION_POINTERS*)
 {
   throw;
@@ -1559,7 +1626,9 @@ public:
     orbOptions::singleton().registerHandler(traceLevelHandler_);
     orbOptions::singleton().registerHandler(traceExceptionsHandler_);
     orbOptions::singleton().registerHandler(traceInvocationsHandler_);
+    orbOptions::singleton().registerHandler(traceInvocationReturnsHandler_);
     orbOptions::singleton().registerHandler(traceThreadIdHandler_);
+    orbOptions::singleton().registerHandler(traceTimeHandler_);
     orbOptions::singleton().registerHandler(traceFileHandler_);
     orbOptions::singleton().registerHandler(objectTableSizeHandler_);
     orbOptions::singleton().registerHandler(abortOnInternalErrorHandler_);
@@ -1587,7 +1656,7 @@ public:
     objectTable = new omniObjTableEntry* [objectTableSize];
     for( CORBA::ULong i = 0; i < objectTableSize; i++ )  objectTable[i] = 0;
 
-#ifdef __WIN32__
+#ifdef WIN32_EXCEPTION_HANDLING
     if (abortOnNativeException) {
       omniInterceptors* interceptors = omniORB::getInterceptors();
       interceptors->createThread.add(abortOnNativeExceptionInterceptor);
@@ -1596,6 +1665,13 @@ public:
   }
 
   void detach() {
+    if (numObjectsInTable && omniORB::trace(1)) {
+      omniORB::logger l;
+      l << "Error: the object table still contains "
+	<< numObjectsInTable << " entr"
+	<< (numObjectsInTable == 1 ? "y" : "ies")
+	<< " at ORB shutdown time.";
+    }
     OMNIORB_ASSERT(numObjectsInTable == 0);
     delete [] objectTable;
     objectTable = 0;
