@@ -29,27 +29,34 @@
 
 /*
   $Log: sslConnection.cc,v $
-  Revision 1.1.2.18  2005/09/05 14:31:08  dgrisby
-  SSL transport extensions from Jon Biggar; SSL command line options.
+  Revision 1.1.4.9  2006/10/09 13:08:58  dgrisby
+  Rename SOCKADDR_STORAGE define to OMNI_SOCKADDR_STORAGE, to avoid
+  clash on Win32 2003 SDK.
 
-  Revision 1.1.2.17  2005/08/23 11:45:05  dgrisby
-  New maxSocketSend and maxSocketRecv parameters.
+  Revision 1.1.4.8  2006/03/25 18:54:03  dgrisby
+  Initial IPv6 support.
 
-  Revision 1.1.2.16  2005/03/10 11:28:28  dgrisby
-  Race condition between setSelectable / clearSelectable.
+  Revision 1.1.4.7  2005/09/05 17:12:20  dgrisby
+  Merge again. Mainly SSL transport changes.
 
-  Revision 1.1.2.15  2005/02/23 12:27:31  dgrisby
-  Another race in setSelectable with connection shutdown. Thanks Peter
-  Klotz.
+  Revision 1.1.4.6  2005/09/01 14:52:12  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.2.14  2004/04/08 10:02:21  dgrisby
-  In thread pool mode, close connections that will not be selectable.
+  Revision 1.1.4.5  2005/03/02 12:39:17  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.2.13  2003/11/12 16:04:17  dgrisby
-  Set sockets to close on exec.
+  Revision 1.1.4.4  2005/03/02 12:10:49  dgrisby
+  setSelectable / Peek fixes.
 
-  Revision 1.1.2.12  2003/04/15 10:40:47  dgrisby
-  Timeouts ignored in some cases when scan granularity set to zero.
+  Revision 1.1.4.3  2005/01/13 21:10:01  dgrisby
+  New SocketCollection implementation, using poll() where available and
+  select() otherwise. Windows specific version to follow.
+
+  Revision 1.1.4.2  2005/01/06 23:10:52  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.4.1  2003/03/23 21:01:59  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.11  2001/12/03 13:39:55  dpg1
   Explicit socket shutdown flag for Windows.
@@ -213,12 +220,8 @@ sslConnection::Recv(void* buf, size_t sz,
 
   do {
 
-#ifdef NEED_SOCKET_SHUTDOWN_FLAG
-    // Unfortunately, select() on Windows does not return an error
-    // after the socket has shutdown. We have to use this hack.
     if (pd_shutdown)
       return -1;
-#endif
 
     struct timeval t;
 
@@ -321,9 +324,7 @@ sslConnection::Shutdown() {
   SSL_set_shutdown(pd_ssl, SSL_SENT_SHUTDOWN | SSL_RECEIVED_SHUTDOWN);
   SSL_shutdown(pd_ssl);
   SHUTDOWNSOCKET(pd_socket);
-#ifdef NEED_SOCKET_SHUTDOWN_FLAG
   pd_shutdown = 1;
-#endif
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -346,31 +347,27 @@ sslConnection::peeridentity() {
 /////////////////////////////////////////////////////////////////////////
 sslConnection::sslConnection(SocketHandle_t sock,::SSL* ssl, 
 			     SocketCollection* belong_to) : 
-  SocketLink(sock), pd_ssl(ssl), pd_belong_to(belong_to) {
+  SocketHolder(sock), pd_ssl(ssl) {
 
-  struct sockaddr_in addr;
+  OMNI_SOCKADDR_STORAGE addr;
   SOCKNAME_SIZE_T l;
 
-  l = sizeof(struct sockaddr_in);
+  l = sizeof(OMNI_SOCKADDR_STORAGE);
   if (getsockname(pd_socket,
 		  (struct sockaddr *)&addr,&l) == RC_SOCKET_ERROR) {
     pd_myaddress = (const char*)"giop:ssl:255.255.255.255:65535";
   }
   else {
-    pd_myaddress = tcpConnection::ip4ToString(
-			     (CORBA::ULong)addr.sin_addr.s_addr,
-			     (CORBA::UShort)addr.sin_port,"giop:ssl:");
+    pd_myaddress = tcpConnection::addrToURI((sockaddr*)&addr, "giop:ssl:");
   }
 
-  l = sizeof(struct sockaddr_in);
+  l = sizeof(OMNI_SOCKADDR_STORAGE);
   if (getpeername(pd_socket,
 		  (struct sockaddr *)&addr,&l) == RC_SOCKET_ERROR) {
     pd_peeraddress = (const char*)"giop:ssl:255.255.255.255:65535";
   }
   else {
-    pd_peeraddress = tcpConnection::ip4ToString(
-			       (CORBA::ULong)addr.sin_addr.s_addr,
-			       (CORBA::UShort)addr.sin_port,"giop:ssl:");
+    pd_peeraddress = tcpConnection::addrToURI((sockaddr*)&addr, "giop:ssl:");
   }
   SocketSetCloseOnExec(sock);
 
@@ -396,8 +393,8 @@ sslConnection::sslConnection(SocketHandle_t sock,::SSL* ssl,
 /////////////////////////////////////////////////////////////////////////
 sslConnection::~sslConnection() {
 
-  pd_belong_to->clearSelectable(pd_socket);
-  pd_belong_to->removeSocket(pd_socket);
+  clearSelectable();
+  pd_belong_to->removeSocket(this);
 
   if(pd_ssl != 0) {
     if (SSL_get_shutdown(pd_ssl) == 0) {
@@ -418,7 +415,7 @@ sslConnection::setSelectable(int now,
 
   if (SSL_pending(ssl_handle())) data_in_buffer = 1;
 
-  pd_belong_to->setSelectable(pd_socket,now,data_in_buffer);
+  SocketHolder::setSelectable(now,data_in_buffer);
 }
 
 
@@ -426,7 +423,7 @@ sslConnection::setSelectable(int now,
 void
 sslConnection::clearSelectable() {
 
-  pd_belong_to->clearSelectable(pd_socket);
+  SocketHolder::clearSelectable();
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -437,17 +434,13 @@ sslConnection::isSelectable() {
 
 
 /////////////////////////////////////////////////////////////////////////
-void
-sslConnection::Peek(giopConnection::notifyReadable_t func, void* cookie) {
+CORBA::Boolean
+sslConnection::Peek() {
 
   if (SSL_pending(ssl_handle())) {
-    func(cookie,this);
-    return;
-  }
-
-  if (pd_belong_to->Peek(pd_socket)) {
-    func(cookie,this);
+    return 1;
   }
+  return SocketHolder::Peek();
 }
 
 
