@@ -29,14 +29,17 @@
 
 /*
   $Log: sslContext.cc,v $
-  Revision 1.1.2.15  2005/09/05 14:31:08  dgrisby
-  SSL transport extensions from Jon Biggar; SSL command line options.
+  Revision 1.1.4.4  2005/09/05 17:12:20  dgrisby
+  Merge again. Mainly SSL transport changes.
 
-  Revision 1.1.2.14  2005/03/14 13:37:09  dgrisby
-  Reorder includes for compilers with pre-compiled headers.
+  Revision 1.1.4.3  2005/03/30 23:35:59  dgrisby
+  Another merge from omni4_0_develop.
 
-  Revision 1.1.2.13  2004/02/11 15:44:54  dgrisby
-  Hook to set SSL verify mode. Thanks Matthew Wood.
+  Revision 1.1.4.2  2005/01/06 23:10:53  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.4.1  2003/03/23 21:01:59  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.12  2002/12/19 12:23:02  dgrisby
   Don't set SSL verify depth to 1.
