@@ -29,11 +29,17 @@
 
 /*
   $Log: sslTransportImpl.cc,v $
-  Revision 1.1.2.11  2005/09/05 14:31:08  dgrisby
-  SSL transport extensions from Jon Biggar; SSL command line options.
+  Revision 1.1.4.4  2006/03/25 18:54:03  dgrisby
+  Initial IPv6 support.
 
-  Revision 1.1.2.10  2005/03/14 13:37:09  dgrisby
-  Reorder includes for compilers with pre-compiled headers.
+  Revision 1.1.4.3  2005/09/05 17:12:20  dgrisby
+  Merge again. Mainly SSL transport changes.
+
+  Revision 1.1.4.2  2005/03/30 23:35:58  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.1.4.1  2003/03/23 21:01:59  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.9  2002/09/09 22:11:51  dgrisby
   SSL transport cleanup even if certificates are wrong.
@@ -70,6 +76,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <omniORB4/giopEndpoint.h>
+#include <omniORB4/omniURI.h>
 #include <objectAdapter.h>
 #include <omniORB4/sslContext.h>
 #include <ssl/sslConnection.h>
@@ -108,40 +115,29 @@ sslTransportImpl::~sslTransportImpl() {
 giopEndpoint*
 sslTransportImpl::toEndpoint(const char* param) {
 
-  const char* p = strchr(param,':');
-  if (!p) return 0;
   IIOP::Address address;
-  if (param == p) {
+
+  char* host = omniURI::extractHostPort(param, address.port);
+  if (!host)
+    return 0;
+
+  if (*host == '\0') {
+    // No name in param -- try environment variable.
     const char* hostname = getenv(OMNIORB_USEHOSTNAME_VAR);
-    if (hostname) address.host = hostname;
+    if (hostname)
+      address.host = hostname;
   }
   else {
-    address.host = CORBA::string_alloc(p-param);
-    strncpy(address.host,param,p-param);
-    ((char*)address.host)[p-param] = '\0';
+    address.host = host;
   }
-  if (*(++p) != '\0') {
-    int v;
-    if (sscanf(p,"%d",&v) != 1) return 0;
-    if (v < 0 || v > 65536) return 0;
-    address.port = v;
-  }
-  else {
-    address.port = 0;
-  }
-  return (giopEndpoint*)(new sslEndpoint(address,pd_ctx));
+  return (giopEndpoint*)(new sslEndpoint(address, pd_ctx));
 }
 
 /////////////////////////////////////////////////////////////////////////
 CORBA::Boolean
 sslTransportImpl::isValid(const char* param) {
   
-  const char* p = strchr(param,':');
-  if (!p || param == p || *p == '\0') return 0;
-  int v;
-  if (sscanf(p+1,"%d",&v) != 1) return 0;
-  if (v < 0 || v > 65536) return 0;
-  return 1;
+  return omniURI::validHostPort(param);
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -149,16 +145,11 @@ static
 CORBA::Boolean
 parseAddress(const char* param, IIOP::Address& address) {
 
-  const char* p = strchr(param,':');
-  if (!p || param == p || *p == '\0') return 0;
-  address.host = CORBA::string_alloc(p-param);
-  strncpy(address.host,param,p-param);
-  ((char*)address.host)[p-param] = '\0';
-  ++p;
-  int v;
-  if (sscanf(p,"%d",&v) != 1) return 0;
-  if (v < 0 || v > 65536) return 0;
-  address.port = v;
+  char* host = omniURI::extractHostPort(param, address.port);
+  if (!host)
+    return 0;
+
+  address.host = host;
   return 1;
 }
 
@@ -182,9 +173,10 @@ sslTransportImpl::addToIOR(const char* param) {
   IIOP::Address address;
   if (parseAddress(param,address)) {
     // XXX, hardwared security options to:
+    //       Integrity (0x02) Confidentiality (0x04) |
     //       EstablishTrustInTarget (0x20) | EstablishTrustInClient (0x40)
     // In future, this will be expanded configurable options.
-    omniIOR::add_TAG_SSL_SEC_TRANS(address,0x60,0x60);
+    omniIOR::add_TAG_SSL_SEC_TRANS(address,0x66,0x66);
     return 1;
   }
   return 0;
