@@ -29,27 +29,31 @@
 
 /*
   $Log: tcpConnection.cc,v $
-  Revision 1.1.2.18  2005/08/23 11:45:05  dgrisby
-  New maxSocketSend and maxSocketRecv parameters.
+  Revision 1.1.4.8  2006/10/09 13:08:58  dgrisby
+  Rename SOCKADDR_STORAGE define to OMNI_SOCKADDR_STORAGE, to avoid
+  clash on Win32 2003 SDK.
 
-  Revision 1.1.2.17  2005/03/10 11:28:28  dgrisby
-  Race condition between setSelectable / clearSelectable.
+  Revision 1.1.4.7  2006/03/25 18:54:03  dgrisby
+  Initial IPv6 support.
 
-  Revision 1.1.2.16  2005/02/23 12:27:31  dgrisby
-  Another race in setSelectable with connection shutdown. Thanks Peter
-  Klotz.
+  Revision 1.1.4.6  2005/09/01 14:52:12  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.2.15  2004/04/08 10:02:22  dgrisby
-  In thread pool mode, close connections that will not be selectable.
+  Revision 1.1.4.5  2005/03/02 12:39:17  dgrisby
+  Merge from omni4_0_develop.
 
-  Revision 1.1.2.14  2003/11/12 16:04:17  dgrisby
-  Set sockets to close on exec.
+  Revision 1.1.4.4  2005/03/02 12:10:48  dgrisby
+  setSelectable / Peek fixes.
 
-  Revision 1.1.2.13  2003/07/25 16:04:57  dgrisby
-  vxWorks patches.
+  Revision 1.1.4.3  2005/01/13 21:10:03  dgrisby
+  New SocketCollection implementation, using poll() where available and
+  select() otherwise. Windows specific version to follow.
 
-  Revision 1.1.2.12  2003/04/15 10:40:47  dgrisby
-  Timeouts ignored in some cases when scan granularity set to zero.
+  Revision 1.1.4.2  2005/01/06 23:10:55  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.4.1  2003/03/23 21:01:58  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.2.11  2003/01/06 11:11:55  dgrisby
   New AddrInfo instead of gethostbyname.
@@ -94,14 +98,19 @@
 
 #include <omniORB4/CORBA.h>
 #include <omniORB4/giopEndpoint.h>
+#include <omniORB4/omniURI.h>
 #include <orbParameters.h>
 #include <SocketCollection.h>
 #include <tcp/tcpConnection.h>
 #include <stdio.h>
 #include <omniORB4/linkHacks.h>
 
+#if defined(NTArchitecture)
+#  include <ws2tcpip.h>
+#endif
+
 #if defined(__vxWorks__)
-#include "selectLib.h"
+#  include "selectLib.h"
 #endif
 
 OMNI_EXPORT_LINK_FORCE_SYMBOL(tcpConnection);
@@ -110,7 +119,56 @@ OMNI_NAMESPACE_BEGIN(omni)
 
 /////////////////////////////////////////////////////////////////////////
 char*
-tcpConnection::ip4ToString(CORBA::ULong ipv4) {
+tcpConnection::addrToString(sockaddr* addr)
+{
+#if defined(HAVE_INET_NTOP)
+
+  char dest[80];
+  const char* addrstr;
+
+  if (addr->sa_family == AF_INET) {
+    sockaddr_in* addr_in = (sockaddr_in*)addr;
+    addrstr = inet_ntop(AF_INET, &addr_in->sin_addr, dest, sizeof(dest));
+  }
+#if defined(OMNI_SUPPORT_IPV6)
+  else {
+    OMNIORB_ASSERT(addr->sa_family == AF_INET6);
+    sockaddr_in6* addr_in6 = (sockaddr_in6*)addr;
+    addrstr = inet_ntop(AF_INET6, &addr_in6->sin6_addr, dest, sizeof(dest));
+  }
+#endif
+  OMNIORB_ASSERT(addrstr);
+
+  return CORBA::string_dup(addrstr);
+
+#elif defined (HAVE_GETNAMEINFO)
+
+  char dest[80];
+  socklen_t addrlen = 0;
+
+  if (addr->sa_family == AF_INET) {
+    addrlen = sizeof(sockaddr_in);
+  }
+#if defined(OMNI_SUPPORT_IPV6)
+  else {
+    OMNIORB_ASSERT(addr->sa_family == AF_INET6);
+    addrlen = sizeof(sockaddr_in6);
+  }
+#endif
+  OMNIORB_ASSERT(addrlen);
+
+  int result = getnameinfo(addr, addrlen, dest, sizeof(dest), 0, 0,
+			   NI_NUMERICHOST);
+  if (result != 0) {
+    omniORB::logs(1, "Unable to convert IP address to a string!");
+    return CORBA::string_dup("**invalid**");
+  }
+  return CORBA::string_dup(dest);
+
+#else
+
+  OMNIORB_ASSERT(addr->sa_family == AF_INET);
+  CORBA::ULong ipv4 = ((sockaddr_in*)addr)->sin_addr.s_addr;
   CORBA::ULong hipv4 = ntohl(ipv4);
   int ip1 = (int)((hipv4 & 0xff000000) >> 24);
   int ip2 = (int)((hipv4 & 0x00ff0000) >> 16);
@@ -120,33 +178,105 @@ tcpConnection::ip4ToString(CORBA::ULong ipv4) {
   char* result = CORBA::string_alloc(16);
   sprintf(result,"%d.%d.%d.%d",ip1,ip2,ip3,ip4);
   return result;
+
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////////
 char*
-tcpConnection::ip4ToString(CORBA::ULong ipv4,CORBA::UShort port,
-			   const char* prefix) {
+tcpConnection::addrToURI(sockaddr* addr, const char* prefix)
+{
+#if defined(HAVE_INET_NTOP)
 
+  char dest[80];
+  int port;
+  const char* addrstr;
+
+  if (addr->sa_family == AF_INET) {
+    sockaddr_in* addr_in = (sockaddr_in*)addr;
+    port = ntohs(addr_in->sin_port);
+    addrstr = inet_ntop(AF_INET, &addr_in->sin_addr, dest, sizeof(dest));
+  }
+#if defined(OMNI_SUPPORT_IPV6)
+  else {
+    OMNIORB_ASSERT(addr->sa_family == AF_INET6);
+    sockaddr_in6* addr_in6 = (sockaddr_in6*)addr;
+    port = ntohs(addr_in6->sin6_port);
+    addrstr = inet_ntop(AF_INET6, &addr_in6->sin6_addr, dest, sizeof(dest));
+  }
+#endif
+  OMNIORB_ASSERT(addrstr);
+
+  return omniURI::buildURI(prefix, addrstr, port);
+
+#elif defined (HAVE_GETNAMEINFO)
+
+  char dest[80];
+  int port;
+  socklen_t addrlen = 0;
+
+  if (addr->sa_family == AF_INET) {
+    addrlen = sizeof(sockaddr_in);
+    sockaddr_in* addr_in = (sockaddr_in*)addr;
+    port = ntohs(addr_in->sin_port);
+  }
+#if defined(OMNI_SUPPORT_IPV6)
+  else {
+    OMNIORB_ASSERT(addr->sa_family == AF_INET6);
+    addrlen = sizeof(sockaddr_in6);
+    sockaddr_in6* addr_in6 = (sockaddr_in6*)addr;
+    port = ntohs(addr_in6->sin6_port);
+  }
+#endif
+  OMNIORB_ASSERT(addrlen);
+
+  int result = getnameinfo(addr, addrlen, dest, sizeof(dest), 0, 0,
+			   NI_NUMERICHOST);
+  if (result != 0) {
+    omniORB::logs(1, "Unable to convert IP address to a string!");
+    return CORBA::string_dup("**invalid**");
+  }
+  return omniURI::buildURI(prefix, dest, port);
+
+#else
+
+  OMNIORB_ASSERT(addr->sa_family == AF_INET);
+  sockaddr_in* addr_in = (sockaddr_in*)addr;
+
+  CORBA::ULong ipv4 = addr_in->sin_addr.s_addr;
   CORBA::ULong hipv4 = ntohl(ipv4);
   int ip1 = (int)((hipv4 & 0xff000000) >> 24);
   int ip2 = (int)((hipv4 & 0x00ff0000) >> 16);
   int ip3 = (int)((hipv4 & 0x0000ff00) >> 8);
   int ip4 = (int)((hipv4 & 0x000000ff));
+  int port = ntohs(addr_in->sin_port);
 
-  char* result;
-  if (!prefix) {
-    result = CORBA::string_alloc(24);
-    sprintf((char*)result,"%d.%d.%d.%d:%d",ip1,ip2,ip3,ip4,ntohs(port));
-
-  }
-  else {
-    result = CORBA::string_alloc(24+strlen(prefix));
-    sprintf((char*)result,"%s%d.%d.%d.%d:%d",prefix,ip1,ip2,ip3,ip4,
-	    ntohs(port));
-  }
+  char* result = CORBA::string_alloc(strlen(prefix) + 24);
+  sprintf(result,"%s%d.%d.%d.%d:%d",prefix, ip1, ip2, ip3, ip4, port);
   return result;
+
+#endif
 }
 
+/////////////////////////////////////////////////////////////////////////
+CORBA::UShort
+tcpConnection::addrToPort(sockaddr* addr)
+{
+  if (addr->sa_family == AF_INET) {
+    sockaddr_in* addr_in = (sockaddr_in*)addr;
+    return ntohs(addr_in->sin_port);
+  }
+#if defined(OMNI_SUPPORT_IPV6)
+  else {
+    OMNIORB_ASSERT(addr->sa_family == AF_INET6);
+    sockaddr_in6* addr_in6 = (sockaddr_in6*)addr;
+    return ntohs(addr_in6->sin6_port);
+  }
+#else
+  OMNIORB_ASSERT(0);
+  return 0;
+#endif
+}
 
 /////////////////////////////////////////////////////////////////////////
 int
@@ -229,12 +359,8 @@ tcpConnection::Recv(void* buf, size_t sz,
 
   do {
 
-#ifdef NEED_SOCKET_SHUTDOWN_FLAG
-    // Unfortunately, select() on Windows does not return an error
-    // after the socket has shutdown. We have to use this hack.
     if (pd_shutdown)
       return -1;
-#endif
 
     struct timeval t;
 
@@ -313,9 +439,7 @@ tcpConnection::Recv(void* buf, size_t sz,
 void
 tcpConnection::Shutdown() {
   SHUTDOWNSOCKET(pd_socket);
-#ifdef NEED_SOCKET_SHUTDOWN_FLAG
   pd_shutdown = 1;
-#endif
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -333,29 +457,27 @@ tcpConnection::peeraddress() {
 /////////////////////////////////////////////////////////////////////////
 tcpConnection::tcpConnection(SocketHandle_t sock,
 			     SocketCollection* belong_to) :
-  SocketLink(sock), pd_belong_to(belong_to) {
+  SocketHolder(sock) {
 
-  struct sockaddr_in addr;
+  OMNI_SOCKADDR_STORAGE addr;
   SOCKNAME_SIZE_T l;
 
-  l = sizeof(struct sockaddr_in);
+  l = sizeof(OMNI_SOCKADDR_STORAGE);
   if (getsockname(pd_socket,
 		  (struct sockaddr *)&addr,&l) == RC_SOCKET_ERROR) {
     pd_myaddress = (const char*)"giop:tcp:255.255.255.255:65535";
   }
   else {
-    pd_myaddress = ip4ToString((CORBA::ULong)addr.sin_addr.s_addr,
-			       (CORBA::UShort)addr.sin_port,"giop:tcp:");
+    pd_myaddress = addrToURI((sockaddr*)&addr, "giop:tcp:");
   }
 
-  l = sizeof(struct sockaddr_in);
+  l = sizeof(OMNI_SOCKADDR_STORAGE);
   if (getpeername(pd_socket,
 		  (struct sockaddr *)&addr,&l) == RC_SOCKET_ERROR) {
     pd_peeraddress = (const char*)"giop:tcp:255.255.255.255:65535";
   }
   else {
-    pd_peeraddress = ip4ToString((CORBA::ULong)addr.sin_addr.s_addr,
-				 (CORBA::UShort)addr.sin_port,"giop:tcp:");
+    pd_peeraddress = addrToURI((sockaddr*)&addr, "giop:tcp:");
   }
 
   SocketSetCloseOnExec(sock);
@@ -365,8 +487,8 @@ tcpConnection::tcpConnection(SocketHandle_t sock,
 
 /////////////////////////////////////////////////////////////////////////
 tcpConnection::~tcpConnection() {
-  pd_belong_to->clearSelectable(pd_socket);
-  pd_belong_to->removeSocket(pd_socket);
+  clearSelectable();
+  pd_belong_to->removeSocket(this);
   CLOSESOCKET(pd_socket);
 }
 
@@ -375,15 +497,14 @@ void
 tcpConnection::setSelectable(int now,
 			     CORBA::Boolean data_in_buffer) {
 
-  pd_belong_to->setSelectable(pd_socket,now,data_in_buffer);
+  SocketHolder::setSelectable(now, data_in_buffer);
 }
 
-
 /////////////////////////////////////////////////////////////////////////
 void
 tcpConnection::clearSelectable() {
 
-  pd_belong_to->clearSelectable(pd_socket);
+  SocketHolder::clearSelectable();
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -392,14 +513,10 @@ tcpConnection::isSelectable() {
   return pd_belong_to->isSelectable(pd_socket);
 }
 
-
 /////////////////////////////////////////////////////////////////////////
-void
-tcpConnection::Peek(giopConnection::notifyReadable_t func, void* cookie) {
-
-  if (pd_belong_to->Peek(pd_socket)) {
-    func(cookie,this);
-  }
+CORBA::Boolean
+tcpConnection::Peek() {
+  return SocketHolder::Peek();
 }
 
 
