@@ -26,10 +26,25 @@
 //   
 //   Abstract Syntax Tree objects and support functions
 
-// $Id: idlast.cc,v 1.16.2.13 2005/05/03 10:12:40 dgrisby Exp $
+// $Id: idlast.cc,v 1.22.2.6 2006/11/02 14:00:54 dgrisby Exp $
 // $Log: idlast.cc,v $
-// Revision 1.16.2.13  2005/05/03 10:12:40  dgrisby
-// Trying to redefine built in CORBA module types led to a segfault.
+// Revision 1.22.2.6  2006/11/02 14:00:54  dgrisby
+// Remove a few warnings.
+//
+// Revision 1.22.2.5  2006/05/24 18:30:53  dgrisby
+// Segfault with some uses of aliases to unknown types.
+//
+// Revision 1.22.2.4  2005/05/10 22:07:31  dgrisby
+// Merge again.
+//
+// Revision 1.22.2.3  2004/02/16 10:10:33  dgrisby
+// More valuetype, including value boxes. C++ mapping updates.
+//
+// Revision 1.22.2.2  2003/09/04 14:00:24  dgrisby
+// ValueType IDL updates.
+//
+// Revision 1.22.2.1  2003/03/23 21:01:48  dgrisby
+// Start of omniORB 4.1.x development branch.
 //
 // Revision 1.16.2.12  2002/04/30 14:59:37  dgrisby
 // omniidl segfault when checking a non-existent identifier is not a forward.
@@ -571,7 +586,7 @@ void
 InheritSpec::
 append(InheritSpec* is, const char* file, int line)
 {
-  InheritSpec *i, *last;
+  InheritSpec *i, *last = 0;
 
   if (is->interface()) {
     for (i=this; i; i = i->next_) {
@@ -708,6 +723,20 @@ Interface::
   delete thisType_;
 }
 
+IDL_Boolean
+Interface::isDerived(const Interface* base) const
+{
+  if (base == this)
+    return 1;
+
+  for (InheritSpec* is = inherits_; is; is = is->next()) {
+    if (is->interface() == base || is->interface()->isDerived(base))
+      return 1;
+  }
+  return 0;
+}
+
+
 void
 Interface::
 finishConstruction(Decl* decls)
@@ -1189,6 +1218,9 @@ Member(const char* file, int line, IDL_Boolean mainFile,
 
   IdlType* bareType = memberType->unalias();
 
+  if (!bareType)
+    return;
+
   if (bareType->kind() == IdlType::tk_struct) {
     Struct* s = (Struct*)((DeclaredType*)bareType)->decl();
     if (!s->finished()) {
@@ -1575,6 +1607,9 @@ UnionCase(const char* file, int line, IDL_Boolean mainFile,
 
   IdlType* bareType = caseType->unalias();
 
+  if (!bareType)
+    return;
+
   if (bareType->kind() == IdlType::tk_struct) {
     Struct* s = (Struct*)((DeclaredType*)bareType)->decl();
     if (!s->finished()) {
@@ -2325,9 +2360,10 @@ closeParens()
 
 void
 Factory::
-finishConstruction(Parameter* parameters)
+finishConstruction(Parameter* parameters, RaisesSpec* raises)
 {
-  parameters_ = parameters_;
+  parameters_ = parameters;
+  raises_     = raises;
   Scope::endScope();
 }
 
@@ -2479,14 +2515,19 @@ ValueBox(const char* file, int line, IDL_Boolean mainFile,
     boxedType_(boxedType),
     constrType_(constrType)
 {
-  checkValidType(file, line, boxedType);
+  if (boxedType) {
+    checkValidType(file, line, boxedType);
 
-  IdlType* ubt = boxedType->unalias();
-  if (ubt->kind() == IdlType::tk_value ||
-      ubt->kind() == IdlType::tk_value_box) {
+    IdlType* ubt = boxedType->unalias();
+    if (ubt->kind() == IdlType::tk_value ||
+	ubt->kind() == IdlType::tk_value_box) {
 
-    IdlError(file, line, "Value types cannot be boxed");
+      IdlError(file, line, "Value types cannot be boxed");
+    }
+    delType_ = boxedType->shouldDelete();
   }
+  else
+    delType_ = 0;
 
   thisType_ = new DeclaredType(IdlType::tk_value_box, this, this);
   Scope::current()->addDecl(identifier, 0, this, thisType_, file, line);
@@ -2496,6 +2537,7 @@ ValueBox::
 ~ValueBox()
 {
   delete thisType_;
+  if (delType_) delete boxedType_;
 }
 
 ValueInheritSpec::
@@ -2576,7 +2618,7 @@ void
 ValueInheritSpec::
 append(ValueInheritSpec* is, const char* file, int line)
 {
-  ValueInheritSpec *i, *last;
+  ValueInheritSpec *i, *last = 0;
 
   for (i=this; i; i = i->next_) {
     last = i;
@@ -2639,19 +2681,36 @@ ValueAbs(const char* file, int line, IDL_Boolean mainFile,
   thisType_ = new DeclaredType(IdlType::tk_value, this, this);
 
   // Check that all inherited valuetypes are abstract
-  for (ValueInheritSpec* vinh = inherits; vinh; vinh = vinh->next()) {
-    if (vinh->value()->kind() == D_VALUE) {
-      char* ssn = vinh->scope()->scopedName()->toString();
-      IdlError(file, line,
-	       "In declaration of abstract valuetype '%s', inherited "
-	       "valuetype '%s' is not abstract", identifier, ssn);
-      IdlErrorCont(vinh->value()->file(), vinh->value()->line(),
-		   "(%s declared here)", ssn);
-      delete [] ssn;
+  if (inherits) {
+    for (ValueInheritSpec* vinh = inherits; vinh; vinh = vinh->next()) {
+      if (vinh->value()->kind() == D_VALUE) {
+	char* ssn = vinh->scope()->scopedName()->toString();
+	IdlError(file, line,
+		 "In declaration of abstract valuetype '%s', inherited "
+		 "valuetype '%s' is not abstract", identifier, ssn);
+	IdlErrorCont(vinh->value()->file(), vinh->value()->line(),
+		     "(%s declared here)", ssn);
+	delete [] ssn;
+      }
     }
+    scope_->setInherited(inherits, file, line);
+  }
+  // Supports rules
+  if (supports) {
+    for (InheritSpec* inh = supports->next(); inh; inh = inh->next()) {
+      if (!inh->interface()->abstract()) {
+	char* ssn = inh->scope()->scopedName()->toString();
+	IdlError(file, line,
+		 "In declaration of abstract valuetype '%s', supported "
+		 "interface '%s' is non-abstract but is not specified first",
+		 identifier, ssn);
+	IdlErrorCont(inh->interface()->file(), inh->interface()->line(),
+		     "(%s declared here)", ssn);
+	delete [] ssn;
+      }
+    }
+    scope_->setInherited(supports, file, line);
   }
-  scope_->setInherited(inherits, file, line);
-  scope_->setInherited(supports, file, line);
   Scope::current()->addDecl(identifier, scope_, this, thisType_, file, line);
   Scope::startScope(scope_);
   Prefix::newScope(identifier);
@@ -2773,8 +2832,111 @@ Value(const char* file, int line, IDL_Boolean mainFile,
 	delete [] ssn;
       }
     }
+    if (!supports->interface()->abstract() && inherits) {
+      // Check that supported interface does not conflict with any
+      // inherited supported interfaces.
+      Interface*   thisi = (Interface*)supports->interface();
+      InheritSpec* baseis;
+      ValueBase*   iv;
+
+      for (ValueInheritSpec* vinh = inherits; vinh; vinh = vinh->next()) {
+	iv = vinh->value();
+
+	do {
+	  ValueInheritSpec* parent;
+
+	  if (iv->kind() == D_VALUE) {
+	    baseis = ((Value*)iv)->supports();
+	    parent = ((Value*)iv)->inherits();
+	  }
+	  else {
+	    baseis = ((ValueAbs*)iv)->supports();
+	    parent = ((ValueAbs*)iv)->inherits();
+	  }
+	  if (!baseis || baseis->interface()->abstract()) {
+	    if (parent)
+	      iv = parent->value();
+	    else
+	      break;
+	  }
+	} while (!baseis && iv);
+
+	if (baseis &&
+	    ! baseis->interface()->abstract() &&
+	    ! thisi->isDerived(baseis->interface())) {
+
+	  char* tsssn = supports->scope()->scopedName()->toString();
+	  char* isssn = baseis->scope()->scopedName()->toString();
+	  char* ivssn = vinh->scope()->scopedName()->toString();
+	  const char* indirectly;
+	  if (iv == vinh->value())
+	    indirectly = "";
+	  else
+	    indirectly = "indirectly ";
+
+	  IdlError(file, line,
+		   "In declaration of valuetype '%s', supported "
+		   "interface '%s' is not derived from interface '%s' %s"
+		   "supported by inherited valuetype '%s'",
+		   identifier, tsssn, isssn, indirectly, ivssn);
+	  IdlErrorCont(vinh->value()->file(), vinh->value()->line(),
+		       "(%s declared here)", ivssn);
+	  delete [] tsssn;
+	  delete [] isssn;
+	  delete [] ivssn;
+	}
+      }
+    }
     scope_->setInherited(supports, file, line);
   }
+  else {
+    // Check that inherited values do not have conflicting supports
+    InheritSpec* baseis;
+    ValueBase*   iv;
+    Interface*   ii = 0;
+
+    for (ValueInheritSpec* vinh = inherits; vinh; vinh = vinh->next()) {
+      iv = vinh->value();
+
+      do {
+	ValueInheritSpec* parent;
+
+	if (iv->kind() == D_VALUE) {
+	  baseis = ((Value*)iv)->supports();
+	  parent = ((Value*)iv)->inherits();
+	}
+	else {
+	  baseis = ((ValueAbs*)iv)->supports();
+	  parent = ((ValueAbs*)iv)->inherits();
+	}
+	if (!baseis || baseis->interface()->abstract()) {
+	  if (parent)
+	    iv = parent->value();
+	  else
+	    break;
+	}
+      } while (!baseis && iv);
+
+      if (baseis && ! baseis->interface()->abstract()) {
+	if (ii) {
+	  if (baseis->interface() != ii) {
+	    char* ssn1 = ii->scope()->scopedName()->toString();
+	    char* ssn2 = baseis->scope()->scopedName()->toString();
+	    IdlError(file, line,
+		     "In declaration of valuetype '%s', supported "
+		     "interfaces '%s' and '%s' clash",
+		     identifier, ssn1, ssn2);
+	    delete [] ssn1;
+	    delete [] ssn2;
+	  }
+	}
+	else {
+	  ii = baseis->interface();
+	}
+      }
+    }
+  }
+
   Scope::current()->addDecl(identifier, scope_, this, thisType_, file, line);
   Scope::startScope(scope_);
   Prefix::newScope(identifier);
